@@ -1,7 +1,7 @@
 #!/bin/sh
 
-SWNAME=ImageMagick-6.0.6
-SWREV=2
+SWNAME=ImageMagick-6.1.7
+SWREV=5
 MANDIR=/usr/share/man
 DOCDIR=/usr/share/doc
 ARCH=i386
@@ -11,7 +11,7 @@ PKG=${SWNAME}-${ARCH}-${BUILD}.tgz
 rm -rf $SWNAME
 tar zxf ${SWNAME}-${SWREV}.tar.gz
 cd $SWNAME
-patch -p1 < ../ImageMagick-6.0.6-ascii85encode.patch
+patch -p1 < ../ImageMagick-6.1.7-ascii85encode+bugfix.patch
 ln -s /usr/X11R6/include/freetype2/freetype .
 patch <<EOF
 --- configure._orig_	2004-05-24 00:15:22.000000000 +0900
