@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,9 +41,12 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/client.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -79,10 +82,6 @@
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadDPSImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a memory shortage or
-%      if the image cannot be read.
-%
 %    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
@@ -101,7 +100,8 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *image;
 
   int
-    sans;
+    sans,
+    status;
 
   long
     x,
@@ -125,9 +125,6 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   Screen
     *screen;
 
-  unsigned int
-    status;
-
   XColor
     *colors;
 
@@ -155,7 +152,7 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
-  if (image_info->debug != False)
+  if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
@@ -163,7 +160,7 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (display == (Display *) NULL)
     return((Image *) NULL);
   /*
-    Set our forgiving error handler.
+    Set our forgiving exception handler.
   */
   (void) XSetErrorHandler(XError);
   /*
@@ -171,7 +168,7 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
+  if (status == MagickFalse)
     return((Image *) NULL);
   /*
     Get user defaults from X resource database.
@@ -197,7 +194,7 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if ((map_info == (XStandardColormap *) NULL) ||
       (visual_info == (XVisualInfo *) NULL))
     {
-      DestroyImage(image);
+      image=DestroyImage(image);
       XFreeResources(display,visual_info,map_info,(XPixelInfo *) NULL,
         (XFontStruct *) NULL,&resource_info,(XWindowInfo *) NULL);
       return((Image *) NULL);
@@ -214,7 +211,7 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     &bits_per_pixel,&page);
   if ((status == dps_status_failure) || (status == dps_status_no_extension))
     {
-      DestroyImage(image);
+      image=DestroyImage(image);
       XFreeResources(display,visual_info,map_info,(XPixelInfo *) NULL,
         (XFontStruct *) NULL,&resource_info,(XWindowInfo *) NULL);
       return((Image *) NULL);
@@ -224,10 +221,10 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   status=XDPSImageFileIntoDrawable((DPSContext) NULL,screen,pixmap,
     GetBlobFileHandle(image),bits_per_pixel.height,visual_info->depth,&page,
-    -page.x,-page.y,pixels_per_point,True,False,True,&sans);
+    -page.x,-page.y,pixels_per_point,MagickTrue,MagickFalse,MagickTrue,&sans);
   if (status != dps_status_success)
     {
-      DestroyImage(image);
+      image=DestroyImage(image);
       XFreeResources(display,visual_info,map_info,(XPixelInfo *) NULL,
         (XFontStruct *) NULL,&resource_info,(XWindowInfo *) NULL);
       return((Image *) NULL);
@@ -240,7 +237,7 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   (void) XFreePixmap(display,pixmap);
   if (dps_image == (XImage *) NULL)
     {
-      DestroyImage(image);
+      image=DestroyImage(image);
       XFreeResources(display,visual_info,map_info,(XPixelInfo *) NULL,
         (XFontStruct *) NULL,&resource_info,(XWindowInfo *) NULL);
       return((Image *) NULL);
@@ -252,7 +249,7 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     AcquireMagickMemory(visual_info->colormap_size*sizeof(XColor));
   if (colors == (XColor *) NULL)
     {
-      DestroyImage(image);
+      image=DestroyImage(image);
       XDestroyImage(dps_image);
       XFreeResources(display,visual_info,map_info,(XPixelInfo *) NULL,
         (XFontStruct *) NULL,&resource_info,(XWindowInfo *) NULL);
@@ -309,7 +306,7 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     image->storage_class=PseudoClass;
   image->columns=dps_image->width;
   image->rows=dps_image->height;
-  if (image_info->ping != False)
+  if (image_info->ping != MagickFalse)
     {
       CloseBlob(image);
       return(GetFirstImageInList(image));
@@ -376,11 +373,16 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
             q->blue=ScaleShortToQuantum(colors[index].blue);
             q++;
           }
-          if (SyncImagePixels(image) == False)
+          if (SyncImagePixels(image) == MagickFalse)
             break;
-          if (QuantumTick(y,image->rows))
-            if (MagickMonitor(LoadImageTag,y,image->rows,exception) == False)
-              break;
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(y,image->rows) != MagickFalse))
+            {
+              status=image->progress_monitor(LoadImageTag,y,image->rows,
+                image->client_data);
+              if (status == MagickFalse)
+                break;
+            }
         }
       else
         for (y=0; y < (long) image->rows; y++)
@@ -399,11 +401,16 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
             q->blue=ScaleShortToQuantum((color*65535L)/blue_mask);
             q++;
           }
-          if (SyncImagePixels(image) == False)
+          if (SyncImagePixels(image) == MagickFalse)
             break;
-          if (QuantumTick(y,image->rows))
-            if (MagickMonitor(LoadImageTag,y,image->rows,exception) == False)
-              break;
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(y,image->rows) != MagickFalse))
+            {
+              status=image->progress_monitor(LoadImageTag,y,image->rows,
+                image->client_data);
+              if (status == MagickFalse)
+                break;
+            }
         }
       break;
     }
@@ -412,9 +419,9 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       /*
         Create colormap.
       */
-      if (AllocateImageColormap(image,visual_info->colormap_size) == False)
+      if (AllocateImageColormap(image,visual_info->colormap_size) == MagickFalse)
         {
-          DestroyImage(image);
+          image=DestroyImage(image);
           colors=(XColor *) RelinquishMagickMemory(colors);
           XDestroyImage(dps_image);
           XFreeResources(display,visual_info,map_info,(XPixelInfo *) NULL,
@@ -440,11 +447,16 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
         indexes=GetIndexes(image);
         for (x=0; x < (long) image->columns; x++)
           indexes[x]=(unsigned short) XGetPixel(dps_image,x,y);
-        if (SyncImagePixels(image) == False)
+        if (SyncImagePixels(image) == MagickFalse)
           break;
-        if (QuantumTick(y,image->rows))
-          if (MagickMonitor(LoadImageTag,y,image->rows,exception) == False)
-            break;
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
+          {
+            status=image->progress_monitor(LoadImageTag,y,image->rows,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
       break;
     }
@@ -462,7 +474,7 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     {
       status=XDPSImageFileIntoDrawable((DPSContext) NULL,screen,pixmap,
         GetBlobFileHandle(image),bits_per_pixel.height,1,&page,-page.x,
-        -page.y,pixels_per_point,True,True,True,&sans);
+        -page.y,pixels_per_point,MagickTrue,MagickTrue,MagickTrue,&sans);
       if (status == dps_status_success)
         {
           XImage
@@ -477,7 +489,7 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (matte_image != (XImage *) NULL)
             {
               image->storage_class=DirectClass;
-              image->matte=True;
+              image->matte=MagickTrue;
               for (y=0; y < (long) image->rows; y++)
               {
                 q=SetImagePixels(image,0,y,image->columns,1);
@@ -490,7 +502,7 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     q->opacity=TransparentOpacity;
                   q++;
                 }
-                if (SyncImagePixels(image) == False)
+                if (SyncImagePixels(image) == MagickFalse)
                   break;
               }
               XDestroyImage(matte_image);
@@ -539,7 +551,7 @@ ModuleExport void RegisterDPSImage(void)
 #if defined(HasDPS)
   entry->decoder=(DecoderHandler *) ReadDPSImage;
 #endif
-  entry->blob_support=False;
+  entry->blob_support=MagickFalse;
   entry->description=AcquireString("Display Postscript Interpreter");
   entry->module=AcquireString("DPS");
   (void) RegisterMagickInfo(entry);
