@@ -16,7 +16,7 @@
 %                              Bill Radcliffe                                 %
 %                                   2001                                      %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -48,9 +48,12 @@
 #endif
 
 #include "magick/blob.h"
-#include "magick/error.h"
+#include "magick/blob_private.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/geometry.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -67,17 +70,15 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  IsEMF() returns True if the image format type, identified by the
+%  IsEMF() returns MagickTrue if the image format type, identified by the
 %  magick string, is a Microsoft Windows Enhanced MetaFile (EMF) file.
 %
 %  The format of the ReadEMFImage method is:
 %
-%      unsigned int IsEMF(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsEMF(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsEMF() returns True if the image format type is EMF.
-%
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
 %
@@ -85,13 +86,13 @@
 %
 %
 */
-static unsigned int IsEMF(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsEMF(const unsigned char *magick,const size_t length)
 {
   if (length < 48)
-    return(False);
+    return(MagickFalse);
   if (memcmp(magick+40,"\040\105\115\106\000\000\001\000",8) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -105,17 +106,15 @@ static unsigned int IsEMF(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  IsWMF() returns True if the image format type, identified by the
+%  IsWMF() returns MagickTrue if the image format type, identified by the
 %  magick string, is a Windows MetaFile (WMF) file.
 %
 %  The format of the ReadEMFImage method is:
 %
-%      unsigned int IsEMF(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsEMF(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsEMF() returns True if the image format type is WMF.
-%
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
 %
@@ -123,15 +122,15 @@ static unsigned int IsEMF(const unsigned char *magick,const size_t length)
 %
 %
 */
-static unsigned int IsWMF(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsWMF(const unsigned char *magick,const size_t length)
 {
   if (length < 4)
-    return(False);
+    return(MagickFalse);
   if (memcmp(magick,"\327\315\306\232",4) == 0)
-    return(True);
+    return(MagickTrue);
   if (memcmp(magick,"\001\000\011\000",4) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -157,10 +156,6 @@ static unsigned int IsWMF(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadEMFImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a a memory shortage or
-%      if the image cannot be read.
-%
 %    o image_info: The image info..
 %
 %    o exception: Return any errors or warnings in this structure.
@@ -390,7 +385,7 @@ static Image *ReadEMFImage(const ImageInfo *image_info,
       register char
         *p;
 
-      unsigned int
+      MagickStatusType
         flags;
 
       geometry=GetPageGeometry(image_info->page);
@@ -400,10 +395,10 @@ static Image *ReadEMFImage(const ImageInfo *image_info,
           flags=ParseMetaGeometry(geometry,&sans,&sans,&image->columns,
             &image->rows);
           if (image->x_resolution != 0.0)
-            image->columns=(unsigned int)
+            image->columns=(unsigned long)
               ((image->columns*image->x_resolution)+0.5);
           if (image->y_resolution != 0.0)
-            image->rows=(unsigned int)
+            image->rows=(unsigned long)
               ((image->rows*image->y_resolution)+0.5);
         }
       else
@@ -412,10 +407,10 @@ static Image *ReadEMFImage(const ImageInfo *image_info,
           flags=ParseMetaGeometry(geometry,&sans,&sans,&image->columns,
             &image->rows);
           if (image->x_resolution != 0.0)
-            image->columns=(unsigned int)
+            image->columns=(unsigned long)
               (((image->columns*image->x_resolution)/72.0)+0.5);
           if (image->y_resolution != 0.0)
-            image->rows=(unsigned int)
+            image->rows=(unsigned long)
               (((image->rows*image->y_resolution)/72.0)+0.5);
         }
       geometry=(char *) RelinquishMagickMemory(geometry);
@@ -488,7 +483,7 @@ static Image *ReadEMFImage(const ImageInfo *image_info,
       pBits++;
       q++;
     }
-    if (SyncImagePixels(image) == False)
+    if (SyncImagePixels(image) == MagickFalse)
       break;
   }
   if (hemf)
@@ -535,7 +530,7 @@ ModuleExport void RegisterEMFImage(void)
   entry->description=
     AcquireString("Windows WIN32 API rendered Enhanced Meta File");
   entry->magick=(MagickHandler *) IsEMF;
-  entry->blob_support=False;
+  entry->blob_support=MagickFalse;
   entry->module=AcquireString("WMF");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("WMFWIN32");
@@ -544,7 +539,7 @@ ModuleExport void RegisterEMFImage(void)
 #endif
   entry->description=AcquireString("Windows WIN32 API rendered Meta File");
   entry->magick=(MagickHandler *) IsWMF;
-  entry->blob_support=False;
+  entry->blob_support=MagickFalse;
   entry->module=AcquireString("WMFWIN32");
   (void) RegisterMagickInfo(entry);
 }
