@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,12 +41,16 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
+#include "magick/color.h"
 #include "magick/constitute.h"
 #include "magick/draw.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/delegate.h"
 #include "magick/geometry.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -82,7 +86,7 @@ typedef struct _EPTInfo
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WriteEPTImage(const ImageInfo *,Image *);
 
 /*
@@ -96,17 +100,15 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  IsEPT() returns True if the image format type, identified by the
+%  IsEPT() returns MagickTrue if the image format type, identified by the
 %  magick string, is EPT.
 %
 %  The format of the IsEPT method is:
 %
-%      unsigned int IsEPT(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsEPT(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsEPT() returns True if the image format type is EPT.
-%
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
 %
@@ -114,13 +116,13 @@ static unsigned int
 %
 %
 */
-static unsigned int IsEPT(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsEPT(const unsigned char *magick,const size_t length)
 {
   if (length < 4)
-    return(False);
+    return(MagickFalse);
   if (memcmp(magick,"\305\320\323\306",4) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -145,10 +147,6 @@ static unsigned int IsEPT(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadEPTImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a memory shortage or
-%      if the image cannot be read.
-%
 %    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
@@ -163,7 +161,10 @@ static Image *ReadEPTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   Image
     *image;
 
-  unsigned int
+  ImageInfo
+    *read_info;
+
+  MagickBooleanType
     status;
 
   /*
@@ -171,15 +172,15 @@ static Image *ReadEPTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
-  if (image_info->debug != False)
+  if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
+  if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   ept_info.magick=ReadBlobLSBLong(image);
@@ -203,14 +204,22 @@ static Image *ReadEPTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   (void) SeekBlob(image,ept_info.postscript_offset,SEEK_SET);
   (void) ReadBlob(image,ept_info.postscript_length,ept_info.postscript);
   CloseBlob(image);
-  DestroyImage(image);
-  image=BlobToImage(image_info,ept_info.postscript,ept_info.postscript_length,
+  image=DestroyImage(image);
+  read_info=CloneImageInfo(image_info);
+  (void) CopyMagickString(read_info->magick,"EPS",MaxTextExtent);
+  image=BlobToImage(read_info,ept_info.postscript,ept_info.postscript_length,
     exception);
   if (image == (Image *) NULL)
-    image=BlobToImage(image_info,ept_info.tiff,ept_info.tiff_length,
-      exception);
-  RelinquishMagickMemory(ept_info.tiff);
-  RelinquishMagickMemory(ept_info.postscript);
+    {
+      (void) CopyMagickString(read_info->magick,"TIFF",MaxTextExtent);
+      image=BlobToImage(read_info,ept_info.tiff,ept_info.tiff_length,exception);
+    }
+  read_info=DestroyImageInfo(read_info);
+  if (image != (Image *) NULL)
+    (void) CopyMagickString(image->filename,image_info->filename,MaxTextExtent);
+  ept_info.tiff=(unsigned char *) RelinquishMagickMemory(ept_info.tiff);
+  ept_info.postscript=(unsigned char *)
+    RelinquishMagickMemory(ept_info.postscript);
   return(image);
 }
 
@@ -246,8 +255,8 @@ ModuleExport void RegisterEPTImage(void)
   entry->decoder=(DecoderHandler *) ReadEPTImage;
   entry->encoder=(EncoderHandler *) WriteEPTImage;
   entry->magick=(MagickHandler *) IsEPT;
-  entry->adjoin=False;
-  entry->blob_support=False;
+  entry->adjoin=MagickFalse;
+  entry->blob_support=MagickFalse;
   entry->description=AcquireString("Encapsulated PostScript with TIFF preview");
   entry->module=AcquireString("EPT");
   (void) RegisterMagickInfo(entry);
@@ -255,8 +264,8 @@ ModuleExport void RegisterEPTImage(void)
   entry->decoder=(DecoderHandler *) ReadEPTImage;
   entry->encoder=(EncoderHandler *) WriteEPTImage;
   entry->magick=(MagickHandler *) IsEPT;
-  entry->adjoin=False;
-  entry->blob_support=False;
+  entry->adjoin=MagickFalse;
+  entry->blob_support=MagickFalse;
   entry->description=
     AcquireString("Encapsulated PostScript Level II with TIFF preview");
   entry->module=AcquireString("EPT");
@@ -265,7 +274,7 @@ ModuleExport void RegisterEPTImage(void)
   entry->decoder=(DecoderHandler *) ReadEPTImage;
   entry->encoder=(EncoderHandler *) WriteEPTImage;
   entry->magick=(MagickHandler *) IsEPT;
-  entry->blob_support=False;
+  entry->blob_support=MagickFalse;
   entry->description=
     AcquireString("Encapsulated PostScript Level III with TIFF preview");
   entry->module=AcquireString("EPT");
@@ -312,7 +321,7 @@ ModuleExport void UnregisterEPTImage(void)
 %
 %  The format of the WriteEPTImage method is:
 %
-%      unsigned int WriteEPTImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteEPTImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
@@ -322,7 +331,7 @@ ModuleExport void UnregisterEPTImage(void)
 %
 %
 */
-static unsigned int WriteEPTImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteEPTImage(const ImageInfo *image_info,Image *image)
 {
   EPTInfo
     ept_info;
@@ -333,7 +342,7 @@ static unsigned int WriteEPTImage(const ImageInfo *image_info,Image *image)
   ImageInfo
     *write_info;
 
-  unsigned int
+  MagickBooleanType
     status;
 
   /*
@@ -343,35 +352,35 @@ static unsigned int WriteEPTImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  if (image->debug != False)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
+  if (status == MagickFalse)
     return(status);
-  write_image=CloneImage(image,0,0,True,&image->exception);
+  write_image=CloneImage(image,0,0,MagickTrue,&image->exception);
   if (write_image == (Image *) NULL)
-    return(False);
+    return(MagickFalse);
   write_info=CloneImageInfo(image_info);
   DestroyBlob(write_image);
   write_image->blob=CloneBlobInfo((BlobInfo *) NULL);
-  (void) strcpy(write_image->magick,"EPS");
+  (void) CopyMagickString(write_image->magick,"EPS",MaxTextExtent);
   if (LocaleCompare(image_info->magick,"EPT2") == 0)
-    (void) strcpy(write_image->magick,"EPS2");
+    (void) CopyMagickString(write_image->magick,"EPS2",MaxTextExtent);
   if (LocaleCompare(image_info->magick,"EPT3") == 0)
-    (void) strcpy(write_image->magick,"EPS3");
+    (void) CopyMagickString(write_image->magick,"EPS3",MaxTextExtent);
   (void) ResetMagickMemory(&ept_info,0,sizeof(ept_info));
   ept_info.magick=0xc6d3d0c5ul;
   ept_info.postscript=(unsigned char *) ImageToBlob(write_info,write_image,
     &ept_info.postscript_length,&image->exception);
   if (ept_info.postscript == (void *) NULL)
     {
-      DestroyImage(write_image);
-      DestroyImageInfo(write_info);
-      return(False);
+      write_image=DestroyImage(write_image);
+      write_info=DestroyImageInfo(write_info);
+      return(MagickFalse);
     }
   DestroyBlob(write_image);
   write_image->blob=CloneBlobInfo((BlobInfo *) NULL);
-  (void) strcpy(write_image->magick,"TIFF");
+  (void) CopyMagickString(write_image->magick,"TIFF",MaxTextExtent);
   (void) TransformImage(&write_image,(char *) NULL,"512x512>");
   if ((write_image->storage_class == DirectClass) ||
       (write_image->colors > 256))
@@ -383,34 +392,34 @@ static unsigned int WriteEPTImage(const ImageInfo *image_info,Image *image)
         EPT preview requires that the image is colormapped.
       */
       GetQuantizeInfo(&quantize_info);
-      quantize_info.dither=image_info->dither;
-      quantize_info.number_colors=256;
+      quantize_info.dither=IsPaletteImage(write_image,&image->exception) ==
+        MagickFalse ? MagickTrue : MagickFalse;
       (void) QuantizeImage(&quantize_info,write_image);
     }
   write_image->compression=NoCompression;
   ept_info.tiff=(unsigned char *) ImageToBlob(write_info,write_image,
     &ept_info.tiff_length,&image->exception);
-  DestroyImage(write_image);
-  DestroyImageInfo(write_info);
+  write_image=DestroyImage(write_image);
+  write_info=DestroyImageInfo(write_info);
   if (ept_info.tiff == (void *) NULL)
     {
       ept_info.postscript=(unsigned char *)
         RelinquishMagickMemory(ept_info.postscript);
-      return(False);
+      return(MagickFalse);
     }
   /*
     Write EPT image.
   */
   (void) WriteBlobLSBLong(image,ept_info.magick);
-  (void) WriteBlobLSBLong(image,ept_info.tiff_length+30);
+  (void) WriteBlobLSBLong(image,30);
   (void) WriteBlobLSBLong(image,ept_info.postscript_length);
   (void) WriteBlobLSBLong(image,0);
   (void) WriteBlobLSBLong(image,0);
-  (void) WriteBlobLSBLong(image,30);
+  (void) WriteBlobLSBLong(image,ept_info.postscript_length+30);
   (void) WriteBlobLSBLong(image,ept_info.tiff_length);
   (void) WriteBlobLSBShort(image,0xffff);
-  (void) WriteBlob(image,ept_info.tiff_length,ept_info.tiff);
   (void) WriteBlob(image,ept_info.postscript_length,ept_info.postscript);
+  (void) WriteBlob(image,ept_info.tiff_length,ept_info.tiff);
   /*
     Relinquish resources.
   */
@@ -418,5 +427,5 @@ static unsigned int WriteEPTImage(const ImageInfo *image_info,Image *image)
     RelinquishMagickMemory(ept_info.postscript);
   ept_info.tiff=(unsigned char *) RelinquishMagickMemory(ept_info.tiff);
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
