@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,11 +42,14 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/color_private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -57,7 +60,7 @@
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WriteFITSImage(const ImageInfo *,Image *);
 
 /*
@@ -71,17 +74,15 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  IsFITS() returns True if the image format type, identified by the
+%  IsFITS() returns MagickTrue if the image format type, identified by the
 %  magick string, is FITS.
 %
 %  The format of the IsFITS method is:
 %
-%      unsigned int IsFITS(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsFITS(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsFITS() returns True if the image format type is FITS.
-%
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
 %
@@ -89,15 +90,15 @@ static unsigned int
 %
 %
 */
-static unsigned int IsFITS(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsFITS(const unsigned char *magick,const size_t length)
 {
   if (length < 6)
-    return(False);
+    return(MagickFalse);
   if (LocaleNCompare((char *) magick,"IT0",3) == 0)
-    return(True);
+    return(MagickTrue);
   if (LocaleNCompare((char *) magick,"SIMPLE",6) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -183,6 +184,10 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
     scene,
     y;
 
+  MagickBooleanType
+    status,
+    value_expected;
+
   MagickSizeType
     number_pixels;
 
@@ -211,33 +216,31 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
     *fits_pixels,
     long_quantum[8];
 
-  unsigned int
-    status,
-    value_expected;
   /*
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
-  if (image_info->debug != False)
+  if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
+  if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
     Initialize image header.
   */
-  fits_info.simple=False;
+  fits_info.simple=MagickFalse;
   fits_info.bits_per_pixel=8;
   fits_info.columns=1;
   fits_info.rows=1;
   fits_info.number_scenes=1;
+  fits_info.number_axes=1;
   fits_info.min_data=0.0;
   fits_info.max_data=0.0;
   fits_info.zero=0.0;
@@ -248,7 +251,7 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
   c=ReadBlobByte(image);
   if (c == EOF)
     {
-      DestroyImage(image);
+      image=DestroyImage(image);
       return((Image *) NULL);
     }
   for ( ; ; )
@@ -269,20 +272,19 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
           if ((size_t) (p-keyword) < (MaxTextExtent-1))
             *p++=(char) c;
           c=ReadBlobByte(image);
-        } while ((isalnum((int) ((unsigned char) c)) != False) ||
+        } while ((isalnum((int) ((unsigned char) c)) != MagickFalse) ||
                  ((int) c == '_'));
         *p='\0';
         if (LocaleCompare(keyword,"END") == 0)
           break;
-        value_expected=False;
-        while ((isspace((int) ((unsigned char) c)) != False) ||
-               ((int) c == '='))
+        value_expected=MagickFalse;
+        while ((isspace((int) ((unsigned char) c)) != 0) || ((int) c == '='))
         {
           if ((int) c == '=')
-            value_expected=True;
+            value_expected=MagickTrue;
           c=ReadBlobByte(image);
         }
-        if (value_expected == False)
+        if (value_expected == MagickFalse)
           continue;
         p=value;
         while (isalnum(c) || (c == '-') || (c == '+') || (c == '.'))
@@ -353,12 +355,12 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
     */
     image->columns=(unsigned long) fits_info.columns;
     image->rows=(unsigned long) fits_info.rows;
-    image->depth=fits_info.bits_per_pixel <= 8 ? 8UL : QuantumDepth;
+    image->depth=fits_info.bits_per_pixel <= 8 ? 8UL : Min(QuantumDepth,16);
     image->storage_class=PseudoClass;
     image->scene=(unsigned long) scene;
-    if (AllocateImageColormap(image,1 << image->depth) == False)
+    if (AllocateImageColormap(image,1UL << image->depth) == MagickFalse)
       ThrowReaderException(ResourceLimitError,"UnableToAllocateColormap");
-    if ((image_info->ping != False) && (image_info->number_scenes != 0))
+    if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     /*
@@ -381,7 +383,10 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
     count=ReadBlob(image,packet_size*image->columns*image->rows,fits_pixels);
     if (count == 0)
       ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
-    if ((fits_info.min_data == 0.0) || (fits_info.max_data == 0.0))
+    if ((fits_info.bits_per_pixel > 0) &&
+        (fits_info.min_data == 0.0) && (fits_info.max_data == 0.0))
+      fits_info.max_data=(1UL << fits_info.bits_per_pixel)-1;
+    else
       {
         /*
           Determine minimum and maximum intensity.
@@ -484,7 +489,7 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
     scale=1.0;
     if ((fits_info.bits_per_pixel < 0) || ((fits_info.max_data-
         fits_info.min_data) > (double) ((1UL << image->depth)-1)))
-      scale=(double) ((1 << image->depth)-1)/
+      scale=(double) ((1UL << image->depth)-1)/
         (fits_info.max_data-fits_info.min_data);
     p=fits_pixels;
     for (y=(long) image->rows-1; y >= 0; y--)
@@ -545,14 +550,19 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
         indexes[x]=index;
         *q++=image->colormap[index];
       }
-      if (SyncImagePixels(image) == False)
+      if (SyncImagePixels(image) == MagickFalse)
         break;
-      if (QuantumTick(y,image->rows))
-        if (MagickMonitor(LoadImageTag,y,image->rows,exception) == False)
-          break;
+      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+          (QuantumTick(y,image->rows) != MagickFalse))
+        {
+          status=image->progress_monitor(LoadImageTag,y,image->rows,
+            image->client_data);
+          if (status == MagickFalse)
+            break;
+        }
     }
     fits_pixels=(unsigned char *) RelinquishMagickMemory(fits_pixels);
-    if (EOFBlob(image) != False)
+    if (EOFBlob(image) != MagickFalse)
       {
         ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
           image->filename);
@@ -572,14 +582,17 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
         AllocateNextImage(image_info,image);
         if (image->next == (Image *) NULL)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
-          exception);
-        if (status == False)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
+              GetBlobSize(image),image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
   }
   CloseBlob(image);
@@ -618,7 +631,7 @@ ModuleExport void RegisterFITSImage(void)
   entry->decoder=(DecoderHandler *) ReadFITSImage;
   entry->encoder=(EncoderHandler *) WriteFITSImage;
   entry->magick=(MagickHandler *) IsFITS;
-  entry->adjoin=False;
+  entry->adjoin=MagickFalse;
   entry->description=AcquireString("Flexible Image Transport System");
   entry->module=AcquireString("FITS");
   (void) RegisterMagickInfo(entry);
@@ -664,21 +677,17 @@ ModuleExport void UnregisterFITSImage(void)
 %
 %  The format of the WriteFITSImage method is:
 %
-%      unsigned int WriteFITSImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteFITSImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteFITSImage return True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
-%
 %    o image_info: The image info.
 %
 %    o image:  The image.
 %
 %
 */
-static unsigned int WriteFITSImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteFITSImage(const ImageInfo *image_info,Image *image)
 {
   char
     buffer[81],
@@ -687,6 +696,12 @@ static unsigned int WriteFITSImage(const ImageInfo *image_info,Image *image)
   long
     y;
 
+  MagickBooleanType
+    status;
+
+  MagickSizeType
+    length;
+
   register const PixelPacket
     *p;
 
@@ -699,9 +714,6 @@ static unsigned int WriteFITSImage(const ImageInfo *image_info,Image *image)
   unsigned char
     *pixels;
 
-  unsigned int
-    status;
-
   /*
     Open output image file.
   */
@@ -709,17 +721,17 @@ static unsigned int WriteFITSImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  if (image->debug != False)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
+  if (status == MagickFalse)
     return(status);
   (void) SetImageColorspace(image,RGBColorspace);
   /*
     Allocate image memory.
   */
   packet_size=(size_t) (image->depth > 8 ? 2 : 1);
-  fits_info=(char *) AcquireMagickMemory(2880);
+  fits_info=(char *) AcquireMagickMemory(2880*sizeof(*fits_info));
   pixels=(unsigned char *) AcquireMagickMemory(packet_size*image->columns);
   if ((fits_info == (char *) NULL) || (pixels == (unsigned char *) NULL))
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
@@ -751,7 +763,6 @@ static unsigned int WriteFITSImage(const ImageInfo *image_info,Image *image)
   (void) strcpy(buffer,"END");
   (void) strncpy(fits_info+640,buffer,strlen(buffer));
   (void) WriteBlob(image,2880,(unsigned char *) fits_info);
-  fits_info=(char *) RelinquishMagickMemory(fits_info);
   /*
     Convert image to fits scale PseudoColor class.
   */
@@ -762,15 +773,20 @@ static unsigned int WriteFITSImage(const ImageInfo *image_info,Image *image)
       break;
     (void) PopImagePixels(image,GrayQuantum,pixels);
     (void) WriteBlob(image,packet_size*image->columns,pixels);
-    if (QuantumTick(image->rows-y-1,image->rows))
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
       {
-        status=MagickMonitor(SaveImageTag,(MagickOffsetType) image->rows-y-1,
-          image->rows,&image->exception);
-        if (status == False)
+        status=image->progress_monitor(SaveImageTag,image->rows-y-1,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
           break;
       }
   }
+  (void) ResetMagickMemory(fits_info,0,2880*sizeof(*fits_info));
+  length=packet_size*image->columns*image->rows;
+  (void) WriteBlob(image,2880-(length % 2880),(unsigned char *) fits_info);
+  fits_info=(char *) RelinquishMagickMemory(fits_info);
   pixels=(unsigned char *) RelinquishMagickMemory(pixels);
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
