@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,13 +42,16 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/color.h"
 #include "magick/color_private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/geometry.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -67,7 +70,7 @@
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WriteFPXImage(const ImageInfo *,Image *);
 #endif
 
@@ -82,17 +85,15 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  IsFPX() returns True if the image format type, identified by the
+%  IsFPX() returns MagickTrue if the image format type, identified by the
 %  magick string, is FPX.
 %
 %  The format of the IsFPX method is:
 %
-%      unsigned int IsFPX(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsFPX(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsFPX() returns True if the image format type is FPX.
-%
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
 %
@@ -100,13 +101,13 @@ static unsigned int
 %
 %
 */
-static unsigned int IsFPX(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsFPX(const unsigned char *magick,const size_t length)
 {
   if (length < 4)
-    return(False);
+    return(MagickFalse);
   if (memcmp(magick,"\320\317\021\340",4) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 #if defined(HasFPX)
@@ -131,10 +132,6 @@ static unsigned int IsFPX(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadFPXImage returns a pointer to the image after
-%      reading. A null image is returned if there is a memory shortage or if
-%      the image cannot be read.
-%
 %    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
@@ -173,6 +170,9 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   long
     y;
 
+  MagickBooleanType
+    status;
+
   register IndexPacket
     *indexes;
 
@@ -192,13 +192,10 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   unsigned char
     *scanline;
 
-  unsigned int
-    status,
-    scene;
-
   unsigned long
     height,
     memory_limit,
+    scene,
     tile_width,
     tile_height,
     width;
@@ -208,15 +205,15 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
-  if (image_info->debug != False)
+  if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
+  if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   CloseBlob(image);
@@ -258,8 +255,8 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       FPX_ClearSystem();
       ThrowFileException(exception,FileOpenError,"UnableToOpenFile",
         image->filename);
-      DestroyImageList(image);
-      return(False);
+      image=DestroyImageList(image);
+      return((Image *) NULL);
     }
   if (colorspace.numberOfComponents == 0)
     {
@@ -297,15 +294,15 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Note image label.
         */
-        label=(char *) AcquireMagickMemory(summary_info.title.length+1);
+        label=(char *)
+          AcquireMagickMemory(summary_info.title.length+MaxTextExtent);
         if (label == (char *) NULL)
           {
             FPX_ClearSystem();
             ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
           }
-        (void) strncpy(label,(char *) summary_info.title.ptr,
-          summary_info.title.length);
-        label[summary_info.title.length]='\0';
+        (void) CopyMagickString(label,(char *) summary_info.title.ptr,
+          summary_info.title.length+1);
         (void) SetImageAttribute(image,"label",label);
         label=(char *) RelinquishMagickMemory(label);
       }
@@ -319,16 +316,16 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Note image comment.
         */
-        comments=(char *) AcquireMagickMemory(summary_info.comments.length+1);
+        comments=(char *)
+          AcquireMagickMemory(summary_info.comments.length+MaxTextExtent);
         if (comments == (char *) NULL)
           {
             FPX_ClearSystem();
             ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
           }
-        (void) strncpy(comments,(char *) summary_info.comments.ptr,
-          summary_info.comments.length);
-        comments[summary_info.comments.length]='\0';
-        (void) SetImageAttribute(image,"comment",comments);
+        (void) CopyMagickString(comments,(char *) summary_info.comments.ptr,
+          summary_info.comments.length+1);
+        (void) SetImageAttribute(image,"Comment",comments);
         comments=(char *) RelinquishMagickMemory(comments);
       }
   /*
@@ -356,19 +353,19 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   image->columns=width;
   image->rows=height;
   if ((colorspace.numberOfComponents % 2) == 0)
-    image->matte=True;
+    image->matte=MagickTrue;
   if (colorspace.numberOfComponents == 1)
     {
       /*
         Create linear colormap.
       */
-      if (AllocateImageColormap(image,MaxColormapSize) == False)
+      if (AllocateImageColormap(image,MaxColormapSize) == MagickFalse)
         {
           FPX_ClearSystem();
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         }
     }
-  if (image_info->ping != False)
+  if (image_info->ping != MagickFalse)
     {
       (void) FPX_CloseImage(flashpix);
       FPX_ClearSystem();
@@ -463,7 +460,7 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
           q->green=index;
           q->blue=index;
         }
-      if (image->matte != False)
+      if (image->matte != MagickFalse)
         q->opacity=ScaleCharToQuantum(255-*a);
       q++;
       r+=red_component->columnStride;
@@ -471,11 +468,16 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       b+=blue_component->columnStride;
       a+=alpha_component->columnStride;
     }
-    if (SyncImagePixels(image) == False)
+    if (SyncImagePixels(image) == MagickFalse)
       break;
-    if (QuantumTick(y,image->rows))
-      if (MagickMonitor(LoadImageTag,y,image->rows,exception) == False)
-        break;
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(LoadImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
   scanline=(char *) RelinquishMagickMemory(scanline);
   (void) FPX_CloseImage(flashpix);
@@ -517,8 +519,9 @@ ModuleExport void RegisterFPXImage(void)
   entry->decoder=(DecoderHandler *) ReadFPXImage;
   entry->encoder=(EncoderHandler *) WriteFPXImage;
 #endif
-  entry->adjoin=False;
-  entry->seekable_stream=True;
+  entry->adjoin=MagickFalse;
+  entry->seekable_stream=MagickTrue;
+  entry->blob_support=MagickFalse;
   entry->magick=(MagickHandler *) IsFPX;
   entry->description=AcquireString("FlashPix Format");
   entry->module=AcquireString("FPX");
@@ -566,14 +569,10 @@ ModuleExport void UnregisterFPXImage(void)
 %
 %  The format of the WriteFPXImage method is:
 %
-%      unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteFPXImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o fpx_status: Method WriteFPXImage return True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
-%
 %    o image_info: The image info.
 %
 %    o image:  The image.
@@ -762,7 +761,7 @@ static void SetSaturation(double saturation,FPXColorTwistMatrix *color_twist)
   *color_twist=result;
 }
 
-static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteFPXImage(const ImageInfo *image_info,Image *image)
 {
   FPXBackground
     background_color;
@@ -801,15 +800,18 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
   long
     y;
 
+  MagickBooleanType
+    status;
+
+  register const PixelPacket
+    *p;
+
   register long
     i;
 
   unsigned char
     *pixels;
 
-  unsigned int
-    status;
-
   unsigned long
     memory_limit,
     tile_height,
@@ -822,10 +824,10 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  if (image->debug != False)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
+  if (status == MagickFalse)
     return(status);
   CloseBlob(image);
   /*
@@ -840,7 +842,7 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
   tile_width=64;
   tile_height=64;
   colorspace.numberOfComponents=3;
-  if (image->matte != False)
+  if (image->matte != MagickFalse)
     colorspace.numberOfComponents=4;
   if ((image_info->type != TrueColorType) &&
       IsGrayImage(image,&image->exception))
@@ -876,56 +878,56 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
         Initialize the compression by quality for the entire image.
       */
       fpx_status=FPX_SetJPEGCompression(flashpix,(unsigned short)
-        image_info->quality == 0 ? 75 : image_info->quality);
+        image->quality == UndefinedCompressionQuality ? 75 : image->quality);
       if (fpx_status != FPX_OK)
         ThrowWriterException(DelegateError,"UnableToSetJPEGLevel");
     }
   /*
     Set image summary info.
   */
-  summary_info.title_valid=False;
-  summary_info.subject_valid=False;
-  summary_info.author_valid=False;
-  summary_info.comments_valid=False;
-  summary_info.keywords_valid=False;
-  summary_info.OLEtemplate_valid=False;
-  summary_info.last_author_valid=False;
-  summary_info.rev_number_valid=False;
-  summary_info.edit_time_valid=False;
-  summary_info.last_printed_valid=False;
-  summary_info.create_dtm_valid=False;
-  summary_info.last_save_dtm_valid=False;
-  summary_info.page_count_valid=False;
-  summary_info.word_count_valid=False;
-  summary_info.char_count_valid=False;
-  summary_info.thumbnail_valid=False;
-  summary_info.appname_valid=False;
-  summary_info.security_valid=False;
+  summary_info.title_valid=MagickFalse;
+  summary_info.subject_valid=MagickFalse;
+  summary_info.author_valid=MagickFalse;
+  summary_info.comments_valid=MagickFalse;
+  summary_info.keywords_valid=MagickFalse;
+  summary_info.OLEtemplate_valid=MagickFalse;
+  summary_info.last_author_valid=MagickFalse;
+  summary_info.rev_number_valid=MagickFalse;
+  summary_info.edit_time_valid=MagickFalse;
+  summary_info.last_printed_valid=MagickFalse;
+  summary_info.create_dtm_valid=MagickFalse;
+  summary_info.last_save_dtm_valid=MagickFalse;
+  summary_info.page_count_valid=MagickFalse;
+  summary_info.word_count_valid=MagickFalse;
+  summary_info.char_count_valid=MagickFalse;
+  summary_info.thumbnail_valid=MagickFalse;
+  summary_info.appname_valid=MagickFalse;
+  summary_info.security_valid=MagickFalse;
   label=GetImageAttribute(image,"label");
   if (label != (ImageAttribute *) NULL)
     {
       /*
         Note image label.
       */
-      summary_info.title_valid=True;
+      summary_info.title_valid=MagickTrue;
       summary_info.title.length=strlen(label->value);
       summary_info.title.ptr=(unsigned char *)
-        AcquireMagickMemory(strlen(label->value)+1);
+        AcquireMagickMemory(strlen(label->value)+MaxTextExtent);
       if (summary_info.title.ptr == (unsigned char *) NULL)
         ThrowWriterException(DelegateError,"UnableToSetImageTitle");
       (void) CopyMagickString((char *) summary_info.title.ptr,label->value,
         MaxTextExtent);
     }
-  comment=GetImageAttribute(image,"comment");
+  comment=GetImageAttribute(image,"Comment");
   if (comment != (ImageAttribute *) NULL)
     {
       /*
         Note image comment.
       */
-      summary_info.comments_valid=True;
+      summary_info.comments_valid=MagickTrue;
       summary_info.comments.length=strlen(comment->value);
       summary_info.comments.ptr=(unsigned char *)
-        AcquireMagickMemory(strlen(comment->value)+1);
+        AcquireMagickMemory(strlen(comment->value)+MaxTextExtent);
       if (summary_info.comments.ptr == (unsigned char *) NULL)
         ThrowWriterException(DelegateError,"UnableToSetImageComment");
       (void) strcpy((char *) summary_info.comments.ptr,comment->value);
@@ -968,21 +970,27 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
   */
   for (y=0; y < (long) image->rows; y++)
   {
-    if (AcquireImagePixels(image,0,y,image->columns,1,&image->exception) == False)
+    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    if (p == (const PixelPacket *) NULL)
       break;
     if (fpx_info.numberOfComponents == 1)
       (void) PopImagePixels(image,GrayQuantum,pixels);
     else
-      if (image->matte == False)
+      if (image->matte == MagickFalse)
         (void) PopImagePixels(image,RGBQuantum,pixels);
       else
         (void) PopImagePixels(image,RGBAQuantum,pixels);
     fpx_status=FPX_WriteImageLine(flashpix,&fpx_info);
     if (fpx_status != FPX_OK)
       break;
-    if (QuantumTick(y,image->rows))
-      if (MagickMonitor(SaveImageTag,y,image->rows,&image->exception) == False)
-        break;
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(SaveImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
   if (image_info->view != (char *) NULL)
     {
@@ -1004,7 +1012,7 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
       FPXROI
         view_rect;
 
-      unsigned int
+      MagickBooleanType
         affine_valid,
         aspect_ratio_valid,
         color_twist_valid,
@@ -1016,7 +1024,7 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
         Initialize default viewing parameters.
       */
       contrast=1.0;
-      contrast_valid=False;
+      contrast_valid=MagickFalse;
       color_twist.byy=1.0;
       color_twist.byc1=0.0;
       color_twist.byc2=0.0;
@@ -1033,16 +1041,16 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
       color_twist.dummy5_zero=0.0;
       color_twist.dummy6_zero=0.0;
       color_twist.dummy7_one=1.0;
-      color_twist_valid=False;
+      color_twist_valid=MagickFalse;
       sharpen=0.0;
-      sharpen_valid=False;
+      sharpen_valid=MagickFalse;
       aspect_ratio=(double) image->columns/image->rows;
-      aspect_ratio_valid=False;
+      aspect_ratio_valid=MagickFalse;
       view_rect.left=(float) 0.1;
       view_rect.width=aspect_ratio-0.2;
       view_rect.top=(float) 0.1;
       view_rect.height=(float) 0.8; /* 1.0-0.2 */
-      view_rect_valid=False;
+      view_rect_valid=MagickFalse;
       affine.a11=1.0;
       affine.a12=0.0;
       affine.a13=0.0;
@@ -1059,7 +1067,7 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
       affine.a42=0.0;
       affine.a43=0.0;
       affine.a44=1.0;
-      affine_valid=False;
+      affine_valid=MagickFalse;
       if (0)
         {
           /*
@@ -1068,7 +1076,7 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
           SetBrightness(0.5,&color_twist);
           SetSaturation(0.5,&color_twist);
           SetColorBalance(0.5,1.0,1.0,&color_twist);
-          color_twist_valid=True;
+          color_twist_valid=MagickTrue;
         }
       if (affine_valid)
         {
@@ -1110,6 +1118,6 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
   (void) FPX_CloseImage(flashpix);
   FPX_ClearSystem();
   pixels=(unsigned char *) RelinquishMagickMemory(pixels);
-  return(True);
+  return(MagickTrue);
 }
 #endif
