@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,9 +41,12 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/colorspace.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -129,7 +132,7 @@ typedef struct _IconInfo
 /*
   Forward declaractions.
 */
-static unsigned int
+static MagickBooleanType
   WriteICONImage(const ImageInfo *,Image *);
 
 /*
@@ -154,10 +157,6 @@ static unsigned int
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadICONImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a memory shortage or
-%      if the image cannot be read.
-%
 %    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
@@ -179,6 +178,9 @@ static Image *ReadICONImage(const ImageInfo *image_info,
   long
     y;
 
+  MagickBooleanType
+    status;
+
   register IndexPacket
     *indexes;
 
@@ -192,9 +194,6 @@ static Image *ReadICONImage(const ImageInfo *image_info,
   register unsigned char
     *p;
 
-  unsigned int
-    status;
-
   unsigned long
     bit,
     byte;
@@ -204,15 +203,15 @@ static Image *ReadICONImage(const ImageInfo *image_info,
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
-  if (image_info->debug != False)
+  if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
+  if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   icon_file.reserved=(short) ReadBlobLSBShort(image);
@@ -251,7 +250,7 @@ static Image *ReadICONImage(const ImageInfo *image_info,
     icon_info.y_pixels=ReadBlobLSBLong(image);
     icon_info.number_colors=ReadBlobLSBLong(image);
     icon_info.colors_important=ReadBlobLSBLong(image);
-    image->matte=(unsigned int) (icon_info.bits_per_pixel == 32);
+    image->matte=(MagickBooleanType) (icon_info.bits_per_pixel == 32);
     image->columns=icon_info.width;
     image->rows=icon_info.height;
     image->depth=8;
@@ -273,7 +272,7 @@ static Image *ReadICONImage(const ImageInfo *image_info,
       /*
         Read Icon raster colormap.
       */
-      if (AllocateImageColormap(image,1 << icon_info.bits_per_pixel) == False)
+      if (AllocateImageColormap(image,1 << icon_info.bits_per_pixel) == MagickFalse)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       icon_colormap=(unsigned char *)
         AcquireMagickMemory((size_t) (4*image->colors));
@@ -295,7 +294,7 @@ static Image *ReadICONImage(const ImageInfo *image_info,
     */
     image->columns=(unsigned long) icon_file.directory[i].width;
     image->rows=(unsigned long) icon_file.directory[i].height;
-    if ((image_info->ping != False) && (image_info->number_scenes != 0))
+    if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     switch (icon_info.bits_per_pixel)
@@ -325,14 +324,15 @@ static Image *ReadICONImage(const ImageInfo *image_info,
                 indexes[x+bit]=(IndexPacket)
                   ((byte & (0x80 >> bit)) != 0 ? 0x01 : 0x00);
             }
-          if (SyncImagePixels(image) == False)
+          if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows))
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(LoadImageTag,(MagickOffsetType)
-                  image->rows-y-1,image->rows,&image->exception);
-                if (status == False)
+                status=image->progress_monitor(LoadImageTag,y,image->rows,
+                  image->client_data);
+                if (status == MagickFalse)
                   break;
               }
         }
@@ -360,14 +360,15 @@ static Image *ReadICONImage(const ImageInfo *image_info,
               byte=(unsigned long) ReadBlobByte(image);
               indexes[x]=(IndexPacket) ((byte >> 4) & 0xf);
             }
-          if (SyncImagePixels(image) == False)
+          if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows))
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(LoadImageTag,(MagickOffsetType)
-                  image->rows-y-1,image->rows,&image->exception);
-                if (status == False)
+                status=image->progress_monitor(LoadImageTag,image->rows-y-1,
+                  image->rows,image->client_data);
+                if (status == MagickFalse)
                   break;
               }
         }
@@ -389,14 +390,15 @@ static Image *ReadICONImage(const ImageInfo *image_info,
             byte=(unsigned long) ReadBlobByte(image);
             indexes[x]=(IndexPacket) byte;
           }
-          if (SyncImagePixels(image) == False)
+          if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows))
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(LoadImageTag,(MagickOffsetType)
-                  image->rows-y-1,image->rows,&image->exception);
-                if (status == False)
+                status=image->progress_monitor(LoadImageTag,image->rows-y-1,
+                  image->rows,image->client_data);
+                if (status == MagickFalse)
                   break;
               }
         }
@@ -420,14 +422,15 @@ static Image *ReadICONImage(const ImageInfo *image_info,
             byte=(unsigned long) ReadBlobByte(image);
             indexes[x]|=byte << 8;
           }
-          if (SyncImagePixels(image) == False)
+          if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows))
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(LoadImageTag,(MagickOffsetType)
-                  image->rows-y-1,image->rows,&image->exception);
-                if (status == False)
+                status=image->progress_monitor(LoadImageTag,image->rows-y-1,
+                  image->rows,image->client_data);
+                if (status == MagickFalse)
                   break;
               }
         }
@@ -449,18 +452,19 @@ static Image *ReadICONImage(const ImageInfo *image_info,
             q->blue=(Quantum) ScaleCharToQuantum(ReadBlobByte(image));
             q->green=(Quantum) ScaleCharToQuantum(ReadBlobByte(image));
             q->red=(Quantum) ScaleCharToQuantum(ReadBlobByte(image));
-            if (image->matte != False)
-              q->opacity=(Quantum) ScaleCharToQuantum(ReadBlobByte(image));
+            if (image->matte != MagickFalse)
+              q->opacity=(MaxRGB-ScaleCharToQuantum(ReadBlobByte(image)));
             q++;
           }
-          if (SyncImagePixels(image) == False)
+          if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows))
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(LoadImageTag,(MagickOffsetType)
-                  image->rows-y-1,image->rows,&image->exception);
-                if (status == False)
+                status=image->progress_monitor(LoadImageTag,image->rows-y-1,
+                  image->rows,image->client_data);
+                if (status == MagickFalse)
                   break;
               }
         }
@@ -470,45 +474,40 @@ static Image *ReadICONImage(const ImageInfo *image_info,
         ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     }
     (void) SyncImage(image);
-    /*
-      Convert bitmap scanline to pixel packets.
-    */
-    image->storage_class=DirectClass;
-    image->matte=True;
-    for (y=(long) image->rows-1; y >= 0; y--)
-    {
-      q=GetImagePixels(image,0,y,image->columns,1);
-      if (q == (PixelPacket *) NULL)
-        break;
-      for (x=0; x < ((long) image->columns-7); x+=8)
+    if (image->matte != MagickTrue)
       {
-        byte=(unsigned long) ReadBlobByte(image);
-        for (bit=0; bit < 8; bit++)
-          q[x+bit].opacity=(Quantum) (((byte & (0x80 >> bit)) != False) ?
-            TransparentOpacity : OpaqueOpacity);
-      }
-      if ((image->columns % 8) != 0)
+        /*
+          Convert bitmap scanline to pixel packets.
+        */
+        image->storage_class=DirectClass;
+        image->matte=MagickTrue;
+        for (y=(long) image->rows-1; y >= 0; y--)
         {
-          byte=(unsigned long) ReadBlobByte(image);
-          for (bit=0; bit < (image->columns % 8); bit++)
-            q[x+bit].opacity=(Quantum) (((byte & (0x80 >> bit)) != False) ?
-              TransparentOpacity : OpaqueOpacity);
-        }
-     if ((image->columns % 32)  != 0)
-       for (x=0; x < (long) ((32-(image->columns % 32))/8); x++)
-         (void) ReadBlobByte(image);
-      if (SyncImagePixels(image) == False)
-        break;
-      if (image->previous == (Image *) NULL)
-        if (QuantumTick(y,image->rows))
+          q=GetImagePixels(image,0,y,image->columns,1);
+          if (q == (PixelPacket *) NULL)
+            break;
+          for (x=0; x < ((long) image->columns-7); x+=8)
           {
-            status=MagickMonitor(LoadImageTag,(MagickOffsetType)
-              image->rows-y-1,image->rows,&image->exception);
-            if (status == False)
-              break;
+            byte=(unsigned long) ReadBlobByte(image);
+            for (bit=0; bit < 8; bit++)
+              q[x+bit].opacity=(Quantum) (((byte & (0x80 >> bit)) != 0) ?
+                TransparentOpacity : OpaqueOpacity);
           }
-    }
-    if (EOFBlob(image) != False)
+          if ((image->columns % 8) != 0)
+            {
+              byte=(unsigned long) ReadBlobByte(image);
+              for (bit=0; bit < (image->columns % 8); bit++)
+                q[x+bit].opacity=(Quantum) (((byte & (0x80 >> bit)) != 0) ?
+                  TransparentOpacity : OpaqueOpacity);
+            }
+         if ((image->columns % 32)  != 0)
+           for (x=0; x < (long) ((32-(image->columns % 32))/8); x++)
+             (void) ReadBlobByte(image);
+          if (SyncImagePixels(image) == MagickFalse)
+            break;
+        }
+      }
+    if (EOFBlob(image) != MagickFalse)
       {
         ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
           image->filename);
@@ -528,14 +527,17 @@ static Image *ReadICONImage(const ImageInfo *image_info,
         AllocateNextImage(image_info,image);
         if (image->next == (Image *) NULL)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
-          exception);
-        if (status == False)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
+              GetBlobSize(image),image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
   }
   CloseBlob(image);
@@ -573,24 +575,24 @@ ModuleExport void RegisterICONImage(void)
   entry=SetMagickInfo("CUR");
   entry->decoder=(DecoderHandler *) ReadICONImage;
   entry->encoder=(EncoderHandler *) WriteICONImage;
-  entry->adjoin=False;
-  entry->seekable_stream=True;
+  entry->adjoin=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=AcquireString("Microsoft icon");
   entry->module=AcquireString("CUR");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("ICO");
   entry->decoder=(DecoderHandler *) ReadICONImage;
   entry->encoder=(EncoderHandler *) WriteICONImage;
-  entry->adjoin=False;
-  entry->seekable_stream=True;
+  entry->adjoin=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=AcquireString("Microsoft icon");
   entry->module=AcquireString("ICON");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("ICON");
   entry->decoder=(DecoderHandler *) ReadICONImage;
   entry->encoder=(EncoderHandler *) WriteICONImage;
-  entry->adjoin=False;
-  entry->seekable_stream=True;
+  entry->adjoin=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=AcquireString("Microsoft icon");
   entry->module=AcquireString("ICON");
   (void) RegisterMagickInfo(entry);
@@ -639,7 +641,7 @@ ModuleExport void UnregisterICONImage(void)
 %
 %  The format of the WriteICONImage method is:
 %
-%      unsigned int WriteICONImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteICONImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
@@ -649,7 +651,8 @@ ModuleExport void UnregisterICONImage(void)
 %
 %
 */
-static unsigned int WriteICONImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
+  Image *image)
 {
   IconFile
     icon_file;
@@ -663,6 +666,9 @@ static unsigned int WriteICONImage(const ImageInfo *image_info,Image *image)
   long
     y;
 
+  MagickBooleanType
+    status;
+
   MagickOffsetType
     scene;
 
@@ -684,9 +690,6 @@ static unsigned int WriteICONImage(const ImageInfo *image_info,Image *image)
     byte,
     *pixels;
 
-  unsigned int
-    status;
-
   unsigned long
     bytes_per_line,
     scanline_pad;
@@ -698,10 +701,10 @@ static unsigned int WriteICONImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  if (image->debug != False)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
+  if (status == MagickFalse)
     return(status);
   scene=0;
   next=image;
@@ -711,7 +714,7 @@ static unsigned int WriteICONImage(const ImageInfo *image_info,Image *image)
       ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit");
     scene++;
     next=SyncNextImageInList(next);
-  } while (image_info->adjoin != False);
+  } while (image_info->adjoin != MagickFalse);
   /*
     Dump out a ICON header template to be properly initialized later.
   */
@@ -732,7 +735,7 @@ static unsigned int WriteICONImage(const ImageInfo *image_info,Image *image)
     (void) WriteBlobLSBLong(image,icon_file.directory[scene].offset);
     scene++;
     next=SyncNextImageInList(next);
-  } while (image_info->adjoin != False);
+  } while (image_info->adjoin != MagickFalse);
   scene=0;
   do
   {
@@ -740,7 +743,7 @@ static unsigned int WriteICONImage(const ImageInfo *image_info,Image *image)
       Initialize ICON raster file header.
     */
     (void) SetImageColorspace(image,RGBColorspace);
-    (void) ResetMagickMemory(&icon_info,0,sizeof(IconInfo));
+    (void) ResetMagickMemory(&icon_info,0,sizeof(icon_info));
     icon_info.file_size=14+12+28;
     icon_info.offset_bits=icon_info.file_size;
     icon_info.compression=BI_RGB;
@@ -776,11 +779,11 @@ static unsigned int WriteICONImage(const ImageInfo *image_info,Image *image)
         */
         icon_info.number_colors=0;
         icon_info.bits_per_pixel=(unsigned short) 24;
-        icon_info.compression=(unsigned int) BI_RGB;
+        icon_info.compression=(unsigned long) BI_RGB;
       }
     bytes_per_line=4*((image->columns*icon_info.bits_per_pixel+31)/32);
     icon_info.ba_offset=0;
-    if (image->matte == False)
+    if (image->matte == MagickFalse)
       icon_info.size=40;
     else
       {
@@ -799,16 +802,22 @@ static unsigned int WriteICONImage(const ImageInfo *image_info,Image *image)
     icon_info.file_size+=icon_info.image_size;
     icon_info.x_pixels=75*39;
     icon_info.y_pixels=75*39;
-    if (image->units == PixelsPerInchResolution)
+    switch (image->units)
+    {
+      case UndefinedResolution:
+      case PixelsPerInchResolution:
       {
         icon_info.x_pixels=(unsigned long) (100.0*image->x_resolution/2.54);
         icon_info.y_pixels=(unsigned long) (100.0*image->y_resolution/2.54);
+        break;
       }
-    if (image->units == PixelsPerCentimeterResolution)
+      case PixelsPerCentimeterResolution:
       {
         icon_info.x_pixels=(unsigned long) (100.0*image->x_resolution);
         icon_info.y_pixels=(unsigned long) (100.0*image->y_resolution);
+        break;
       }
+    }
     icon_info.colors_important=icon_info.number_colors;
     /*
       Convert MIFF to ICON raster pixels.
@@ -852,11 +861,12 @@ static unsigned int WriteICONImage(const ImageInfo *image_info,Image *image)
          if (bit != 0)
            *q++=(unsigned char) (byte << (8-bit));
          if (image->previous == (Image *) NULL)
-           if (QuantumTick(y,image->rows))
+           if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+               (QuantumTick(y,image->rows) != MagickFalse))
              {
-               status=MagickMonitor(SaveImageTag,y,image->rows,
-                 &image->exception);
-               if (status == False)
+               status=image->progress_monitor(SaveImageTag,y,image->rows,
+                 image->client_data);
+               if (status == MagickFalse)
                  break;
              }
         }
@@ -895,11 +905,12 @@ static unsigned int WriteICONImage(const ImageInfo *image_info,Image *image)
          if (nibble != 0)
            *q++=(unsigned char) (byte << 4);
          if (image->previous == (Image *) NULL)
-           if (QuantumTick(y,image->rows))
+           if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+               (QuantumTick(y,image->rows) != MagickFalse))
              {
-               status=MagickMonitor(SaveImageTag,y,image->rows,
-                 &image->exception);
-               if (status == False)
+               status=image->progress_monitor(SaveImageTag,y,image->rows,
+                 image->client_data);
+               if (status == MagickFalse)
                  break;
              }
         }
@@ -920,13 +931,14 @@ static unsigned int WriteICONImage(const ImageInfo *image_info,Image *image)
           for (x=0; x < (long) image->columns; x++)
             *q++=(unsigned char) indexes[x];
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows))
-             {
-               status=MagickMonitor(SaveImageTag,y,image->rows,
-                 &image->exception);
-               if (status == False)
-                 break;
-             }
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
+              {
+                status=image->progress_monitor(SaveImageTag,y,image->rows,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
         }
         break;
       }
@@ -952,11 +964,12 @@ static unsigned int WriteICONImage(const ImageInfo *image_info,Image *image)
             p++;
           }
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows))
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(SaveImageTag,y,image->rows,
-                  &image->exception);
-                if (status == False)
+                status=image->progress_monitor(SaveImageTag,y,image->rows,
+                  image->client_data);
+                if (status == MagickFalse)
                   break;
               }
         }
@@ -1050,11 +1063,15 @@ static unsigned int WriteICONImage(const ImageInfo *image_info,Image *image)
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == False)
-      break;
-  } while (image_info->adjoin != False);
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
+  } while (image_info->adjoin != MagickFalse);
   (void) SeekBlob(image,0,SEEK_SET);
   (void) WriteBlobLSBShort(image,0);
   (void) WriteBlobLSBShort(image,1);
@@ -1074,7 +1091,7 @@ static unsigned int WriteICONImage(const ImageInfo *image_info,Image *image)
     (void) WriteBlobLSBLong(image,icon_file.directory[scene].offset);
     scene++;
     next=SyncNextImageInList(next);
-  } while (image_info->adjoin != False);
+  } while (image_info->adjoin != MagickFalse);
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
