@@ -17,32 +17,20 @@
 %                            Christopher R. Hawks                             %
 %                               December 2001                                 %
 %                                                                             %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Copyright (C 2001 ImageMagick Studio LLC, a non-profit organization dedicated  %
-%  to making software imaging solutions freely available.                     %
+%  You may not use this file except in compliance with the License.  You may  %
+   obtain a copy of the License at                                            %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick",   %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -56,12 +44,15 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/color.h"
 #include "magick/color_private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -160,7 +151,7 @@ static unsigned char
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WritePALMImage(const ImageInfo *,Image *);
 
 /*
@@ -242,6 +233,12 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
   long
     y;
 
+  MagickBooleanType
+    status;
+
+  MagickPixelPacket
+    transpix;
+
   register IndexPacket
     *indexes;
 
@@ -250,7 +247,6 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
     x;
 
   register PixelPacket
-    transpix,
     *q;
 
   ssize_t
@@ -261,9 +257,6 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
     *one_row,
     *ptr;
 
-  unsigned int
-    status;
-
   unsigned long
     bytes_per_row,
     flags,
@@ -289,9 +282,9 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
   */
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
+  if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   image->columns=ReadBlobMSBShort(image);
@@ -310,6 +303,7 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
   if ((bits_per_pixel < 16) &&
       !AllocateImageColormap(image,1L << bits_per_pixel))
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  GetMagickPixelPacket(image,&transpix);
   if (bits_per_pixel == 16)  /* Direct Color */
     {
       redbits=ReadBlobByte(image);  /* # of bits of red */
@@ -410,9 +404,9 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
         {
           color16=(*ptr++ << 8);
           color16|=(*ptr++);
-          q->red=((MaxRGB*(color16 >> 11) & 0x1f))/31;
-          q->green=((MaxRGB*(color16 >> 5) & 0x3f))/63;
-          q->blue=(MaxRGB*(color16 & 0x1f))/31;
+          q->red=(MaxRGB*((color16 >> 11) & 0x1f))/0x1f;
+          q->green=(MaxRGB*((color16 >> 5) & 0x3f))/0x3f;
+          q->blue=(MaxRGB*((color16 >> 0) & 0x1f))/0x1f; 
           q->opacity=OpaqueOpacity;
           q++;
         }
@@ -433,17 +427,16 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
               bit=8-bits_per_pixel;
             }
         }
-        if (SyncImagePixels(image) == False)
+        if (SyncImagePixels(image) == MagickFalse)
           break;
       }
     }
   if (flags & PALM_HAS_TRANSPARENCY_FLAG)
     {
-      if (bits_per_pixel == 16)
-        TransparentImage(image,transpix,TransparentOpacity);
-      else
-        TransparentImage(image,image->colormap[mask-transparentIndex],
-          TransparentOpacity);
+      if (bits_per_pixel != 16)
+        SetMagickPixelPacket(image->colormap+mask-transparentIndex,
+          (IndexPacket *) NULL,&transpix);
+      PaintTransparentImage(image,&transpix,TransparentOpacity);
     }
   one_row=(unsigned char *) RelinquishMagickMemory(one_row);
   if (compressionType == PALM_COMPRESSION_SCANLINE)
@@ -482,8 +475,8 @@ ModuleExport void RegisterPALMImage(void)
   entry=SetMagickInfo("PALM");
   entry->decoder=(DecoderHandler *) ReadPALMImage;
   entry->encoder=(EncoderHandler *) WritePALMImage;
-  entry->adjoin=False;
-  entry->seekable_stream=True;
+  entry->adjoin=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=AcquireString("Palm pixmap");
   entry->module=AcquireString("PALM");
   (void) RegisterMagickInfo(entry);
@@ -500,8 +493,8 @@ ModuleExport void RegisterPALMImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterPALMImage removes format registrations made by the
-%  PALM module from the list of supported formats.
+%  UnregisterPALMImage() removes format registrations made by the PALM
+%  module from the list of supported formats.
 %
 %  The format of the UnregisterPALMImage method is:
 %
@@ -528,7 +521,7 @@ ModuleExport void UnregisterPALMImage(void)
 %
 %  The format of the WritePALMImage method is:
 %
-%      unsigned int WritePALMImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WritePALMImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
@@ -538,13 +531,13 @@ ModuleExport void UnregisterPALMImage(void)
 %
 %
 */
-static unsigned int WritePALMImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WritePALMImage(const ImageInfo *image_info,Image *image)
 {
   int
     y;
 
-  QuantizeInfo
-    quantize_info;
+  const ImageAttribute
+    *attribute;
 
   Image
     *map;
@@ -552,8 +545,11 @@ static unsigned int WritePALMImage(const ImageInfo *image_info,Image *image)
   ExceptionInfo
     exception;
 
-  const ImageAttribute
-    *attribute;
+  MagickBooleanType
+    status;
+
+  QuantizeInfo
+    quantize_info;
 
   register IndexPacket
     *indexes;
@@ -575,8 +571,7 @@ static unsigned int WritePALMImage(const ImageInfo *image_info,Image *image)
     version = 0;
 
   unsigned int
-    transparentIndex = 0,
-    status;
+    transparentIndex = 0;
 
   unsigned long
     count,
@@ -593,7 +588,7 @@ static unsigned int WritePALMImage(const ImageInfo *image_info,Image *image)
     Open output image file.
   */
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&exception);
-  if (status == False)
+  if (status == MagickFalse)
     return(status);
   GetExceptionInfo(&exception);
   attribute=GetImageAttribute(image, "Comment");
@@ -605,7 +600,7 @@ static unsigned int WritePALMImage(const ImageInfo *image_info,Image *image)
   if (image_info->depth > 100)
     bits_per_pixel=image_info->depth-100;
   if (bits_per_pixel < 16)
-    TransformColorspace(image,image->colorspace);
+    SetImageColorspace(image,image->colorspace);
   if (bits_per_pixel < 8)
     {
       SetImageColorspace(image,GRAYColorspace);
@@ -628,7 +623,7 @@ static unsigned int WritePALMImage(const ImageInfo *image_info,Image *image)
       (image->compression == FaxCompression))
     version=2;
   (void) WriteBlobByte(image,version);
-  (void) WriteBlobMSBShort(image,0);  /* offset */
+  (void) WriteBlobMSBShort(image,0);  /* nextDepthOffset */
   (void) WriteBlobByte(image,transparentIndex);
   if (image->compression == RLECompression)
     (void) WriteBlobByte(image,PALM_COMPRESSION_RLE);
@@ -653,7 +648,8 @@ static unsigned int WritePALMImage(const ImageInfo *image_info,Image *image)
       if (flags & PALM_HAS_COLORMAP_FLAG)  /* Write out colormap */
         {
           GetQuantizeInfo(&quantize_info);
-          quantize_info.dither=image_info->dither;
+          quantize_info.dither=IsPaletteImage(image,&image->exception) ==
+            MagickFalse ? MagickTrue : MagickFalse;
           quantize_info.number_colors=image->colors;
           (void) QuantizeImage(&quantize_info,image);
           WriteBlobMSBShort(image,image->colors);
@@ -671,7 +667,7 @@ static unsigned int WritePALMImage(const ImageInfo *image_info,Image *image)
     else  /* Map colors to Palm standard colormap */
       {
         map=ConstituteImage(256,1,"RGB",CharPixel,&PalmPalette,&exception);
-        TransformColorspace(map,map->colorspace);
+        SetImageColorspace(map,map->colorspace);
         MapImage(image,map,image_info->dither);
         for (y=0; y < (long) image->rows; y++)
         {
@@ -680,7 +676,7 @@ static unsigned int WritePALMImage(const ImageInfo *image_info,Image *image)
           for (x=0; x < (long) image->columns; x++)
             indexes[x]=FindColor(&image->colormap[indexes[x]]);
         }
-        DestroyImage(map);
+        map=DestroyImage(map);
       }
     }
   if (flags & PALM_IS_COMPRESSED_FLAG)
@@ -691,6 +687,10 @@ static unsigned int WritePALMImage(const ImageInfo *image_info,Image *image)
   one_row=(unsigned char *) AcquireMagickMemory(bytes_per_row);
   if (one_row == (unsigned char *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+  transpix.red=0;
+  transpix.green=0;
+  transpix.blue=0;
+  transpix.opacity=0;
   for (y=0; y < (int) image->rows; y++)
   {
     ptr=one_row;
@@ -800,12 +800,12 @@ static unsigned int WritePALMImage(const ImageInfo *image_info,Image *image)
   if (flags & PALM_IS_COMPRESSED_FLAG)  /* fill in size now */
     {
       SeekBlob(image,offset,SEEK_SET);
-      (void) WriteBlobMSBShort(image,GetBlobSize(image)-16);
+      (void) WriteBlobMSBShort(image,GetBlobSize(image)-offset);
     }
   CloseBlob(image);
   one_row=(unsigned char *) RelinquishMagickMemory(one_row);
   if (image->compression == FaxCompression)
     lastrow=(unsigned char *) RelinquishMagickMemory(lastrow);
   DestroyExceptionInfo(&exception);
-  return(True);
+  return(MagickTrue);
 }
