@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,11 +42,14 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/color_private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -113,7 +116,7 @@ typedef struct _PDBImage
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WritePDBImage(const ImageInfo *,Image *);
 
 /*
@@ -132,14 +135,11 @@ static unsigned int
 %
 %  The format of the DecodeImage method is:
 %
-%      unsigned int DecodeImage(Image *image,unsigned char *pixels,
+%      MagickBooleanType DecodeImage(Image *image,unsigned char *pixels,
 %        const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method DecodeImage returns True if all the pixels are
-%      uncompressed without error, otherwise False.
-%
 %    o image: The address of a structure of type Image.
 %
 %    o pixels:  The address of a byte (8 bits) array of pixel data created by
@@ -147,7 +147,7 @@ static unsigned int
 %
 %
 */
-static unsigned int DecodeImage(Image *image,unsigned char *pixels,
+static MagickBooleanType DecodeImage(Image *image,unsigned char *pixels,
   const size_t length)
 {
   long
@@ -178,7 +178,7 @@ static unsigned int DecodeImage(Image *image,unsigned char *pixels,
     for (i=0; i < count; i++)
       *p++=(unsigned char) pixel;
   }
-  return(True);
+  return(MagickTrue);
 }
 
 /*
@@ -192,17 +192,15 @@ static unsigned int DecodeImage(Image *image,unsigned char *pixels,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  IsPDB() returns True if the image format type, identified by the
+%  IsPDB() returns MagickTrue if the image format type, identified by the
 %  magick string, is PDB.
 %
 %  The format of the ReadPDBImage method is:
 %
-%      unsigned int IsPDB(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsPDB(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsPDB() returns True if the image format type is PDB.
-%
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
 %
@@ -210,13 +208,13 @@ static unsigned int DecodeImage(Image *image,unsigned char *pixels,
 %
 %
 */
-static unsigned int IsPDB(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsPDB(const unsigned char *magick,const size_t length)
 {
   if (length < 68)
-    return(False);
+    return(MagickFalse);
   if (memcmp(magick+60,"vIMGView",8) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -240,10 +238,6 @@ static unsigned int IsPDB(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadPDBImage returns a pointer to the image after
-%      reading. A null image is returned if there is a memory shortage or if
-%      the image cannot be read.
-%
 %    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
@@ -266,6 +260,9 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
     offset,
     y;
 
+  MagickBooleanType
+    status;
+
   PDBImage
     pdb_image;
 
@@ -290,9 +287,6 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   unsigned char
     *pixels;
 
-  unsigned int
-    status;
-
   unsigned long
     bits_per_pixel,
     packets;
@@ -302,15 +296,15 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
-  if (image_info->debug != False)
+  if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
+  if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
@@ -386,9 +380,9 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   image->depth=8;
   image->storage_class=PseudoClass;
   bits_per_pixel=pdb_image.type == 0 ? 2 : pdb_image.type == 2 ? 4 : 1;
-  if (AllocateImageColormap(image,1 << bits_per_pixel) == False)
+  if (AllocateImageColormap(image,1 << bits_per_pixel) == MagickFalse)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-  if (image_info->ping != False)
+  if (image_info->ping != MagickFalse)
     {
       CloseBlob(image);
       return(GetFirstImageInList(image));
@@ -412,7 +406,8 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       break;
     }
     default:
-      ThrowReaderException(CorruptImageError,"UnrecognizedCompressionType");
+      ThrowReaderException(CorruptImageError,
+        "UnrecognizedImageCompressionType");
   }
   p=pixels;
   switch (bits_per_pixel)
@@ -441,11 +436,16 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           }
           p++;
         }
-        if (SyncImagePixels(image) == False)
+        if (SyncImagePixels(image) == MagickFalse)
           break;
-        if (QuantumTick(y,image->rows))
-          if (MagickMonitor(LoadImageTag,y,image->rows,exception) == False)
-            break;
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
+          {
+            status=image->progress_monitor(LoadImageTag,y,image->rows,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
       break;
     }
@@ -476,11 +476,16 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           *q++=image->colormap[index];
           p++;
         }
-        if (SyncImagePixels(image) == False)
+        if (SyncImagePixels(image) == MagickFalse)
           break;
-        if (QuantumTick(y,image->rows))
-          if (MagickMonitor(LoadImageTag,y,image->rows,exception) == False)
-            break;
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
+          {
+            status=image->progress_monitor(LoadImageTag,y,image->rows,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
       break;
     }
@@ -505,11 +510,16 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           *q++=image->colormap[index];
           p++;
         }
-        if (SyncImagePixels(image) == False)
+        if (SyncImagePixels(image) == MagickFalse)
           break;
-        if (QuantumTick(y,image->rows))
-          if (MagickMonitor(LoadImageTag,y,image->rows,exception) == False)
-            break;
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
+          {
+            status=image->progress_monitor(LoadImageTag,y,image->rows,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
       break;
     }
@@ -517,7 +527,7 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   }
   pixels=(unsigned char *) RelinquishMagickMemory(pixels);
-  if (EOFBlob(image) != False)
+  if (EOFBlob(image) != MagickFalse)
     ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
       image->filename);
   if ((offset-TellBlob(image)) == 0)
@@ -531,7 +541,7 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       register char
         *p;
 
-      unsigned int
+      unsigned long
         length;
 
       /*
@@ -539,27 +549,27 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       */
       c=ReadBlobByte(image);
       length=MaxTextExtent;
-      comment=(char *) AcquireMagickMemory(length+1);
-      p=comment;
-      if (comment != (char *) NULL)
-        for ( ; c != EOF; p++)
-        {
-          if ((p-comment) >= (int) length)
-            {
-              length<<=1;
-              length+=MaxTextExtent;
-              comment=(char *) ResizeMagickMemory(comment,length+1);
-              if (comment == (char *) NULL)
-                break;
-              p=comment+strlen(comment);
-            }
-          *p=c;
-          *(p+1)='\0';
-          c=ReadBlobByte(image);
-        }
+      comment=AcquireString((char *) NULL);
+      for (p=comment; c != EOF; p++)
+      {
+        if ((size_t) (p-comment+MaxTextExtent) >= length)
+          {
+            *p='\0';
+            length<<=1;
+            length+=MaxTextExtent;
+            comment=(char *) ResizeMagickMemory(comment,
+              (length+MaxTextExtent)*sizeof(*comment));
+            if (comment == (char *) NULL)
+              break;
+            p=comment+strlen(comment);
+          }
+        *p=c;
+        c=ReadBlobByte(image);
+      }
+      *p='\0';
       if (comment == (char *) NULL)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-      (void) SetImageAttribute(image,"comment",comment);
+      (void) SetImageAttribute(image,"Comment",comment);
       comment=(char *) RelinquishMagickMemory(comment);
     }
   CloseBlob(image);
@@ -642,14 +652,10 @@ ModuleExport void UnregisterPDBImage(void)
 %
 %  The format of the WritePDBImage method is:
 %
-%      unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WritePDBImage return True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
-%
 %    o image_info: The image info.
 %
 %    o image:  The image.
@@ -658,7 +664,7 @@ ModuleExport void UnregisterPDBImage(void)
 */
 
 static unsigned char *EncodeRLE(unsigned char *destination,
-  unsigned char *source,unsigned int literal,unsigned int repeat)
+  unsigned char *source,unsigned long literal,unsigned long repeat)
 {
   if (literal > 0)
     *destination++=literal-1;
@@ -672,7 +678,7 @@ static unsigned char *EncodeRLE(unsigned char *destination,
   return(destination);
 }
 
-static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
 {
   int
     bits;
@@ -680,27 +686,32 @@ static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
   long
     y;
 
+  MagickBooleanType
+    status;
+
   PDBImage
     pdb_image;
 
   PDBInfo
     pdb_info;
 
+  register const PixelPacket
+    *p;
+
   register long
     x;
 
+  register unsigned char
+    *q;
+
   size_t
     packet_size;
 
   unsigned char
     *buffer,
-    *p,
-    *q,
+    *runlength,
     *scanline;
 
-  unsigned int
-    status;
-
   unsigned long
     bits_per_pixel,
     literal,
@@ -717,10 +728,10 @@ static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  if (image->debug != False)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
+  if (status == MagickFalse)
     return(status);
   (void) SetImageColorspace(image,RGBColorspace);
   bits_per_pixel=image->depth;
@@ -742,7 +753,7 @@ static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
   CopyMagickMemory(pdb_info.id,"View",4);
   pdb_info.seed=0;
   pdb_info.next_record=0;
-  comment=GetImageAttribute(image,"comment");
+  comment=GetImageAttribute(image,"Comment");
   pdb_info.number_records=(comment == (ImageAttribute *) NULL ? 1 : 2);
   (void) WriteBlob(image,32,(unsigned char *) pdb_info.name);
   (void) WriteBlobMSBShort(image,pdb_info.attributes);
@@ -778,8 +789,8 @@ static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
     pdb_image.width=(short) (16*(image->columns/16+1));
   pdb_image.height=(short) image->rows;
   packets=(bits_per_pixel*image->columns/8)*image->rows;
-  p=(unsigned char *) AcquireMagickMemory(2*packets);
-  if (p == (unsigned char *) NULL)
+  runlength=(unsigned char *) AcquireMagickMemory(2*packets);
+  if (runlength == (unsigned char *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
   buffer=(unsigned char *) AcquireMagickMemory(256);
   if (buffer == (unsigned char *) NULL)
@@ -795,11 +806,12 @@ static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
   bits=8/(long) bits_per_pixel-1;  /* start at most significant bits */
   literal=0;
   repeat=0;
-  q=p;
+  q=runlength;
   buffer[0]=0x00;
   for (y=0; y < (long) image->rows; y++)
   {
-    if (AcquireImagePixels(image,0,y,image->columns,1,&image->exception) == False)
+    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    if (p == (const PixelPacket *) NULL)
       break;
     (void) PopImagePixels(image,GrayQuantum,scanline);
     for (x=0; x < pdb_image.width; x++)
@@ -849,9 +861,14 @@ static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
         buffer[literal+repeat]=0x00;
       }
     }
-    if (QuantumTick(y,image->rows))
-      if (MagickMonitor(SaveImageTag,y,image->rows,&image->exception) == False)
-        break;
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(SaveImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
   q=EncodeRLE(q,buffer,literal,repeat);
   scanline=(unsigned char *) RelinquishMagickMemory(scanline);
@@ -870,7 +887,7 @@ static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
       /*
         Write the comment record header.
       */
-      (void) WriteBlobMSBLong(image,TellBlob(image)+8+58+q-p);
+      (void) WriteBlobMSBLong(image,TellBlob(image)+8+58+q-runlength);
       (void) WriteBlobByte(image,0x40);
       (void) WriteBlobByte(image,0x6f);
       (void) WriteBlobByte(image,0x80);
@@ -891,10 +908,10 @@ static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
   (void) WriteBlobMSBShort(image,pdb_image.y_anchor);
   (void) WriteBlobMSBShort(image,pdb_image.width);
   (void) WriteBlobMSBShort(image,pdb_image.height);
-  (void) WriteBlob(image,q-p,p);
-  p=(unsigned char *) RelinquishMagickMemory(p);
+  (void) WriteBlob(image,q-runlength,runlength);
+  runlength=(unsigned char *) RelinquishMagickMemory(runlength);
   if (pdb_info.number_records > 1)
     WriteBlobString(image,comment->value);
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
