@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,12 +41,15 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/color_private.h"
 #include "magick/colorspace.h"
 #include "magick/composite.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/log.h"
 #include "magick/magick.h"
@@ -812,12 +815,14 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
     Read PICT header.
   */
+  pixmap.bits_per_pixel=0;
+  pixmap.component_count=0;
   for (i=0; i < 512; i++)
     (void) ReadBlobByte(image);  /* skip header */
   (void) ReadBlobMSBShort(image);  /* skip picture size */
@@ -1046,7 +1051,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 status=AllocateImageColormap(tile_image,tile_image->colors);
                 if (status == MagickFalse)
                   {
-                    DestroyImage(tile_image);
+                    tile_image=DestroyImage(tile_image);
                     ThrowReaderException(ResourceLimitError,
                       "MemoryAllocationFailed");
                   }
@@ -1098,7 +1103,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 pixmap.bits_per_pixel);
             if (pixels == (unsigned char *) NULL)
               {
-                DestroyImage(tile_image);
+                tile_image=DestroyImage(tile_image);
                 ThrowReaderException(ResourceLimitError,
                   "MemoryAllocationFailed");
               }
@@ -1161,10 +1166,11 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                   (pixmap.bits_per_pixel != 16))
                 p+=(pixmap.component_count-1)*tile_image->columns;
               if (destination.bottom == (long) image->rows)
-                if (QuantumTick(y,tile_image->rows))
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,tile_image->rows) != MagickFalse))
                   {
-                    status=MagickMonitor(LoadImageTag,y,tile_image->rows,
-                      &image->exception);
+                    status=image->progress_monitor(LoadImageTag,y,
+                      tile_image->rows,image->client_data);
                     if (status == MagickFalse)
                       break;
                   }
@@ -1175,11 +1181,11 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                   ((bytes_per_line & 0x8000) != 0))
                 (void) CompositeImage(image,CopyCompositeOp,tile_image,
                   destination.left,destination.top);
-            DestroyImage(tile_image);
-            if (destination.bottom != (long) image->rows)
+            tile_image=DestroyImage(tile_image);
+            if (image->progress_monitor != (MagickProgressMonitor) NULL)
               {
-                status=MagickMonitor(LoadImageTag,destination.bottom,
-                  image->rows,&image->exception);
+                status=image->progress_monitor(LoadImageTag,destination.bottom,
+                  image->rows,image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -1217,7 +1223,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 profile=AcquireStringInfo(length);
                 SetStringInfoDatum(profile,info);
                 status=SetImageProfile(image,"icc",profile);
-                DestroyStringInfo(profile);
+                profile=DestroyStringInfo(profile);
                 if (status == MagickFalse)
                   ThrowReaderException(ResourceLimitError,
                     "MemoryAllocationFailed");
@@ -1233,7 +1239,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 if (status == MagickFalse)
                   ThrowReaderException(ResourceLimitError,
                     "MemoryAllocationFailed");
-                DestroyStringInfo(profile);
+                profile=DestroyStringInfo(profile);
                 break;
               }
               default:
@@ -1298,7 +1304,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               MaxTextExtent);
             ThrowFileException(exception,FileOpenError,
               "UnableToCreateTemporaryFile",image->filename);
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
         length=ReadBlobMSBLong(image);
@@ -1314,7 +1320,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
         }
         (void) fclose(file);
         tile_image=ReadImage(read_info,exception);
-        DestroyImageInfo(read_info);
+        read_info=DestroyImageInfo(read_info);
         (void) RelinquishUniqueFileResource(read_info->filename);
         if (tile_image == (Image *) NULL)
           continue;
@@ -1325,7 +1331,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
         (void) CompositeImage(image,CopyCompositeOp,tile_image,frame.left,
           frame.right);
         image->compression=tile_image->compression;
-        DestroyImage(tile_image);
+        tile_image=DestroyImage(tile_image);
         continue;
       }
     if ((code == 0xff) || (code == 0xffff))
@@ -1674,7 +1680,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
         &image->exception);
       if (blob == (unsigned char *) NULL)
         return(MagickFalse);
-      DestroyImage(jpeg_image);
+      jpeg_image=DestroyImage(jpeg_image);
       (void) WriteBlobMSBShort(image,PictJPEGOp);
       (void) WriteBlobMSBLong(image,length+154);
       (void) WriteBlobMSBShort(image,0x0000);
@@ -1807,9 +1813,11 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
       for (x=0; x < (long) image->columns; x++)
         scanline[x]=indexes[x];
       count+=EncodeImage(image,scanline,row_bytes & 0x7FFF,packed_scanline);
-      if (QuantumTick(y,image->rows) != MagickFalse)
+      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+          (QuantumTick(y,image->rows) != MagickFalse))
         {
-          status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
+          status=image->progress_monitor(SaveImageTag,y,image->rows,
+            image->client_data);
           if (status == MagickFalse)
             break;
         }
@@ -1859,10 +1867,11 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
           }
           count+=EncodeImage(image,scanline,bytes_per_line & 0x7FFF,
             packed_scanline);
-          if (QuantumTick(y,image->rows) != MagickFalse)
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(y,image->rows) != MagickFalse))
             {
-              status=MagickMonitor(SaveImageTag,y,image->rows,
-                &image->exception);
+              status=image->progress_monitor(SaveImageTag,y,image->rows,
+                image->client_data);
               if (status == MagickFalse)
                 break;
             }
