@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,14 +42,17 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/color.h"
 #include "magick/color_private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
 #include "magick/delegate.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/geometry.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -58,12 +61,14 @@
 #include "magick/resource_.h"
 #include "magick/static.h"
 #include "magick/string_.h"
+#include "magick/token.h"
+#include "magick/transform.h"
 #include "magick/utility.h"
 
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WritePSImage(const ImageInfo *,Image *);
 
 /*
@@ -77,17 +82,15 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  IsPS() returns True if the image format type, identified by the
+%  IsPS() returns MagickTrue if the image format type, identified by the
 %  magick string, is PS.
 %
 %  The format of the IsPS method is:
 %
-%      unsigned int IsPS(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsPS(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsPS() returns True if the image format type is PS.
-%
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
 %
@@ -95,15 +98,15 @@ static unsigned int
 %
 %
 */
-static unsigned int IsPS(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsPS(const unsigned char *magick,const size_t length)
 {
   if (length < 4)
-    return(False);
+    return(MagickFalse);
   if (memcmp(magick,"%!",2) == 0)
-    return(True);
+    return(MagickTrue);
   if (memcmp(magick,"\004%!",3) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -127,10 +130,6 @@ static unsigned int IsPS(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadPSImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a memory shortage or
-%      if the image cannot be read.
-%
 %    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
@@ -157,7 +156,7 @@ static inline int ProfileInteger(Image *image,short int *hex_digits)
         break;
       }
     c&=0xff;
-    if (isxdigit(c) == False)
+    if (isxdigit(c) == MagickFalse)
       continue;
     value=(int) ((unsigned long) value << 4)+hex_digits[c];
     i++;
@@ -170,12 +169,10 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
 #define BoundingBox  "BoundingBox:"
 #define BeginDocument  "BeginDocument:"
 #define DocumentMedia  "DocumentMedia:"
-#define DocumentProcessColors  "DocumentProcessColors:"
 #define EndDocument  "EndDocument:"
 #define HiResBoundingBox  "HiResBoundingBox:"
 #define PageBoundingBox  "PageBoundingBox:"
 #define PageMedia  "PageMedia:"
-#define PageProcessColors  "PageProcessColors:"
 #define PhotoshopProfile  "BeginPhotoshop:"
 #define PostscriptLevel  "%!PS-"
 #define RenderPostscriptText  "  Rendering postscript...  "
@@ -183,11 +180,12 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   char
     command[MaxTextExtent],
     density[MaxTextExtent],
-    *escape_filename,
     filename[MaxTextExtent],
     geometry[MaxTextExtent],
     options[MaxTextExtent],
     postscript_filename[MaxTextExtent],
+    *q,
+    token[MaxTextExtent],
     translate_geometry[MaxTextExtent];
 
   const DelegateInfo
@@ -197,9 +195,6 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     dx_resolution,
     dy_resolution;
 
-  FILE
-    *file;
-
   GeometryInfo
     geometry_info;
 
@@ -212,7 +207,15 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   int
     c,
-    unique_file;
+    file;
+
+  MagickBooleanType
+    cmyk,
+    skip,
+    status;
+
+  MagickStatusType
+    flags;
 
   RectangleInfo
     page;
@@ -233,13 +236,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     count;
 
   StringInfo
-    *bim_profile;
-
-  unsigned int
-    cmyk,
-    flags,
-    skip,
-    status;
+    *iptc_profile;
 
   unsigned long
     height,
@@ -251,32 +248,17 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
-  if (image_info->debug != False)
+  if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
+  if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
-  /*
-    Open temporary output file.
-  */
-  file=(FILE *) NULL;
-  unique_file=AcquireUniqueFileResource(postscript_filename);
-  if (unique_file != -1)
-    file=fdopen(unique_file,"wb");
-  if ((unique_file == -1) || (file == (FILE *) NULL))
-    {
-      (void) CopyMagickString(image->filename,postscript_filename,
-        MaxTextExtent);
-      ThrowFileException(&image->exception,FileOpenError,"UnableToOpenFile",
-        image->filename);
-      return(False);
-    }
   /*
     Initialize hex values.
   */
@@ -320,11 +302,11 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Determine page geometry from the Postscript bounding box.
   */
-  (void) ResetMagickMemory(&bounds,0,sizeof(SegmentInfo));
-  (void) ResetMagickMemory(&page,0,sizeof(RectangleInfo));
-  bim_profile=(StringInfo *) NULL;
-  skip=False;
-  cmyk=False;
+  (void) ResetMagickMemory(&bounds,0,sizeof(bounds));
+  (void) ResetMagickMemory(&page,0,sizeof(page));
+  iptc_profile=(StringInfo *) NULL;
+  skip=MagickFalse;
+  cmyk=MagickFalse;
   for (p=command; (c=ReadBlobByte(image)) != EOF; )
   {
     if (image_info->page != (char *) NULL)
@@ -341,10 +323,10 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       Skip %%BeginDocument thru %%EndDocument.
     */
     if (LocaleNCompare(BeginDocument,command,strlen(BeginDocument)) == 0)
-      skip=True;
+      skip=MagickTrue;
     if (LocaleNCompare(EndDocument,command,strlen(EndDocument)) == 0)
-      skip=False;
-    if (skip != False)
+      skip=MagickFalse;
+    if (skip != MagickFalse)
       continue;
     if (LocaleNCompare(PhotoshopProfile,command,strlen(PhotoshopProfile)) == 0)
       {
@@ -354,25 +336,33 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
         count=(ssize_t) sscanf(command,"BeginPhotoshop: %lu",&length);
         if (count != 1)
           continue;
-        bim_profile=AcquireStringInfo(length);
+        iptc_profile=AcquireStringInfo(length);
         for (i=0; i < (long) length; i++)
-          bim_profile->datum[i]=(unsigned char)
+          iptc_profile->datum[i]=(unsigned char)
             ProfileInteger(image,hex_digits);
       }
-    cmyk|=LocaleNCompare(DocumentProcessColors,command,
-      strlen(DocumentProcessColors)) == 0;
-    cmyk|=LocaleNCompare(PageProcessColors,command,
-      strlen(PageProcessColors)) == 0;
+    /*
+      Is this a CMYK document?
+    */
+    for (q=p; *q != '\0'; )
+    {
+      GetToken(q,&q,token);
+      if (LocaleCompare(token,"cmyk") == 0)
+        cmyk=MagickTrue;
+    }
     /*
       Note region defined by bounding box.
     */
+    if ((bounds.x1 != 0.0) || (bounds.y1 != 0.0) || (bounds.x2 != 0.0) ||
+        (bounds.y2 != 0.0))
+      continue;
     count=0;
     if (LocaleNCompare(BoundingBox,command,strlen(BoundingBox)) == 0)
-      count=(ssize_t) sscanf(command,"BoundingBox: %lf %lf %lf %lf",
-        &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
+      count=(ssize_t) sscanf(command,"BoundingBox: %lf %lf %lf %lf",&bounds.x1,
+        &bounds.y1,&bounds.x2,&bounds.y2);
     if (LocaleNCompare(DocumentMedia,command,strlen(DocumentMedia)) == 0)
-      count=(ssize_t) sscanf(command,"DocumentMedia: %*s %lf %lf",
-        &bounds.x2,&bounds.y2)+2;
+      count=(ssize_t) sscanf(command,"DocumentMedia: %*s %lf %lf",&bounds.x2,
+        &bounds.y2)+2;
     if (LocaleNCompare(HiResBoundingBox,command,strlen(HiResBoundingBox)) == 0)
       count=(ssize_t) sscanf(command,"HiResBoundingBox: %lf %lf %lf %lf",
         &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
@@ -380,12 +370,10 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       count=(ssize_t) sscanf(command,"PageBoundingBox: %lf %lf %lf %lf",
         &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
     if (LocaleNCompare(PageMedia,command,strlen(PageMedia)) == 0)
-      count=(ssize_t) sscanf(command,"PageMedia: %*s %lf %lf",
-        &bounds.x2,&bounds.y2)+2;
+      count=(ssize_t) sscanf(command,"PageMedia: %*s %lf %lf",&bounds.x2,
+        &bounds.y2)+2;
     if (count != 4)
       continue;
-    if ((bounds.x1 > bounds.x2) || (bounds.y1 > bounds.y2))
-      continue;
     /*
       Set Postscript render geometry.
     */
@@ -396,13 +384,25 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (height > page.height)
       page.height=height;
   }
+  CloseBlob(image);
+  /*
+    Create Ghostscript control file.
+  */
+  file=AcquireUniqueFileResource(postscript_filename);
+  if (file == -1)
+    {
+      ThrowFileException(&image->exception,FileOpenError,"UnableToOpenFile",
+        image_info->filename);
+      image=DestroyImageList(image);
+      return((Image *) NULL);
+    }
+  (void) CopyMagickString(command,"/setpagedevice {pop} bind 1 index where {"
+    "dup wcheck {3 1 roll put} {pop def} ifelse} {def} ifelse\n",MaxTextExtent);
+  (void) write(file,command,strlen(command));
   (void) FormatMagickString(translate_geometry,MaxTextExtent,
     "%g %g translate\n",-bounds.x1,-bounds.y1);
-  (void) fputs(translate_geometry,file);
-  escape_filename=EscapeString(image_info->filename,'\'');
-  (void) fprintf(file,"(%s) run\n",escape_filename);
-  escape_filename=(char *) RelinquishMagickMemory(escape_filename);
-  (void) fclose(file);
+  (void) write(file,translate_geometry,strlen(translate_geometry));
+  file=close(file)-1;
   /*
     Render postscript with the Ghostscript delegate.
   */
@@ -414,14 +414,13 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     (page.width*image->x_resolution/dx_resolution+0.5);
   page.height=(unsigned long)
     (page.height*image->y_resolution/dy_resolution+0.5);
+  image=DestroyImageList(image);
   (void) FormatMagickString(geometry,MaxTextExtent,"%lux%lu",
     page.width,page.height);
-  CloseBlob(image);
-  DestroyImage(image);
-  if (image_info->monochrome != False)
+  if (image_info->monochrome != MagickFalse)
     delegate_info=GetDelegateInfo("gs-mono",(char *) NULL,exception);
   else
-    if (cmyk != False)
+    if (cmyk != MagickFalse)
       delegate_info=GetDelegateInfo("gs-cmyk",(char *) NULL,exception);
     else
       delegate_info=GetDelegateInfo("gs-color",(char *) NULL,exception);
@@ -432,50 +431,77 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
   *options='\0';
   read_info=CloneImageInfo(image_info);
+  *read_info->magick='\0';
   if (read_info->number_scenes != 0)
-    (void) FormatMagickString(options,MaxTextExtent,
-      "-dFirstPage=%lu -dLastPage=%lu",read_info->scene+1,read_info->scene+
-      read_info->number_scenes);
+    {
+      if (read_info->number_scenes != 1)
+        (void) FormatMagickString(options,MaxTextExtent,"-dLastPage=%lu",
+          read_info->scene+read_info->number_scenes);
+      else
+        (void) FormatMagickString(options,MaxTextExtent,
+          "-dFirstPage=%lu -dLastPage=%lu",read_info->scene+1,
+          read_info->scene+read_info->number_scenes);
+      read_info->number_scenes=0;
+      if (read_info->scenes != (char *) NULL)
+        *read_info->scenes='\0';
+    }
   (void) CopyMagickString(filename,read_info->filename,MaxTextExtent);
   (void) AcquireUniqueFilename(read_info->filename);
   (void) FormatMagickString(command,MaxTextExtent,
-    GetDelegateCommands(delegate_info),read_info->antialias != False ? 4 : 1,
-    read_info->antialias != False ? 4 : 1,geometry,density,options,
-    read_info->filename,postscript_filename);
-  (void) MagickMonitor(RenderPostscriptText,0,8,&image->exception);
+    GetDelegateCommands(delegate_info),
+    read_info->antialias != MagickFalse ? 4 : 1,
+    read_info->antialias != MagickFalse ? 4 : 1,geometry,density,options,
+    read_info->filename,postscript_filename,image_info->filename);
   status=InvokePostscriptDelegate(read_info->verbose,command);
-  *read_info->magick='\0';
   image=ReadImage(read_info,exception);
+  if (image == (Image *) NULL)
+    {
+      (void) ConcatenateMagickString(command," -c showpage",MaxTextExtent);
+      status=InvokePostscriptDelegate(read_info->verbose,command);
+      *read_info->magick='\0';
+      image=ReadImage(read_info,exception);
+    }
   (void) RelinquishUniqueFileResource(postscript_filename);
-  (void) MagickMonitor(RenderPostscriptText,7,8,&image->exception);
   (void) RelinquishUniqueFileResource(read_info->filename);
   if (image == (Image *) NULL)
     {
       /*
         Ghostscript has failed-- try the Display Postscript Extension.
       */
-      (void) FormatMagickString(read_info->filename,MaxTextExtent,
-        "dps:%s",filename);
+      (void) FormatMagickString(read_info->filename,MaxTextExtent,"dps:%s",
+        filename);
       image=ReadImage(read_info,exception);
       if (image != (Image *) NULL)
         return(GetFirstImageInList(image));
       ThrowReaderException(DelegateError,"PostscriptDelegateFailed");
     }
-  DestroyImageInfo(read_info);
+  read_info=DestroyImageInfo(read_info);
   if (image == (Image *) NULL)
     ThrowReaderException(DelegateError,"PostscriptDelegateFailed");
+  if (LocaleCompare(image->magick,"BMP") == 0)
+    {
+      Image
+        *cmyk_image;
+
+      cmyk_image=ConsolidateCMYKImages(image,&image->exception);
+      if (cmyk_image != (Image *) NULL)
+        {
+          image=DestroyImageList(image);
+          image=cmyk_image;
+        }
+    }
   do
   {
     (void) CopyMagickString(image->filename,filename,MaxTextExtent);
-    if (bim_profile != (StringInfo *) NULL)
-      (void) SetImageProfile(image,"8bim",bim_profile);
+    if (iptc_profile != (StringInfo *) NULL)
+      (void) SetImageProfile(image,"iptc",iptc_profile);
     image->page=page;
     next_image=SyncNextImageInList(image);
     if (next_image != (Image *) NULL)
       image=next_image;
   } while (next_image != (Image *) NULL);
-  if (bim_profile != (StringInfo *) NULL)
-    DestroyStringInfo(bim_profile);
+  if (iptc_profile != (StringInfo *) NULL)
+    iptc_profile=DestroyStringInfo(iptc_profile);
   return(GetFirstImageInList(image));
 }
 
@@ -511,8 +537,9 @@ ModuleExport void RegisterPSImage(void)
   entry->decoder=(DecoderHandler *) ReadPSImage;
   entry->encoder=(EncoderHandler *) WritePSImage;
   entry->magick=(MagickHandler *) IsPS;
-  entry->adjoin=False;
-  entry->blob_support=False;
+  entry->adjoin=MagickFalse;
+  entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=
     AcquireString("Encapsulated PostScript Interchange format");
   entry->module=AcquireString("PS");
@@ -521,8 +548,9 @@ ModuleExport void RegisterPSImage(void)
   entry->decoder=(DecoderHandler *) ReadPSImage;
   entry->encoder=(EncoderHandler *) WritePSImage;
   entry->magick=(MagickHandler *) IsPS;
-  entry->adjoin=False;
-  entry->blob_support=False;
+  entry->adjoin=MagickFalse;
+  entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=AcquireString("Encapsulated PostScript");
   entry->module=AcquireString("PS");
   (void) RegisterMagickInfo(entry);
@@ -530,8 +558,9 @@ ModuleExport void RegisterPSImage(void)
   entry->decoder=(DecoderHandler *) ReadPSImage;
   entry->encoder=(EncoderHandler *) WritePSImage;
   entry->magick=(MagickHandler *) IsPS;
-  entry->adjoin=False;
-  entry->blob_support=False;
+  entry->adjoin=MagickFalse;
+  entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=AcquireString("Encapsulated PostScript");
   entry->module=AcquireString("PS");
   (void) RegisterMagickInfo(entry);
@@ -539,8 +568,9 @@ ModuleExport void RegisterPSImage(void)
   entry->decoder=(DecoderHandler *) ReadPSImage;
   entry->encoder=(EncoderHandler *) WritePSImage;
   entry->magick=(MagickHandler *) IsPS;
-  entry->adjoin=False;
-  entry->blob_support=False;
+  entry->adjoin=MagickFalse;
+  entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=
     AcquireString("Encapsulated PostScript Interchange format");
   entry->module=AcquireString("PS");
@@ -550,7 +580,8 @@ ModuleExport void RegisterPSImage(void)
   entry->encoder=(EncoderHandler *) WritePSImage;
   entry->magick=(MagickHandler *) IsPS;
   entry->module=AcquireString("PS");
-  entry->blob_support=False;
+  entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=AcquireString("PostScript");
   (void) RegisterMagickInfo(entry);
 }
@@ -601,31 +632,27 @@ ModuleExport void UnregisterPSImage(void)
 %
 %  The format of the WritePSImage method is:
 %
-%      unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows:
 %
-%    o status: Method WritePSImage return True if the image is printed.
-%      False is returned if the image file cannot be opened for printing.
-%
 %    o image_info: The image info.
 %
-%    o image: The address of a structure of type Image;  returned from
-%      ReadImage.
+%    o image: The image.
 %
 %
 */
-static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
 {
 #define WriteRunlengthPacket(image,pixel,length,p) \
 { \
-  if ((image->matte != False) && (p->opacity == TransparentOpacity)) \
+  if ((image->matte != MagickFalse) && (p->opacity == TransparentOpacity)) \
     (void) FormatMagickString(buffer,MaxTextExtent,"ffffff%02lX", \
-      Min(length,0xff)); \
+      (unsigned long) Min(length,0xff)); \
   else \
     (void) FormatMagickString(buffer,MaxTextExtent,"%02X%02X%02X%02lX", \
       ScaleQuantumToChar(pixel.red),ScaleQuantumToChar(pixel.green), \
-      ScaleQuantumToChar(pixel.blue),Min(length,0xff)); \
+      ScaleQuantumToChar(pixel.blue),(unsigned long) Min(length,0xff)); \
   (void) WriteBlobString(image,buffer); \
 }
 
@@ -918,9 +945,15 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
     j,
     y;
 
+  MagickBooleanType
+    status;
+
   MagickOffsetType
     scene;
 
+  MagickStatusType
+    flags;
+
   PixelPacket
     pixel;
 
@@ -951,10 +984,6 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
   time_t
     timer;
 
-  unsigned int
-    flags,
-    status;
-
   unsigned long
     bit,
     byte,
@@ -968,26 +997,45 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  if (image->debug != False)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
+  if (status == MagickFalse)
     return(status);
+  (void) ResetMagickMemory(&bounds,0,sizeof(bounds));
   page=1;
   scene=0;
   do
   {
     /*
-      Scale image to size of Postscript page.
+      Scale relative to dots-per-inch.
     */
     (void) SetImageColorspace(image,RGBColorspace);
-    text_size=0;
-    attribute=GetImageAttribute(image,"label");
-    if (attribute != (const ImageAttribute *) NULL)
-      text_size=(unsigned int)
-        (MultilineCensus(attribute->value)*image_info->pointsize+12);
+    dx_resolution=72.0;
+    dy_resolution=72.0;
+    flags=ParseGeometry(PSDensityGeometry,&geometry_info);
+    x_resolution=geometry_info.rho;
+    y_resolution=geometry_info.sigma;
+    if ((flags & SigmaValue) == 0)
+      y_resolution=x_resolution;
+    if (image->x_resolution != 0.0)
+      x_resolution=image->x_resolution;
+    if (image->y_resolution != 0.0)
+      y_resolution=image->y_resolution;
+    if (image_info->density != (char *) NULL)
+      {
+        flags=ParseGeometry(image_info->density,&geometry_info);
+        x_resolution=geometry_info.rho;
+        y_resolution=geometry_info.sigma;
+        if ((flags & SigmaValue) == 0)
+          y_resolution=x_resolution;
+      }
+    if (image->units == PixelsPerCentimeterResolution)
+      {
+        x_resolution*=2.54;
+        y_resolution*=2.54;
+      }
     SetGeometry(image,&geometry);
-    geometry.y=(long) text_size;
     (void) FormatMagickString(page_geometry,MaxTextExtent,"%lux%lu",
       image->columns,image->rows);
     if (image_info->page != (char *) NULL)
@@ -1002,38 +1050,22 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
           (void) strcpy(page_geometry,PSPageGeometry);
     (void) ParseMetaGeometry(page_geometry,&geometry.x,&geometry.y,
       &geometry.width,&geometry.height);
-    (void) ParseAbsoluteGeometry(page_geometry,&media_info);
-    (void) ParseGravityGeometry(image,page_geometry,&page_info);
-    geometry.x=(-page_info.x);
-    geometry.y=(long) (media_info.height+page_info.y-image->rows);
-    /*
-      Scale relative to dots-per-inch.
-    */
-    dx_resolution=72.0;
-    dy_resolution=72.0;
-    x_resolution=72.0;
-    flags=ParseGeometry(PSDensityGeometry,&geometry_info);
-    x_resolution=geometry_info.rho;
-    y_resolution=geometry_info.sigma;
-    if ((flags & SigmaValue) == 0)
-      y_resolution=x_resolution;
-    if (image_info->density != (char *) NULL)
-      {
-        flags=ParseGeometry(image_info->density,&geometry_info);
-        x_resolution=geometry_info.rho;
-        y_resolution=geometry_info.sigma;
-        if ((flags & SigmaValue) == 0)
-          y_resolution=x_resolution;
-      }
-    if (image->units == PixelsPerCentimeterResolution)
-      {
-        x_resolution*=2.54;
-        y_resolution*=2.54;
-      }
     x_scale=(double) (geometry.width*dx_resolution)/x_resolution;
     geometry.width=(unsigned long) (x_scale+0.5);
     y_scale=(double) (geometry.height*dy_resolution)/y_resolution;
     geometry.height=(unsigned long) (y_scale+0.5);
+    (void) ParseAbsoluteGeometry(page_geometry,&media_info);
+    (void) ParseGravityGeometry(image,page_geometry,&page_info);
+    if (image->gravity != UndefinedGravity)
+      {
+        geometry.x=(-page_info.x);
+	    	geometry.y=(long) (media_info.height+page_info.y-image->rows);
+      }
+    text_size=0;
+    attribute=GetImageAttribute(image,"label");
+    if (attribute != (const ImageAttribute *) NULL)
+      text_size=(unsigned long)
+        (MultilineCensus(attribute->value)*image_info->pointsize+12);
     count=0;
     if (page == 1)
       {
@@ -1060,7 +1092,8 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
         bounds.y1=(double) geometry.y;
         bounds.x2=(double) geometry.x+x_scale;
         bounds.y2=(double) geometry.y+(geometry.height+text_size);
-        if ((image_info->adjoin != False) && (image->next != (Image *) NULL))
+        if ((image_info->adjoin != MagickFalse) &&
+            (image->next != (Image *) NULL))
           (void) strcpy(buffer,"%%%%BoundingBox: (atend)\n");
         else
           {
@@ -1090,7 +1123,7 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
             (void) WriteBlobString(image,"%%Orientation: Portrait\n");
             (void) WriteBlobString(image,"%%PageOrder: Ascend\n");
             (void) FormatMagickString(buffer,MaxTextExtent,"%%%%Pages: %lu\n",
-              image_info->adjoin != False ? (unsigned long)
+              image_info->adjoin != MagickFalse ? (unsigned long)
               GetImageListLength(image) : 1UL);
             (void) WriteBlobString(image,buffer);
           }
@@ -1112,7 +1145,7 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
             /*
               Create preview image.
             */
-            preview_image=CloneImage(image,0,0,True,&image->exception);
+            preview_image=CloneImage(image,0,0,MagickTrue,&image->exception);
             if (preview_image == (Image *) NULL)
               ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
             /*
@@ -1149,7 +1182,7 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                 if (bit == 8)
                   {
                     (void) FormatMagickString(buffer,MaxTextExtent,"%02X",
-                      byte & 0xff);
+                      (unsigned int) (byte & 0xff));
                     (void) WriteBlobString(image,buffer);
                     count++;
                     if (count == 36)
@@ -1165,7 +1198,7 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                 {
                   byte<<=(8-bit);
                   (void) FormatMagickString(buffer,MaxTextExtent,"%02X",
-                    byte & 0xff);
+                    (unsigned int) (byte & 0xff));
                   (void) WriteBlobString(image,buffer);
                   count++;
                   if (count == 36)
@@ -1176,7 +1209,7 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                 };
             }
             (void) WriteBlobString(image,"\n%%EndPreview\n");
-            DestroyImage(preview_image);
+            preview_image=DestroyImage(preview_image);
           }
         /*
           Output Postscript commands.
@@ -1249,19 +1282,19 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
         }
         labels=(char **) RelinquishMagickMemory(labels);
       }
-    (void) ResetMagickMemory(&pixel,0,sizeof(PixelPacket));
+    (void) ResetMagickMemory(&pixel,0,sizeof(pixel));
     pixel.opacity=TransparentOpacity;
     i=0;
     index=0;
     x=0;
     if ((image_info->type != TrueColorType) &&
-        (IsGrayImage(image,&image->exception) != False))
+        (IsGrayImage(image,&image->exception) != MagickFalse))
       {
         (void) FormatMagickString(buffer,MaxTextExtent,"%lu %lu\n1\n1\n1\n%d\n",
           image->columns,image->rows,
-          IsMonochromeImage(image,&image->exception) != False ? 1 : 8);
+          IsMonochromeImage(image,&image->exception) != MagickFalse ? 1 : 8);
         (void) WriteBlobString(image,buffer);
-        if (IsMonochromeImage(image,&image->exception) == False)
+        if (IsMonochromeImage(image,&image->exception) == MagickFalse)
           {
             /*
               Dump image as grayscale.
@@ -1287,11 +1320,12 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                 p++;
               }
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(y,image->rows))
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
                   {
-                    status=MagickMonitor(SaveImageTag,y,image->rows,
-                      &image->exception);
-                    if (status == False)
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
+                    if (status == MagickFalse)
                       break;
                   }
             }
@@ -1330,7 +1364,7 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                 if (bit == 8)
                   {
                     (void) FormatMagickString(buffer,MaxTextExtent,"%02X",
-                      byte & 0xff);
+                      (unsigned int) (byte & 0xff));
                     (void) WriteBlobString(image,buffer);
                     count++;
                     if (count == 36)
@@ -1347,7 +1381,7 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                 {
                   byte<<=(8-bit);
                   (void) FormatMagickString(buffer,MaxTextExtent,"%02X",
-                    byte & 0xff);
+                    (unsigned int) (byte & 0xff));
                   (void) WriteBlobString(image,buffer);
                   count++;
                   if (count == 36)
@@ -1357,11 +1391,12 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                     };
                 };
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(y,image->rows))
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
                   {
-                    status=MagickMonitor(SaveImageTag,y,image->rows,
-                      &image->exception);
-                    if (status == False)
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
+                    if (status == MagickFalse)
                       break;
                   }
             }
@@ -1370,7 +1405,7 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
           (void) WriteBlobByte(image,'\n');
       }
     else
-      if ((image->storage_class == DirectClass) || (image->matte != False))
+      if ((image->storage_class == DirectClass) || (image->matte != MagickFalse))
         {
           /*
             Dump DirectClass image.
@@ -1419,13 +1454,14 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                 }
                 WriteRunlengthPacket(image,pixel,length,p);
                 if (image->previous == (Image *) NULL)
-                  if (QuantumTick(y,image->rows))
-                    {
-                      status=MagickMonitor(SaveImageTag,y,image->rows,
-                        &image->exception);
-                      if (status == False)
-                        break;
-                    }
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
+                  {
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
+                    if (status == MagickFalse)
+                      break;
+                  }
               }
               break;
             }
@@ -1444,7 +1480,7 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                   break;
                 for (x=0; x < (long) image->columns; x++)
                 {
-                  if ((image->matte != False) &&
+                  if ((image->matte != MagickFalse) &&
                       (p->opacity == TransparentOpacity))
                     (void) strcpy(buffer,"ffffff");
                   else
@@ -1461,13 +1497,14 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                   p++;
                 }
                 if (image->previous == (Image *) NULL)
-                  if (QuantumTick(y,image->rows))
-                    {
-                      status=MagickMonitor(SaveImageTag,y,image->rows,
-                        &image->exception);
-                      if (status == False)
-                        break;
-                    }
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
+                  {
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
+                    if (status == MagickFalse)
+                      break;
+                  }
               }
               break;
             }
@@ -1524,7 +1561,7 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                       if (x > 0)
                         {
                           (void) FormatMagickString(buffer,MaxTextExtent,
-                            "%02X%02lX",index,Min(length,0xff));
+                            "%02X%02lX",index,(unsigned long) Min(length,0xff));
                           (void) WriteBlobString(image,buffer);
                           i++;
                           if (i == 18)
@@ -1540,16 +1577,17 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                   p++;
                 }
                 (void) FormatMagickString(buffer,MaxTextExtent,"%02X%02lX",
-                  index,Min(length,0xff));
+                  index,(unsigned long) Min(length,0xff));
                 (void) WriteBlobString(image,buffer);
                 if (image->previous == (Image *) NULL)
-                  if (QuantumTick(y,image->rows))
-                    {
-                      status=MagickMonitor(SaveImageTag,y,image->rows,
-                        &image->exception);
-                      if (status == False)
-                        break;
-                    }
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
+                  {
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
+                    if (status == MagickFalse)
+                      break;
+                  }
               }
               break;
             }
@@ -1581,13 +1619,14 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                   p++;
                 }
                 if (image->previous == (Image *) NULL)
-                  if (QuantumTick(y,image->rows))
-                    {
-                      status=MagickMonitor(SaveImageTag,y,image->rows,
-                        &image->exception);
-                      if (status == False)
-                        break;
-                    }
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
+                  {
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
+                    if (status == MagickFalse)
+                      break;
+                  }
               }
               break;
             }
@@ -1600,11 +1639,15 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == False)
-      break;
-  } while (image_info->adjoin != False);
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
+  } while (image_info->adjoin != MagickFalse);
   (void) WriteBlobString(image,"%%Trailer\n");
   if (page > 2)
     {
@@ -1619,5 +1662,5 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
     }
   (void) WriteBlobString(image,"%%EOF\n");
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
