@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -43,13 +43,16 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/color.h"
 #include "magick/color_private.h"
 #include "magick/compress.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/geometry.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -86,7 +89,7 @@
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WritePS3Image(const ImageInfo *,Image *);
 
 /*
@@ -100,12 +103,11 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterPS3Image adds attributes for the PS3 image format to
-%  the list of supported formats.  The attributes include the image format
-%  tag, a method to read and/or write the format, whether the format
-%  supports the saving of more than one frame to the same file or blob,
-%  whether the format supports native in-memory I/O, and a brief
-%  description of the format.
+%  RegisterPS3Image() adds attributes for the PS3 image format to the list of
+%  supported formats.  The attributes include the image format tag, a method to
+%  read and/or write the format, whether the format supports the saving of more
+%  than one frame to the same file or blob, whether the format supports native
+%  in-memory I/O, and a brief description of the format.
 %
 %  The format of the RegisterPS3Image method is:
 %
@@ -140,8 +142,8 @@ ModuleExport void RegisterPS3Image(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterPS3Image removes format registrations made by the
-%  PS3 module from the list of supported formats.
+%  UnregisterPS3Image() removes format registrations made by the PS3 module
+%  from the list of supported formats.
 %
 %  The format of the UnregisterPS3Image method is:
 %
@@ -165,30 +167,26 @@ ModuleExport void UnregisterPS3Image(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WritePS3Image translates an image to encapsulated Postscript
-%  Level III for printing.  If the supplied geometry is null, the image is
-%  centered on the Postscript page.  Otherwise, the image is positioned as
-%  specified by the geometry.
+%  WritePS3Image() translates an image to encapsulated Postscript Level III
+%  for printing.  If the supplied geometry is null, the image is centered on
+%  the Postscript page.  Otherwise, the image is positioned as specified by the
+%  geometry.
 %
 %  The format of the WritePS3Image method is:
 %
-%      unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows:
 %
-%    o status: Method WritePS3Image return True if the image is printed.
-%      False is returned if the image file cannot be opened for printing.
-%
 %    o image_info: Specifies a pointer to a ImageInfo structure.
 %
-%    o image: The address of a structure of type Image;  returned from
-%      ReadImage.
+%    o image: The image.
 %
 %
 */
 
-static unsigned int SerializeImage(const ImageInfo *image_info,Image *image,
-  unsigned char **pixels,size_t *length)
+static MagickBooleanType SerializeImage(const ImageInfo *image_info,
+  Image *image,unsigned char **pixels,size_t *length)
 {
   IndexPacket
     *indexes;
@@ -196,6 +194,9 @@ static unsigned int SerializeImage(const ImageInfo *image_info,Image *image,
   long
     y;
 
+  MagickBooleanType
+    status;
+
   register const PixelPacket
     *p;
 
@@ -205,14 +206,11 @@ static unsigned int SerializeImage(const ImageInfo *image_info,Image *image,
   register unsigned char
     *q;
 
-  unsigned int
-    status;
-
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  if (image->debug != False)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
-  status=True;
+  status=MagickTrue;
   *length=(image->colorspace == CMYKColorspace ? 4 : 3)*
     (size_t) image->columns*image->rows;
   *pixels=(unsigned char *) AcquireMagickMemory(*length);
@@ -243,24 +241,29 @@ static unsigned int SerializeImage(const ImageInfo *image_info,Image *image,
         p++;
       }
     if (image->previous == (Image *) NULL)
-      if (QuantumTick(y,image->rows))
+      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+          (QuantumTick(y,image->rows) != MagickFalse))
         {
-          status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
-          if (status == False)
+          status=image->progress_monitor(SaveImageTag,y,image->rows,
+            image->client_data);
+          if (status == MagickFalse)
             break;
         }
   }
-  if (status == False)
+  if (status == MagickFalse)
     *pixels=(unsigned char *) RelinquishMagickMemory(*pixels);
   return(status);
 }
 
-static unsigned int SerializeImageChannel(const ImageInfo *image_info,
+static MagickBooleanType SerializeImageChannel(const ImageInfo *image_info,
   Image *image,unsigned char **pixels,size_t *length)
 {
   long
     y;
 
+  MagickBooleanType
+    status;
+
   register const PixelPacket
     *p;
 
@@ -278,16 +281,13 @@ static unsigned int SerializeImageChannel(const ImageInfo *image_info,
     pack,
     padded_columns;
 
-  unsigned int
-    status;
-
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  if (image->debug != False)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
-  status=True;
+  status=MagickTrue;
   pack=(unsigned long)
-    (IsMonochromeImage(image,&image->exception) != False ? 8 : 1);
+    (IsMonochromeImage(image,&image->exception) != MagickFalse ? 8 : 1);
   padded_columns=((image->columns+pack-1)/pack)*pack;
   *length=(size_t) padded_columns*image->rows/pack;
   *pixels=(unsigned char *) AcquireMagickMemory(*length);
@@ -324,25 +324,29 @@ static unsigned int SerializeImageChannel(const ImageInfo *image_info,
         }
       }
     if (image->previous == (Image *) NULL)
-      if (QuantumTick(y,image->rows))
+      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+          (QuantumTick(y,image->rows) != MagickFalse))
         {
-          status=MagickMonitor(SaveImageTag,y,image->rows,
-            &image->exception);
-          if (status == False)
+          status=image->progress_monitor(SaveImageTag,y,image->rows,
+            image->client_data);
+          if (status == MagickFalse)
             break;
         }
   }
-  if (status == False)
+  if (status == MagickFalse)
     *pixels=(unsigned char *) RelinquishMagickMemory(*pixels);
   return(status);
 }
 
-static unsigned int SerializeImageIndexes(const ImageInfo *image_info,
+static MagickBooleanType SerializeImageIndexes(const ImageInfo *image_info,
   Image *image,unsigned char **pixels,size_t *length)
 {
   long
     y;
 
+  MagickBooleanType
+    status;
+
   register const PixelPacket
     *p;
 
@@ -355,14 +359,11 @@ static unsigned int SerializeImageIndexes(const ImageInfo *image_info,
   register unsigned char
     *q;
 
-  unsigned int
-    status;
-
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  if (image->debug != False)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
-  status=True;
+  status=MagickTrue;
   *length=(size_t) image->columns*image->rows;
   *pixels=(unsigned char *) AcquireMagickMemory(*length);
   if (*pixels == (unsigned char *) NULL)
@@ -377,19 +378,22 @@ static unsigned int SerializeImageIndexes(const ImageInfo *image_info,
     for (x=0; x < (long) image->columns; x++)
       *q++=(unsigned char) indexes[x];
     if (image->previous == (Image *) NULL)
-      if (QuantumTick(y,image->rows))
-        {
-          status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
-          if (status == False)
-            break;
-        }
+     if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+         (QuantumTick(y,image->rows) != MagickFalse))
+       {
+         status=image->progress_monitor(SaveImageTag,y,image->rows,
+           image->client_data);
+         if (status == MagickFalse)
+           break;
+       }
   }
-  if (status == False)
+  if (status == MagickFalse)
     *pixels=(unsigned char *) RelinquishMagickMemory(*pixels);
   return(status);
 }
 
-static unsigned int WritePS3MaskImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
+  Image *image)
 {
   char
     buffer[MaxTextExtent];
@@ -400,6 +404,9 @@ static unsigned int WritePS3MaskImage(const ImageInfo *image_info,Image *image)
   Image
     *mask_image;
 
+  MagickBooleanType
+    status;
+
   MagickOffsetType
     start,
     stop;
@@ -413,18 +420,20 @@ static unsigned int WritePS3MaskImage(const ImageInfo *image_info,Image *image)
   unsigned char
     *pixels;
 
-  unsigned int
-    status;
+  unsigned long
+    quality;
 
   assert(image_info != (ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  if (image->debug != False)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
-  assert(image->matte != False);
-  status=True;
+  assert(image->matte != MagickFalse);
+  status=MagickTrue;
   compression=image->compression;
+  quality=(unsigned long)
+    (image->quality == UndefinedCompressionQuality ? 75 : image->quality);
   /*
     Note BeginData DSC comment for update later.
   */
@@ -446,6 +455,7 @@ static unsigned int WritePS3MaskImage(const ImageInfo *image_info,Image *image)
         image->columns,image->rows);
     }
     case FaxCompression:
+    case Group4Compression:
     default:
     {
       (void) FormatMagickString(buffer,MaxTextExtent,
@@ -484,17 +494,17 @@ static unsigned int WritePS3MaskImage(const ImageInfo *image_info,Image *image)
   }
   (void) WriteBlobString(image,buffer);
   (void) WriteBlobString(image,"/ReusableStreamDecode filter\n");
-  mask_image=CloneImage(image,0,0,True,&image->exception);
+  mask_image=CloneImage(image,0,0,MagickTrue,&image->exception);
   if (mask_image == (Image *) NULL)
     ThrowWriterException(CoderError,image->exception.reason);
   status=SeparateImageChannel(mask_image,OpacityChannel);
-  if (status == False)
+  if (status == MagickFalse)
     {
-      DestroyImage(mask_image);
-      return(False);
+      mask_image=DestroyImage(mask_image);
+      return(MagickFalse);
     }
   (void) SetImageType(mask_image,BilevelType);
-  mask_image->matte=False;
+  mask_image->matte=MagickFalse;
   pixels=(unsigned char *) NULL;
   length=0;
   switch (compression)
@@ -502,7 +512,7 @@ static unsigned int WritePS3MaskImage(const ImageInfo *image_info,Image *image)
     case NoCompression:
     {
       status=SerializeImageChannel(image_info,mask_image,&pixels,&length);
-      if (status == False)
+      if (status == MagickFalse)
         break;
       Ascii85Initialize(image);
       for (i=0; i < (long) length; i++)
@@ -512,6 +522,7 @@ static unsigned int WritePS3MaskImage(const ImageInfo *image_info,Image *image)
       break;
     }
     case FaxCompression:
+    case Group4Compression:
     default:
     {
       if (LocaleCompare(CCITTParam,"0") == 0)
@@ -528,7 +539,7 @@ static unsigned int WritePS3MaskImage(const ImageInfo *image_info,Image *image)
     case LZWCompression:
     {
       status=SerializeImageChannel(image_info,mask_image,&pixels,&length);
-      if (status == False)
+      if (status == MagickFalse)
         break;
       status=LZWEncodeImage(image,length,pixels);
       pixels=(unsigned char *) RelinquishMagickMemory(pixels);
@@ -537,7 +548,7 @@ static unsigned int WritePS3MaskImage(const ImageInfo *image_info,Image *image)
     case RLECompression:
     {
       status=SerializeImageChannel(image_info,mask_image,&pixels,&length);
-      if (status == False)
+      if (status == MagickFalse)
         break;
       status=PackbitsEncodeImage(image,length,pixels);
       pixels=(unsigned char *) RelinquishMagickMemory(pixels);
@@ -546,15 +557,14 @@ static unsigned int WritePS3MaskImage(const ImageInfo *image_info,Image *image)
     case ZipCompression:
     {
       status=SerializeImageChannel(image_info,mask_image,&pixels,&length);
-      if (status == False)
+      if (status == MagickFalse)
         break;
-      status=ZLIBEncodeImage(image,length,image_info->quality == 0UL ?
-        75UL : image_info->quality,pixels);
+      status=ZLIBEncodeImage(image,length,quality,pixels);
       pixels=(unsigned char *) RelinquishMagickMemory(pixels);
       break;
     }
   }
-  DestroyImage(mask_image);
+  mask_image=DestroyImage(mask_image);
   (void) WriteBlobByte(image,'\n');
   length=(size_t) (TellBlob(image)-stop);
   stop=TellBlob(image);
@@ -569,7 +579,7 @@ static unsigned int WritePS3MaskImage(const ImageInfo *image_info,Image *image)
   return(status);
 }
 
-static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
 {
   static const char
     *PostscriptProlog[]=
@@ -795,11 +805,17 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
   long
     j;
 
+  MagickBooleanType
+    status;
+
   MagickOffsetType
     scene,
     start,
     stop;
 
+  MagickStatusType
+    flags;
+
   RectangleInfo
     geometry,
     media_info,
@@ -820,10 +836,6 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
   unsigned char
     *pixels;
 
-  unsigned int
-    flags,
-    status;
-
   unsigned long
     page,
     pixel,
@@ -836,11 +848,11 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  if (image->debug != False)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
-    return(False);
+  if (status == MagickFalse)
+    return(MagickFalse);
   compression=image->compression;
   switch (compression)
   {
@@ -853,15 +865,6 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
       break;
     }
 #endif
-#if !defined(HasLZW)
-    case LZWCompression:
-    {
-      compression=RLECompression;
-      (void) ThrowMagickException(&image->exception,GetMagickModule(),
-        MissingDelegateError,"LZWEncodingNotEnabled",image->filename);
-      break;
-    }
-#endif
 #if !defined(HasZLIB)
     case ZipCompression:
     {
@@ -874,48 +877,25 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
     default:
       break;
   }
+  (void) ResetMagickMemory(&bounds,0,sizeof(bounds));
   page=0;
   scene=0;
   do
   {
-    /*
-      Scale image to size of Postscript page.
-    */
-    page++;
-    text_size=0;
-    attribute=GetImageAttribute(image,"label");
-    if (attribute != (const ImageAttribute *) NULL)
-      text_size=(unsigned int) (MultilineCensus(attribute->value)*
-        image_info->pointsize+12);
-    SetGeometry(image,&geometry);
-    geometry.y=(long) text_size;
-    (void) FormatMagickString(page_geometry,MaxTextExtent,"%lux%lu",
-      image->columns,image->rows);
-    if (image_info->page != (char *) NULL)
-      (void) CopyMagickString(page_geometry,image_info->page,MaxTextExtent);
-    else
-      if ((image->page.width != 0) && (image->page.height != 0))
-        (void) FormatMagickString(page_geometry,MaxTextExtent,"%lux%lu%+ld%+ld",          image->page.width,image->page.height,image->page.x,image->page.y);
-      else
-        if (LocaleCompare(image_info->magick,"PS3") == 0)
-          (void) strcpy(page_geometry,PSPageGeometry);
-    (void) ParseMetaGeometry(page_geometry,&geometry.x,&geometry.y,
-      &geometry.width,&geometry.height);
-    (void) ParseAbsoluteGeometry(page_geometry,&media_info);
-    (void) ParseGravityGeometry(image,page_geometry,&page_info);
-    geometry.x=(-page_info.x);
-    geometry.y=(long) (media_info.height+page_info.y-image->rows);
     /*
       Scale relative to dots-per-inch.
     */
     dx_resolution=72.0;
     dy_resolution=72.0;
-    x_resolution=72.0;
     flags=ParseGeometry(PSDensityGeometry,&geometry_info);
     x_resolution=geometry_info.rho;
     y_resolution=geometry_info.sigma;
     if ((flags & SigmaValue) == 0)
       y_resolution=x_resolution;
+    if (image->x_resolution != 0.0)
+      x_resolution=image->x_resolution;
+    if (image->y_resolution != 0.0)
+      y_resolution=image->y_resolution;
     if (image_info->density != (char *) NULL)
       {
         flags=ParseGeometry(image_info->density,&geometry_info);
@@ -929,10 +909,38 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
         x_resolution*=2.54;
         y_resolution*=2.54;
       }
-    x_scale=(double) ((geometry.width*dx_resolution)/x_resolution);
+    SetGeometry(image,&geometry);
+    (void) FormatMagickString(page_geometry,MaxTextExtent,"%lux%lu",
+      image->columns,image->rows);
+    if (image_info->page != (char *) NULL)
+      (void) CopyMagickString(page_geometry,image_info->page,MaxTextExtent);
+    else
+      if ((image->page.width != 0) && (image->page.height != 0))
+        (void) FormatMagickString(page_geometry,MaxTextExtent,"%lux%lu%+ld%+ld",
+          image->page.width,image->page.height,image->page.x,image->page.y);
+      else
+        if ((image->gravity != UndefinedGravity) &&
+            (LocaleCompare(image_info->magick,"PS") == 0))
+          (void) strcpy(page_geometry,PSPageGeometry);
+    (void) ParseMetaGeometry(page_geometry,&geometry.x,&geometry.y,
+      &geometry.width,&geometry.height);
+    x_scale=(double) (geometry.width*dx_resolution)/x_resolution;
     geometry.width=(unsigned long) (x_scale+0.5);
-    y_scale=(double) ((geometry.height*dy_resolution)/y_resolution);
+    y_scale=(double) (geometry.height*dy_resolution)/y_resolution;
     geometry.height=(unsigned long) (y_scale+0.5);
+    (void) ParseAbsoluteGeometry(page_geometry,&media_info);
+    (void) ParseGravityGeometry(image,page_geometry,&page_info);
+    if (image->gravity != UndefinedGravity)
+      {
+        geometry.x=(-page_info.x);
+	    	geometry.y=(long) (media_info.height+page_info.y-image->rows);
+      }
+    text_size=0;
+    attribute=GetImageAttribute(image,"label");
+    if (attribute != (const ImageAttribute *) NULL)
+      text_size=(unsigned long)
+        (MultilineCensus(attribute->value)*image_info->pointsize+12);
+    page++;
     if (page == 1)
       {
         /*
@@ -951,7 +959,7 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
         (void) WriteBlobString(image,buffer);
         timer=time((time_t *) NULL);
         (void) localtime(&timer);
-        (void) strncpy(date,ctime(&timer),MaxTextExtent-1);
+        (void) CopyMagickString(date,ctime(&timer),MaxTextExtent);
         date[strlen(date)-1]='\0';
         (void) FormatMagickString(buffer,MaxTextExtent,
           "%%%%CreationDate: %s\n",date);
@@ -960,7 +968,8 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
         bounds.y1=(double) geometry.y;
         bounds.x2=(double) geometry.x+x_scale;
         bounds.y2=(double) geometry.y+y_scale+text_size;
-        if ((image_info->adjoin != False) && (image->next != (Image *) NULL))
+        if ((image_info->adjoin != MagickFalse) &&
+            (image->next != (Image *) NULL))
           {
             (void) WriteBlobString(image,"%%BoundingBox: (atend)\n");
             (void) WriteBlobString(image,"%%HiResBoundingBox: (atend)\n");
@@ -979,7 +988,7 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
               (void) WriteBlobString(image,
                 "%%DocumentProcessColors: Cyan Magenta Yellow Black\n");
             else
-              if (IsGrayImage(image,&image->exception) != False)
+              if (IsGrayImage(image,&image->exception) != MagickFalse)
                 (void) WriteBlobString(image,
                   "%%DocumentProcessColors: Black\n");
           }
@@ -1000,7 +1009,7 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
           {
             (void) WriteBlobString(image,"%%Orientation: Portrait\n");
             (void) WriteBlobString(image,"%%PageOrder: Ascend\n");
-            if (image_info->adjoin == False)
+            if (image_info->adjoin == MagickFalse)
               (void) strcpy(buffer,"%%Pages: 1\n");
             else
               (void) FormatMagickString(buffer,MaxTextExtent,"%%%%Pages: %lu\n",
@@ -1060,7 +1069,7 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
       (void) WriteBlobString(image,
         "%%PageProcessColors: Cyan Magenta Yellow Black\n");
     else
-      if (IsGrayImage(image,&image->exception) != False)
+      if (IsGrayImage(image,&image->exception) != MagickFalse)
         (void) WriteBlobString(image,"%%PageProcessColors: Black\n");
     /*
       Adjust document bounding box to bound page bounding box.
@@ -1092,7 +1101,7 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
 
         attribute=GetImageAttribute(image,image->clip_mask->magick_filename);
         if (attribute == (const ImageAttribute *) NULL)
-          return(False);
+          return(MagickFalse);
         (void) WriteBlobString(image,attribute->value);
         (void) WriteBlobByte(image,'\n');
       }
@@ -1105,10 +1114,10 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
       Image mask.
     */
     if ((LocaleCompare(image_info->magick,"EPS3") != 0) && image->matte)
-      if (WritePS3MaskImage(image_info,image) == False)
+      if (WritePS3MaskImage(image_info,image) == MagickFalse)
         {
           CloseBlob(image);
-          return(False);
+          return(MagickFalse);
         }
     /*
       Remember position of BeginData comment so we can update it.
@@ -1163,7 +1172,7 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
         (void) WriteBlobString(image,"true\n");
       else
         (void) WriteBlobString(image,"false\n");
-    if (compression == FaxCompression)
+    if ((compression == FaxCompression) || (compression == Group4Compression))
       (void) SetImageType(image,BilevelType);
     /*
       Showpage for non-EPS.
@@ -1199,8 +1208,8 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
         (image_info->type != ColorSeparationType) &&
         (image_info->type != ColorSeparationMatteType) &&
         (image->colorspace != CMYKColorspace) &&
-        ((IsGrayImage(image,&image->exception) != False) ||
-         (IsMonochromeImage(image,&image->exception) != False)))
+        ((IsGrayImage(image,&image->exception) != MagickFalse) ||
+         (IsMonochromeImage(image,&image->exception) != MagickFalse)))
       {
         /*
           Gray images.
@@ -1215,6 +1224,7 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
             break;
           }
           case FaxCompression:
+          case Group4Compression:
           {
             (void) WriteBlobString(image,PS3_FaxCompression"\n");
             break;
@@ -1248,7 +1258,7 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
           1 bit or 8 bit components?
         */
         (void) FormatMagickString(buffer,MaxTextExtent,"%d\n",
-          IsMonochromeImage(image,&image->exception) != False ? 1 : 8);
+          IsMonochromeImage(image,&image->exception) != MagickFalse ? 1 : 8);
         (void) WriteBlobString(image,buffer);
         /*
           Image data.
@@ -1256,7 +1266,8 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
         if (compression == JPEGCompression)
           status=JPEGEncodeImage(image_info,image);
         else
-          if (compression == FaxCompression)
+          if ((compression == FaxCompression) ||
+              (compression == Group4Compression))
            {
              if (LocaleCompare(CCITTParam,"0") == 0)
                status=HuffmanEncodeImage(image_info,image);
@@ -1266,10 +1277,10 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
         else
           {
             status=SerializeImageChannel(image_info,image,&pixels,&length);
-            if (status == False)
+            if (status == MagickFalse)
               {
                 CloseBlob(image);
-                return(False);
+                return(MagickFalse);
               }
             switch (compression)
             {
@@ -1292,8 +1303,7 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
               }
               case ZipCompression:
               {
-                status=ZLIBEncodeImage(image,length,image_info->quality == 0UL ?
-                  75UL : image_info->quality,pixels);
+                status=ZLIBEncodeImage(image,length,image->quality,pixels);
                 break;
               }
             }
@@ -1348,10 +1358,10 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
                 Stream based compressions.
               */
               status=SerializeImage(image_info,image,&pixels,&length);
-              if (status == False)
+              if (status == MagickFalse)
                 {
                   CloseBlob(image);
-                  return(False);
+                  return(MagickFalse);
                 }
               switch (compression)
               {
@@ -1362,7 +1372,7 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
                   for (i=0; i < (long) length; i++)
                     Ascii85Encode(image,pixels[i]);
                   Ascii85Flush(image);
-                  status=True;
+                  status=MagickTrue;
                   break;
                 }
                 case RLECompression:
@@ -1377,8 +1387,7 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
                 }
                 case ZipCompression:
                 {
-                  status=ZLIBEncodeImage(image,length,image_info->quality ==
-                    0UL ? 75UL : image_info->quality,pixels);
+                  status=ZLIBEncodeImage(image,length,image->quality,pixels);
                   break;
                 }
               }
@@ -1456,10 +1465,10 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
                 Ascii85Flush(image);
               }
             status=SerializeImageIndexes(image_info,image,&pixels,&length);
-            if (status == False)
+            if (status == MagickFalse)
               {
                 CloseBlob(image);
-                return(False);
+                return(MagickFalse);
               }
             switch (compression)
             {
@@ -1470,7 +1479,7 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
                 for (i=0; i < (long) length; i++)
                   Ascii85Encode(image,pixels[i]);
                 Ascii85Flush(image);
-                status=True;
+                status=MagickTrue;
                 break;
               }
               case JPEGCompression:
@@ -1490,18 +1499,17 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
               }
               case ZipCompression:
               {
-                status=ZLIBEncodeImage(image,length,image_info->quality == 0UL ?
-                  75UL : image_info->quality,pixels);
+                status=ZLIBEncodeImage(image,length,image->quality,pixels);
                 break;
               }
             }
             pixels=(unsigned char *) RelinquishMagickMemory(pixels);
           }
     (void) WriteBlobByte(image,'\n');
-    if (status == False)
+    if (status == MagickFalse)
       {
         CloseBlob(image);
-        return(False);
+        return(MagickFalse);
       }
     /*
       Update BeginData now that we know the data size.
@@ -1524,12 +1532,15 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene,GetImageListLength(image),
-      &image->exception);
-    if (status == False)
-      break;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
     scene++;
-  } while (image_info->adjoin != False);
+  } while (image_info->adjoin != MagickFalse);
   (void) WriteBlobString(image,"%%Trailer\n");
   if (page > 1)
     {
@@ -1544,5 +1555,5 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
     }
   (void) WriteBlobString(image,"%%EOF\n");
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
