@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -43,10 +43,13 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/log.h"
 #include "magick/magick.h"
@@ -58,7 +61,7 @@
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WritePSDImage(const ImageInfo *,Image *);
 
 /*
@@ -77,13 +80,10 @@ static unsigned int
 %
 %  The format of the DecodeImage method is:
 %
-%      unsigned int DecodeImage(Image *image,const long channel)
+%      MagickBooleanType DecodeImage(Image *image,const long channel)
 %
 %  A description of each parameter follows:
 %
-%    o status: Method DecodeImage return True if the image is
-%      decoded.  False is returned if there is an error occurs.
-%
 %    o image,image: The address of a structure of type Image.
 %
 %    o channel:  Specifies which channel: red, green, blue, or index to
@@ -91,7 +91,7 @@ static unsigned int
 %
 %
 */
-static unsigned int DecodeImage(Image *image,const long channel)
+static MagickBooleanType DecodeImage(Image *image,const long channel)
 {
   ssize_t
     count;
@@ -177,7 +177,7 @@ static unsigned int DecodeImage(Image *image,const long channel)
             default:
               break;
           }
-          if (SyncImagePixels(image) == False)
+          if (SyncImagePixels(image) == MagickFalse)
             break;
           x++;
           number_pixels--;
@@ -239,7 +239,7 @@ static unsigned int DecodeImage(Image *image,const long channel)
         default:
           break;
       }
-      if (SyncImagePixels(image) == False)
+      if (SyncImagePixels(image) == MagickFalse)
         break;
       x++;
       number_pixels--;
@@ -255,7 +255,7 @@ static unsigned int DecodeImage(Image *image,const long channel)
     if (number_pixels < 0)
       ThrowBinaryException(CorruptImageError,"TooMuchImageDataInFile",
         image->filename);
-  return(True);
+  return(MagickTrue);
 }
 
 /*
@@ -269,17 +269,15 @@ static unsigned int DecodeImage(Image *image,const long channel)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  IsPSD()() returns True if the image format type, identified by the magick
+%  IsPSD()() returns MagickTrue if the image format type, identified by the magick
 %  string, is PSD.
 %
 %  The format of the IsPSD method is:
 %
-%      unsigned int IsPSD(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsPSD(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsPSD() returns True if the image format type is PSD.
-%
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
 %
@@ -287,13 +285,13 @@ static unsigned int DecodeImage(Image *image,const long channel)
 %
 %
 */
-static unsigned int IsPSD(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsPSD(const unsigned char *magick,const size_t length)
 {
   if (length < 4)
-    return(False);
+    return(MagickFalse);
   if (LocaleNCompare((char *) magick,"8BPS",4) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -307,8 +305,8 @@ static unsigned int IsPSD(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  ReadPSDImage() reads an Adobe Photoshop image file and returns it.
-%  It allocates the memory necessary for the new Image structure and returns a
+%  ReadPSDImage() reads an Adobe Photoshop image file and returns it.  It
+%  allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.
 %
 %  The format of the ReadPSDImage method is:
@@ -317,10 +315,6 @@ static unsigned int IsPSD(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadPSDImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a memory shortage or
-%      if the image cannot be read.
-%
 %    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
@@ -423,20 +417,21 @@ typedef enum
 
 static char* ModeToString( PSDImageType inType )
 {
-  switch ( inType ) {
-    case BitmapMode:    return "Bitmap";
-    case GrayscaleMode:    return "Grayscale";
-    case IndexedMode:    return "Indexed";
-    case RGBMode:      return "RGB";
-    case CMYKMode:      return "CMYK";
-    case MultichannelMode:  return "Multichannel";
-    case DuotoneMode:    return "Duotone";
-    case LabMode:      return "L*A*B";
-    default:        return "unknown";
+  switch ( inType )
+  {
+    case BitmapMode: return "Bitmap";
+    case GrayscaleMode: return "Grayscale";
+    case IndexedMode: return "Indexed";
+    case RGBMode: return "RGB";
+    case CMYKMode:  return "CMYK";
+    case MultichannelMode: return "Multichannel";
+    case DuotoneMode: return "Duotone";
+    case LabMode: return "L*A*B";
+    default: return "unknown";
   }
 }
 
-static unsigned int ParseImageResourceBlocks(Image *image,
+static MagickBooleanType ParseImageResourceBlocks(Image *image,
   const unsigned char *blocks,size_t length)
 {
 #define PushCharPixel(pixel,p) \
@@ -469,8 +464,8 @@ static unsigned int ParseImageResourceBlocks(Image *image,
 
   profile=AcquireStringInfo(length);
   SetStringInfoDatum(profile,blocks);
-  (void) SetImageProfile(image,"8bim",profile);
-  DestroyStringInfo(profile);
+  (void) SetImageProfile(image,"iptc",profile);
+  profile=DestroyStringInfo(profile);
   p=blocks;
   while (p < (blocks+length))
   {
@@ -511,7 +506,7 @@ static unsigned int ParseImageResourceBlocks(Image *image,
     if ((count & 0x01) != 0)
       p++;
   }
-  return(True);
+  return(MagickTrue);
 }
 
 static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
@@ -618,6 +613,9 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   ssize_t
     count;
 
+  MagickBooleanType
+    status;
+
   MagickOffsetType
     diff_offset,
     offset;
@@ -628,9 +626,6 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   unsigned char
     *data;
 
-  unsigned int
-    status;
-
   unsigned short
     compression;
 
@@ -644,19 +639,19 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
-  if (image_info->debug != False)
+  if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
+  if (status == MagickFalse)
     {
-      if (image->debug != False)
+      if (image->debug != MagickFalse)
         (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  OpenBlob failed");
       ThrowFileException(&image->exception,FileOpenError,"UnableToOpenFile",
         image->filename);
-      return(False);
+      return((Image *) NULL);
     }
   /*
     Read image header.
@@ -665,7 +660,7 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   psd_info.version=ReadBlobMSBShort(image);
   if ((count == 0) || (LocaleNCompare(psd_info.signature,"8BPS",4) != 0) ||
       (psd_info.version != 1)) {
-      if (image->debug != False)
+      if (image->debug != MagickFalse)
         (void) LogMagickEvent(CoderEvent,GetMagickModule(),
           "  File signature was %.4s instead of '8BPS'",psd_info.signature);
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
@@ -677,7 +672,7 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   psd_info.depth=ReadBlobMSBShort(image);
   psd_info.mode=ReadBlobMSBShort(image);
 
-  if (image->debug != False)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(CoderEvent,GetMagickModule(),
       "  Image is %ld x %ld with channels=%d, depth=%d, mode=%s",
       psd_info.columns, psd_info.rows, psd_info.channels,
@@ -689,30 +684,30 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   image->columns=psd_info.columns;
   image->rows=psd_info.rows;
   SetImage(image,OpaqueOpacity);
-  image->matte=psd_info.channels >= 4;
+  image->matte=psd_info.channels >= 4 ? MagickTrue : MagickFalse;
   if (psd_info.mode == LabMode)
     image->colorspace=LABColorspace;
   if (psd_info.mode == CMYKMode)
     {
       image->colorspace=CMYKColorspace;
-      image->matte=psd_info.channels >= 5;
+      image->matte=psd_info.channels >= 5 ? MagickTrue : MagickFalse;
     }
   if ((psd_info.mode == BitmapMode) ||
       (psd_info.mode == GrayscaleMode) ||
       (psd_info.mode == DuotoneMode))
     {
-      if (AllocateImageColormap(image,256) == False) {
-      if (image->debug != False)
+      if (AllocateImageColormap(image,256) == MagickFalse) {
+      if (image->debug != MagickFalse)
         (void) LogMagickEvent(CoderEvent,GetMagickModule(),
           "  allocation of ImageColorMap failed");
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     }
-      image->matte=psd_info.channels >= 2;
-    if (image->debug != False)
+    image->matte=psd_info.channels >= 2 ? MagickTrue : MagickFalse;
+    if (image->debug != MagickFalse)
       (void) LogMagickEvent(CoderEvent,GetMagickModule(), "  ImageColorMap allocated");
     }
 
-  if (image->debug != False)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(CoderEvent,GetMagickModule(),
       image->matte ? "  image has matte" : "  image has no matte");
   /*
@@ -721,7 +716,7 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   length=ReadBlobMSBLong(image);
   if (length != 0)
     {
-    if (image->debug != False)
+    if (image->debug != MagickFalse)
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  reading colormap");
 
       if (psd_info.mode == DuotoneMode)
@@ -731,7 +726,7 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
           */
           data=(unsigned char *) AcquireMagickMemory(length);
           if (data == (unsigned char *) NULL) {
-        if (image->debug != False)
+        if (image->debug != MagickFalse)
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  allocation of ImageColorMap failed");
             ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       }
@@ -743,8 +738,8 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
           /*
             Read PSD raster colormap.
           */
-          if (AllocateImageColormap(image,length/3) == False) {
-        if (image->debug != False)
+          if (AllocateImageColormap(image,length/3) == MagickFalse) {
+        if (image->debug != MagickFalse)
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  allocation of ImageColorMap failed");
             ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       }
@@ -754,7 +749,7 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
           image->colormap[i].green=ScaleCharToQuantum(ReadBlobByte(image));
         for (i=0; i < (long) image->colors; i++)
           image->colormap[i].blue=ScaleCharToQuantum(ReadBlobByte(image));
-        image->matte=psd_info.channels >= 2;
+        image->matte=psd_info.channels >= 2 ? MagickTrue : MagickFalse;
       }
     }
   length=ReadBlobMSBLong(image);
@@ -766,7 +761,7 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       /*
         Image resources block.
       */
-      if (image->debug != False)
+      if (image->debug != MagickFalse)
         (void) LogMagickEvent(CoderEvent,GetMagickModule(),
           "  reading image resource blocks - %ld bytes", (long) length);
       blocks=(unsigned char *) AcquireMagickMemory(length);
@@ -785,11 +780,11 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     If we are only "pinging" the image, then we're done - so return.
   */
-  if (image_info->ping != False)
+  if (image_info->ping != MagickFalse)
     {
       CloseBlob(image);
 
-      if (image->debug != False)
+      if (image->debug != MagickFalse)
         (void) LogMagickEvent(CoderEvent,GetMagickModule(),
           "  pinging of image complete");
       return(GetFirstImageInList(image));
@@ -821,7 +816,7 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
           offset=TellBlob(image);
           size=ReadBlobMSBLong(image);
           number_layers=(short) ReadBlobMSBShort(image);
-          if (image->debug != False)
+          if (image->debug != MagickFalse)
             (void) LogMagickEvent(CoderEvent,GetMagickModule(),
               "  image contains %ld layers", number_layers);
           if (number_layers < 0)
@@ -831,29 +826,30 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
               */
               skip_first_alpha=1;
               number_layers=AbsoluteValue(number_layers);
-              if (image->debug != False)
+              if (image->debug != MagickFalse)
                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                   "  negative layer count corrected for");
             }
           layer_info=(LayerInfo *)
-            AcquireMagickMemory((size_t) number_layers*sizeof(LayerInfo));
+            AcquireMagickMemory((size_t) number_layers*sizeof(*layer_info));
           if (layer_info == (LayerInfo *) NULL) {
-          if (image->debug != False)
-            (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  allocation of LayerInfo failed");
+          if (image->debug != MagickFalse)
+            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+              "  allocation of LayerInfo failed");
             ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         }
         (void) ResetMagickMemory(layer_info,0,(size_t)
-          number_layers*sizeof(LayerInfo));
+          number_layers*sizeof(*layer_info));
         for (i=0; i < number_layers; i++)
         {
-          if (image->debug != False)
+          if (image->debug != MagickFalse)
               (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  reading layer #%ld", i+1);
             layer_info[i].page.y=(long) ReadBlobMSBLong(image);
             layer_info[i].page.x=(long) ReadBlobMSBLong(image);
             layer_info[i].page.height=(ReadBlobMSBLong(image)-layer_info[i].page.y);
             layer_info[i].page.width=(ReadBlobMSBLong(image)-layer_info[i].page.x);
             layer_info[i].channels=ReadBlobMSBShort(image);
-          if (image->debug != False)
+          if (image->debug != MagickFalse)
             (void) LogMagickEvent(CoderEvent,GetMagickModule(),"    offset(%ld,%ld), size(%ld,%ld), channels=%d",
               layer_info[i].page.x, layer_info[i].page.y,
               layer_info[i].page.height, layer_info[i].page.width,
@@ -864,14 +860,14 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 ReadBlobMSBShort(image);
               layer_info[i].channel_info[j].size=ReadBlobMSBLong(image);
 
-         if (image->debug != False)
+         if (image->debug != MagickFalse)
             (void) LogMagickEvent(CoderEvent,GetMagickModule(),"    channel[%ld]: type=%d, size=%ld",
                j, layer_info[i].channel_info[j].type,
                (long) layer_info[i].channel_info[j].size);
             }
             count=ReadBlob(image,4,(unsigned char *) type);
             if ((count == 0) || (LocaleNCompare(type,"8BIM",4) != 0)) {
-          if (image->debug != False)
+          if (image->debug != MagickFalse)
             (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  layer type was %.4s instead of 8BIM", type);
               ThrowReaderException(CorruptImageError,"ImproperImageHeader");
         }
@@ -881,7 +877,7 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
             layer_info[i].clipping=(unsigned char) ReadBlobByte(image);
             layer_info[i].flags=(unsigned char) ReadBlobByte(image);
             layer_info[i].visible=!(layer_info[i].flags & 0x02);
-          if (image->debug != False)
+          if (image->debug != MagickFalse)
             (void) LogMagickEvent(CoderEvent,GetMagickModule(),"    blend=%.4s, opacity=%d, clipping=%s, flags=%d, visible=%s",
               layer_info[i].blendkey, layer_info[i].opacity,
               layer_info[i].clipping ? "true" : "false",
@@ -893,7 +889,7 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
             size=ReadBlobMSBLong(image);
             if (size != 0)
               {
-            if (image->debug != False)
+            if (image->debug != MagickFalse)
               (void) LogMagickEvent(CoderEvent,GetMagickModule(),"    layer contains additional info");
                 length=ReadBlobMSBLong(image);
                 if (length != 0)
@@ -908,7 +904,7 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     layer_info[i].mask.width=
                       (ReadBlobMSBLong(image)-layer_info[i].mask.x);
 
-              if (image->debug != False)
+              if (image->debug != MagickFalse)
                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),"      layer mask: offset(%ld,%ld), size(%ld,%ld), length=%ld",
                   layer_info[i].mask.x, layer_info[i].mask.y,
                   layer_info[i].mask.width, layer_info[i].mask.height,
@@ -927,7 +923,7 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     /*
                       Layer blending ranges info.
                     */
-              if (image->debug != False)
+              if (image->debug != MagickFalse)
                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                   "      layer blending ranges: length=%ld",
                   (long) length);
@@ -936,13 +932,13 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
                       we read it, but don't use it...
                     */
                     for (j=0; j < (long) (length); j+=8) {
-              size_t blend_source = ReadBlobMSBLong(image);
-              size_t blend_dest = ReadBlobMSBLong(image);
-              if (image->debug != False)
-                (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                  "        source(%x), dest(%x)",
-                  (unsigned int) blend_source, (unsigned int) blend_dest);
-            }
+                      size_t blend_source = ReadBlobMSBLong(image);
+                      size_t blend_dest = ReadBlobMSBLong(image);
+                      if (image->debug != MagickFalse)
+                        (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                          "        source(%x), dest(%x)",
+                          (unsigned int) blend_source,(unsigned int) blend_dest);
+                    }
                   }
                 combinedlength += length + 4;  /* +4 for length */
 
@@ -954,7 +950,7 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
               layer_info[i].name[j]=(unsigned char) ReadBlobByte(image);
             layer_info[i].name[j] = 0;  /* zero term */
 
-            if (image->debug != False)
+            if (image->debug != MagickFalse)
               (void) LogMagickEvent(CoderEvent,GetMagickModule(),"      layer name: %s",
                 layer_info[i].name);
             }
@@ -982,18 +978,18 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
             count=ReadBlob(image,4,alsig);
             if ((count == 0) || (LocaleNCompare(alsig,"8BIM",4) != 0)) {
-              if (debug != False)
+              if (debug != MagickFalse)
               {
-      if (image->debug != False)
+      if (image->debug != MagickFalse)
                   (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  adjustment layer type was %.4s instead of 8BIM", alsig);
               }
               ThrowReaderException(CorruptImageError,"ImproperImageHeader");
             }
             (void) ReadBlob(image,4,alkey);
             length=ReadBlobMSBLong(image);
-              if (debug != False)
+              if (debug != MagickFalse)
               {
-      if (image->debug != False)
+      if (image->debug != MagickFalse)
                   (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                             "      adjustment layer key: %.4s, data length=%ld",
                             alkey, length);
@@ -1011,7 +1007,7 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
                /*
                   Skip the rest of the variable data until we support it.
                 */
-            if (image->debug != False)
+            if (image->debug != MagickFalse)
               (void) LogMagickEvent(CoderEvent,GetMagickModule(),"      unsupported data: length=%ld",
                 (long) size-combinedlength);
                 for (j=0; j < (long) (size-combinedlength); j++)
@@ -1022,29 +1018,29 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
             */
             layer_info[i].image=CloneImage(image,layer_info[i].page.width,
                                            layer_info[i].page.height,
-                                           True,&image->exception);
+                                           MagickTrue,&image->exception);
             if (layer_info[i].image == (Image *) NULL)
               {
                 for (j=0; j < i; j++)
-                  DestroyImage(layer_info[j].image);
+                  layer_info[j].image=DestroyImage(layer_info[j].image);
 
-            if (image->debug != False)
+            if (image->debug != MagickFalse)
               (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  allocation of image for layer %ld failed", i);
 
                 ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
               }
-          if (image->debug != False)
+          if (image->debug != MagickFalse)
             (void) LogMagickEvent(CoderEvent,GetMagickModule(),"    setting up new layer image");
             SetImage(layer_info[i].image,OpaqueOpacity);
             layer_info[i].image->compose=
               PSDBlendModeToCompositeOperator(layer_info[i].blendkey);
-            if (layer_info[i].visible == False)
+            if (layer_info[i].visible == MagickFalse)
               layer_info[i].image->compose=NoCompositeOp;
             if (psd_info.mode == CMYKMode)
               layer_info[i].image->colorspace=CMYKColorspace;
             for (j=0; j < (long) layer_info[i].channels; j++)
               if (layer_info[i].channel_info[j].type == -1)
-                layer_info[i].image->matte=True;
+                layer_info[i].image->matte=MagickTrue;
 
             /* set up some hidden attributes for folks that need them */
         FormatMagickString(s,MaxTextExtent,"%ld", layer_info[i].page.x );
@@ -1053,30 +1049,31 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
         (void) SetImageAttribute(layer_info[i].image,"[layer-ypos]",s);
         FormatMagickString(s,MaxTextExtent,"%d", layer_info[i].opacity );
         (void) SetImageAttribute(layer_info[i].image,"[layer-opacity]",s);
-        (void) SetImageAttribute(layer_info[i].image,"[layer-name]", (char *) layer_info[i].name);
-          }
+        (void) SetImageAttribute(layer_info[i].image,"[layer-name]",
+          (char *) layer_info[i].name);
+      }
 
 
-        if (image->debug != False)
+        if (image->debug != MagickFalse)
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  reading image data for layers");
           /*
             Read pixel data for each layer.
           */
           for (i=0; i < number_layers; i++)
           {
-          if (image->debug != False)
+          if (image->debug != MagickFalse)
               (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  reading data for layer %ld", i);
             for (j=0; j < (long) layer_info[i].channels; j++)
             {
-          if (image->debug != False)
+          if (image->debug != MagickFalse)
             (void) LogMagickEvent(CoderEvent,GetMagickModule(),"    reading data for channel %ld", j);
-#if     0
+#if     1
               if (layer_info[i].channel_info[j].size <= (2*layer_info[i].image->rows))
                 {
                   long
                     k;
 
-      if (image->debug != False)
+      if (image->debug != MagickFalse)
               (void) LogMagickEvent(CoderEvent,GetMagickModule(),"      layer data is empty");
 
                   /*
@@ -1093,7 +1090,7 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   /*
                     Read RLE compressed data.
                   */
-          if (image->debug != False)
+          if (image->debug != MagickFalse)
             (void) LogMagickEvent(CoderEvent,GetMagickModule(),"      layer data is RLE compressed");
 
                   for (y=0; y < (long) layer_info[i].image->rows; y++)
@@ -1106,7 +1103,7 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
               /*
                 Read uncompressed pixel data as separate planes.
               */
-        if (image->debug != False)
+        if (image->debug != MagickFalse)
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),"      layer data is uncompressed");
               packet_size=1;
               if (layer_info[i].image->storage_class == PseudoClass)
@@ -1182,7 +1179,7 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   }
                   q++;
                 }
-                if (SyncImagePixels(layer_info[i].image) == False)
+                if (SyncImagePixels(layer_info[i].image) == MagickFalse)
                   break;
               }
             }
@@ -1204,7 +1201,7 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
                       (q->opacity*layer_info[i].opacity)/MaxRGB);
                     q++;
                   }
-                  if (SyncImagePixels(layer_info[i].image) == False)
+                  if (SyncImagePixels(layer_info[i].image) == MagickFalse)
                     break;
                 }
               }
@@ -1213,7 +1210,7 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 /*
                   Correct CMYK levels.
                 */
-            if (image->debug != False)
+            if (image->debug != MagickFalse)
               (void) LogMagickEvent(CoderEvent,GetMagickModule(),"    correcting CMYK values");
                 for (y=0; y < (long) layer_info[i].image->rows; y++)
                 {
@@ -1230,21 +1227,62 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     indexes[x]=(IndexPacket) (MaxRGB-indexes[x]);
                     q++;
                   }
-                  if (SyncImagePixels(layer_info[i].image) == False)
+                  if (SyncImagePixels(layer_info[i].image) == MagickFalse)
                     break;
                 }
               }
           }
-
         diff_offset = TellBlob(image) - offset;
 
+        /* added by palf -> invisible group layer make layer of this group
+           invisible I consider that all layer with width and height null are
+           layer for group layer */
+       {
+         short inside_layer = 0;
+         short layer_visible = 0;
+       for (i=number_layers-1; i >=0; i--)
+         {
+           if ((layer_info[i].page.width == 0) || (layer_info[i].page.height == 0))
+             {
+          if (inside_layer == 0)
+            {
+              inside_layer = 1;
+              layer_visible = (short int) layer_info[i].visible;
+            }
+          else
+            {
+              inside_layer = 0;
+            }
+             }
+           else if ((inside_layer == 1) && (layer_visible == 0))
+             {
+          layer_info[i].visible = (short int) layer_visible;
+          layer_info[i].image->compose=NoCompositeOp;
+             }
+         }
+       }
+
+       /* added by palf -> suppression of empty layer */
+       /* I consider that all layer with width and height null are layer for group layer */
+       for (i=0; i < number_layers; i++)
+         {
+           if ((layer_info[i].page.width == 0) || (layer_info[i].page.height == 0))
+             {
+          if (layer_info[i].image != (Image *) NULL)
+            layer_info[i].image=DestroyImage(layer_info[i].image);
+          for (j=i; j < number_layers - 1; j++)
+            layer_info[j] = layer_info[j+1];
+          number_layers--;
+          i--;
+             }
+         } 
         mask_size = ReadBlobMSBLong(image);  /* global mask size: currently ignored */
 
           if (number_layers > 0)
             {
           Image*  returnImage = image;
 
-          if (image->debug != False)
+          if (image->debug != MagickFalse)
             (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  putting layers into image list");
               for (i=0; i < number_layers; i++)
               {
@@ -1262,7 +1300,7 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
           }
         else
           {
-            DestroyImage(image);
+            image=DestroyImage(image);
             returnImage = layer_info[0].image;
           }
               layer_info=(LayerInfo *) RelinquishMagickMemory(layer_info);
@@ -1270,7 +1308,7 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
               return(returnImage);
             }
         } else {
-        if (image->debug != False)
+        if (image->debug != MagickFalse)
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  image has no layers");
       }
     }
@@ -1278,7 +1316,7 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
   Read the precombined image, present for PSD < 4 compatibility
   */
-  if (image->debug != False)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  reading the precombined layer");
   compression=ReadBlobMSBShort(image);
   if (compression == 1)
@@ -1370,7 +1408,7 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
             }
             q++;
           }
-          if (SyncImagePixels(image) == False)
+          if (SyncImagePixels(image) == MagickFalse)
             break;
         }
       }
@@ -1394,7 +1432,7 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
           indexes[x]=(IndexPacket) (MaxRGB-indexes[x]);
           q++;
         }
-        if (SyncImagePixels(image) == False)
+        if (SyncImagePixels(image) == MagickFalse)
           break;
       }
     }
@@ -1480,14 +1518,10 @@ ModuleExport void UnregisterPSDImage(void)
 %
 %  The format of the WritePSDImage method is:
 %
-%      unsigned int WritePSDImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WritePSDImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WritePSDImage return True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
-%
 %    o image_info: The image info.
 %
 %    o image:  The image.
@@ -1528,7 +1562,7 @@ static void WriteImageChannels( Image* image, Image* tmp_image, unsigned char *p
   (void) WriteBlobMSBShort(image,0);  /* compression */
   if (tmp_image->storage_class == PseudoClass)
   {
-    if (tmp_image->matte == False)
+    if (tmp_image->matte == MagickFalse)
       WriteOneChannel( image, tmp_image, pixels, IndexQuantum );
     else
       WriteOneChannel( image, tmp_image, pixels, IndexAlphaQuantum );
@@ -1626,11 +1660,16 @@ static void WritePascalString (Image* inImage, const char *inString, int inPad)
     (void) WriteBlobByte(inImage,0);
 }
 
-static unsigned int WritePSDImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WritePSDImage(const ImageInfo *image_info,Image *image)
 {
   ImageAttribute
     *theAttr;
 
+  MagickBooleanType
+    force_white_background = image->matte,
+    invert_layer_count = MagickFalse,
+    status;
+
   register long
     i;
 
@@ -1645,11 +1684,6 @@ static unsigned int WritePSDImage(const ImageInfo *image_info,Image *image)
     *pixels,
     layer_name[4];
 
-  unsigned int
-    force_white_background = image->matte,
-    invert_layer_count = False,
-    status;
-
   unsigned long
     channel_size,
     channelLength,
@@ -1668,13 +1702,13 @@ static unsigned int WritePSDImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  if (image->debug != False)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
+  if (status == MagickFalse)
     return(status);
   packet_size=(size_t) (image->depth > 8 ? 6 : 3);
-  if (image->matte != False)
+  if (image->matte != MagickFalse)
     packet_size+=image->depth > 8 ? 2 : 1;
   pixels=(unsigned char *)
     AcquireMagickMemory(packet_size*image->columns*sizeof(PixelPacket));
@@ -1739,7 +1773,7 @@ static unsigned int WritePSDImage(const ImageInfo *image_info,Image *image)
     }
 
   /* image resource block */
-  profile=GetImageProfile(image,"8bim");
+  profile=GetImageProfile(image,"iptc");
   if (profile == (StringInfo *) NULL)
     (void) WriteBlobMSBLong(image,0);
   else
@@ -1756,12 +1790,12 @@ compute_layer_info:
     packet_size=tmp_image->depth > 8 ? 2UL : 1UL;
 
     if (tmp_image->storage_class == PseudoClass)
-     num_channels = (tmp_image->matte != False ? 2UL : 1UL);
+     num_channels = (tmp_image->matte != MagickFalse ? 2UL : 1UL);
     else
     if (tmp_image->colorspace != CMYKColorspace)
-      num_channels = (tmp_image->matte != False ? 4UL : 3UL);
+      num_channels = (tmp_image->matte != MagickFalse ? 4UL : 3UL);
     else
-      num_channels = (tmp_image->matte != False ? 5UL : 4UL);
+      num_channels = (tmp_image->matte != MagickFalse ? 5UL : 4UL);
 
   channelLength=(tmp_image->columns * tmp_image->rows * packet_size + 2);
   layer_info_size += (4*4 + 2 + num_channels * 6 + 4 + 4 + 4 * 1 + 4 + 12 + num_channels * channelLength);
@@ -1771,9 +1805,9 @@ compute_layer_info:
   }
 
   /* if the image has a matte, then we need to use layers */
-  if ( layer_count == 0 && image->matte == True )
+  if ( layer_count == 0 && image->matte == MagickTrue )
   {
-  invert_layer_count = True;
+  invert_layer_count = MagickTrue;
   base_image = image;
   goto compute_layer_info;  /* yes, goto's suck, but it keeps the code cleaner! */
   }
@@ -1891,12 +1925,12 @@ compute_layer_info:
   }
 
    /* now the background image data! */
-   if (force_white_background  != False)
+   if (force_white_background  != MagickFalse)
      WriteWhiteBackground(image);
    else
   WriteImageChannels(image,image,pixels);
 
   pixels=(unsigned char *) RelinquishMagickMemory(pixels);
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
