@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,10 +41,13 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -55,7 +58,7 @@
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WriteRAWImage(const ImageInfo *,Image *);
 
 /*
@@ -94,8 +97,8 @@ static Image *ReadRAWImage(const ImageInfo *image_info,ExceptionInfo *exception)
     j,
     y;
 
-  QuantumType
-    quantum_type;
+  MagickBooleanType
+    status;
 
   register long
     i,
@@ -113,10 +116,8 @@ static Image *ReadRAWImage(const ImageInfo *image_info,ExceptionInfo *exception)
   unsigned char
     *scanline;
 
-  unsigned int
-    status;
-
   unsigned long
+    depth,
     width;
 
   /*
@@ -124,7 +125,7 @@ static Image *ReadRAWImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
-  if (image_info->debug != False)
+  if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
@@ -132,93 +133,35 @@ static Image *ReadRAWImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if ((image->columns == 0) || (image->rows == 0))
     ThrowReaderException(OptionError,"MustSpecifyImageSize");
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
+  if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   switch (*image->magick)
   {
-    case 'A':
-    case 'a':
-    {
-      quantum_type=AlphaQuantum;
-      break;
-    }
-    case 'B':
-    case 'b':
-    {
-      quantum_type=BlueQuantum;
-      break;
-    }
     case 'C':
     case 'c':
-    {
-      quantum_type=CyanQuantum;
-      if (image->colorspace == CMYKColorspace)
-        break;
-      ThrowWriterException(ImageError,"ColorSeparatedImageRequired");
-    }
-    case 'g':
-    case 'G':
-    {
-      quantum_type=GreenQuantum;
-      break;
-    }
-    case 'I':
-    case 'i':
-    {
-      quantum_type=IndexQuantum;
-      break;
-    }
+    case 'M':
+    case 'm':
+    case 'Y':
+    case 'y':
     case 'K':
     case 'k':
     {
-      quantum_type=BlackQuantum;
-      if (image->colorspace == CMYKColorspace)
-        break;
-      ThrowWriterException(ImageError,"ColorSeparatedImageRequired");
-    }
-    case 'M':
-    case 'm':
-    {
-      quantum_type=MagentaQuantum;
-      if (image->colorspace == CMYKColorspace)
-        break;
-      ThrowWriterException(ImageError,"ColorSeparatedImageRequired");
-    }
-    case 'o':
-    case 'O':
-    {
-      quantum_type=OpacityQuantum;
+      image->colorspace=CMYKColorspace;
       break;
     }
-    case 'R':
-    case 'r':
-    {
-      quantum_type=RedQuantum;
-      break;
-    }
-    case 'Y':
-    case 'y':
-    {
-      quantum_type=YellowQuantum;
-      if (image->colorspace == CMYKColorspace)
-        break;
-      ThrowWriterException(ImageError,"ColorSeparatedImageRequired");
-    }
     default:
-    {
-      quantum_type=GrayQuantum;
       break;
-    }
   }
   for (i=0; i < image->offset; i++)
     (void) ReadBlobByte(image);
   /*
     Allocate memory for a scanline.
   */
-  packet_size=(size_t) (image->depth > 8 ? 2 : 1);
+  depth=GetImageQuantumDepth(image,MagickTrue);
+  packet_size=(size_t) (depth/8);
   scanline=(unsigned char *)
     AcquireMagickMemory(packet_size*image->extract_info.width);
   if (scanline == (unsigned char *) NULL)
@@ -239,9 +182,9 @@ static Image *ReadRAWImage(const ImageInfo *image_info,ExceptionInfo *exception)
     /*
       Convert raster image to pixel packets.
     */
-    if (AllocateImageColormap(image,1 << image->depth) == False)
+    if (AllocateImageColormap(image,1 << image->depth) == MagickFalse)
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-    if ((image_info->ping != False) && (image_info->number_scenes != 0))
+    if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     for (y=0; y < image->extract_info.y; y++)
@@ -254,22 +197,29 @@ static Image *ReadRAWImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (q == (PixelPacket *) NULL)
         break;
       (void) PushImagePixels(image,GrayQuantum,scanline+x);
-      if (SyncImagePixels(image) == False)
+      if (SyncImagePixels(image) == MagickFalse)
         break;
       if (image->previous == (Image *) NULL)
-        if (QuantumTick(y,image->rows))
-          if (MagickMonitor(LoadImageTag,y,image->rows,exception) == False)
-            break;
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
+          {
+            status=image->progress_monitor(LoadImageTag,y,image->rows,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
     }
     width=image->extract_info.height-image->rows-image->extract_info.y;
     for (j=0; j < (long) width; j++)
       (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
-    if (EOFBlob(image) != False)
+    if (EOFBlob(image) != MagickFalse)
       {
         ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
           image->filename);
         break;
       }
+    if (image->colorspace == CMYKColorspace)
+      image->storage_class=DirectClass;
     /*
       Proceed to next image.
     */
@@ -285,14 +235,17 @@ static Image *ReadRAWImage(const ImageInfo *image_info,ExceptionInfo *exception)
         AllocateNextImage(image_info,image);
         if (image->next == (Image *) NULL)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
-          exception);
-        if (status == False)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
+              GetBlobSize(image),image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
   } while (count != 0);
   scanline=(unsigned char *) RelinquishMagickMemory(scanline);
@@ -330,61 +283,72 @@ ModuleExport void RegisterRAWImage(void)
   entry=SetMagickInfo("R");
   entry->decoder=(DecoderHandler *) ReadRAWImage;
   entry->encoder=(EncoderHandler *) WriteRAWImage;
-  entry->raw=True;
+  entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->description=AcquireString("Raw red samples");
   entry->module=AcquireString("RAW");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("C");
   entry->decoder=(DecoderHandler *) ReadRAWImage;
   entry->encoder=(EncoderHandler *) WriteRAWImage;
-  entry->raw=True;
+  entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->description=AcquireString("Raw cyan samples");
   entry->module=AcquireString("RAW");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("G");
   entry->decoder=(DecoderHandler *) ReadRAWImage;
   entry->encoder=(EncoderHandler *) WriteRAWImage;
-  entry->raw=True;
+  entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->description=AcquireString("Raw green samples");
   entry->module=AcquireString("RAW");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("M");
   entry->decoder=(DecoderHandler *) ReadRAWImage;
   entry->encoder=(EncoderHandler *) WriteRAWImage;
-  entry->raw=True;
+  entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->description=AcquireString("Raw magenta samples");
   entry->module=AcquireString("RAW");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("B");
   entry->decoder=(DecoderHandler *) ReadRAWImage;
   entry->encoder=(EncoderHandler *) WriteRAWImage;
-  entry->raw=True;
+  entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->description=AcquireString("Raw blue samples");
   entry->module=AcquireString("RAW");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("Y");
   entry->decoder=(DecoderHandler *) ReadRAWImage;
   entry->encoder=(EncoderHandler *) WriteRAWImage;
-  entry->raw=True;
+  entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->description=AcquireString("Raw yellow samples");
   entry->module=AcquireString("RAW");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("A");
   entry->decoder=(DecoderHandler *) ReadRAWImage;
   entry->encoder=(EncoderHandler *) WriteRAWImage;
-  entry->raw=True;
+  entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->description=AcquireString("Raw alpha samples");
+  entry->module=AcquireString("RAW");
+  (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("O");
   entry->decoder=(DecoderHandler *) ReadRAWImage;
   entry->encoder=(EncoderHandler *) WriteRAWImage;
-  entry->raw=True;
+  entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->description=AcquireString("Raw opacity samples");
   entry->module=AcquireString("RAW");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("K");
   entry->decoder=(DecoderHandler *) ReadRAWImage;
   entry->encoder=(EncoderHandler *) WriteRAWImage;
-  entry->raw=True;
+  entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->description=AcquireString("Raw black samples");
   entry->module=AcquireString("RAW");
   (void) RegisterMagickInfo(entry);
@@ -437,7 +401,7 @@ ModuleExport void UnregisterRAWImage(void)
 %
 %  The format of the WriteRAWImage method is:
 %
-%      unsigned int WriteRAWImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteRAWImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
@@ -447,7 +411,7 @@ ModuleExport void UnregisterRAWImage(void)
 %
 %
 */
-static unsigned int WriteRAWImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteRAWImage(const ImageInfo *image_info,Image *image)
 {
   long
     y;
@@ -458,6 +422,9 @@ static unsigned int WriteRAWImage(const ImageInfo *image_info,Image *image)
   QuantumType
     quantum_type;
 
+  MagickBooleanType
+    status;
+
   register const PixelPacket
     *p;
 
@@ -467,8 +434,8 @@ static unsigned int WriteRAWImage(const ImageInfo *image_info,Image *image)
   unsigned char
     *scanline;
 
-  unsigned int
-    status;
+  unsigned long
+    depth;
 
   /*
     Open output image file.
@@ -477,10 +444,10 @@ static unsigned int WriteRAWImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  if (image->debug != False)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
+  if (status == MagickFalse)
     return(status);
   switch (*image->magick)
   {
@@ -564,7 +531,8 @@ static unsigned int WriteRAWImage(const ImageInfo *image_info,Image *image)
     /*
       Allocate memory for scanline.
     */
-    packet_size=(size_t) (image->depth > 8 ? 2 : 1);
+    depth=GetImageQuantumDepth(image,MagickTrue);
+    packet_size=(size_t) (depth/8);
     scanline=(unsigned char *)
       AcquireMagickMemory(packet_size*image->columns);
     if (scanline == (unsigned char *) NULL)
@@ -580,10 +548,12 @@ static unsigned int WriteRAWImage(const ImageInfo *image_info,Image *image)
       (void) PopImagePixels(image,quantum_type,scanline);
       (void) WriteBlob(image,packet_size*image->columns,scanline);
       if (image->previous == (Image *) NULL)
-        if (QuantumTick(y,image->rows))
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
           {
-            status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
-            if (status == False)
+            status=image->progress_monitor(SaveImageTag,y,image->rows,
+              image->client_data);
+            if (status == MagickFalse)
               break;
           }
     }
@@ -591,11 +561,15 @@ static unsigned int WriteRAWImage(const ImageInfo *image_info,Image *image)
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == False)
-      break;
-  } while (image_info->adjoin != False);
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
+  } while (image_info->adjoin != MagickFalse);
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
