@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,10 +41,13 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -56,7 +59,7 @@
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WriteRGBImage(const ImageInfo *,Image *);
 
 /*
@@ -80,10 +83,6 @@ static unsigned int
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadRGBImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a memory shortage or
-%      if the image cannot be read.
-%
 %    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
@@ -98,9 +97,14 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   long
     y;
 
+  MagickBooleanType
+    status;
+
+  MagickOffsetType
+    offset;
+
   register long
-    i,
-    x;
+    i;
 
   register PixelPacket
     *q;
@@ -114,15 +118,13 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   unsigned char
     *scanline;
 
-  unsigned int
-    status;
-
   unsigned long
+    depth,
     width;
 
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
-  if (image_info->debug != False)
+  if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
@@ -135,9 +137,9 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         Open image file.
       */
       status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-      if (status == False)
+      if (status == MagickFalse)
         {
-          DestroyImageList(image);
+          image=DestroyImageList(image);
           return((Image *) NULL);
         }
       for (i=0; i < image->offset; i++)
@@ -146,11 +148,12 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Allocate memory for a scanline.
   */
-  packet_size=(size_t) (image->depth > 8 ? 6 : 3);
+  depth=GetImageQuantumDepth(image,MagickTrue);
+  packet_size=(size_t) (3*depth/8);
   if (LocaleCompare(image_info->magick,"RGBA") == 0)
     {
-      image->matte=True;
-      packet_size=(size_t) (image->depth > 8 ? 8 : 4);
+      packet_size+=depth/8;
+      image->matte=MagickTrue;
     }
   scanline=(unsigned char *)
     AcquireMagickMemory(packet_size*image->extract_info.width);
@@ -166,13 +169,13 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       for (y=0; y < (long) image->rows; y++)
         (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
     }
-  x=(long) (packet_size*image->extract_info.x);
+  offset=(MagickOffsetType) (packet_size*image->extract_info.x);
   do
   {
     /*
       Convert raster image to pixel packets.
     */
-    if ((image_info->ping != False) && (image_info->number_scenes != 0))
+    if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     switch (image_info->interlace)
@@ -193,16 +196,21 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          if (image->matte == False)
-            (void) PushImagePixels(image,RGBQuantum,scanline+x);
+          if (image->matte == MagickFalse)
+            (void) PushImagePixels(image,RGBQuantum,scanline+offset);
           else
-            (void) PushImagePixels(image,RGBAQuantum,scanline+x);
-          if (SyncImagePixels(image) == False)
+            (void) PushImagePixels(image,RGBAQuantum,scanline+offset);
+          if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows))
-              if (MagickMonitor(LoadImageTag,y,image->rows,exception) == False)
-                break;
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
+              {
+                status=image->progress_monitor(LoadImageTag,y,image->rows,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
         }
         width=image->extract_info.height-image->rows-image->extract_info.y;
         for (i=0; i < (long) width; i++)
@@ -226,23 +234,28 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          (void) PushImagePixels(image,RedQuantum,scanline+x);
+          (void) PushImagePixels(image,RedQuantum,scanline+offset);
           (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
-          (void) PushImagePixels(image,GreenQuantum,scanline+x);
+          (void) PushImagePixels(image,GreenQuantum,scanline+offset);
           (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
-          (void) PushImagePixels(image,BlueQuantum,scanline+x);
-          if (image->matte != False)
+          (void) PushImagePixels(image,BlueQuantum,scanline+offset);
+          if (image->matte != MagickFalse)
             {
               (void) ReadBlob(image,packet_size*image->extract_info.width,
                 scanline);
-              (void) PushImagePixels(image,AlphaQuantum,scanline+x);
+              (void) PushImagePixels(image,AlphaQuantum,scanline+offset);
             }
-          if (SyncImagePixels(image) == False)
+          if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows))
-              if (MagickMonitor(LoadImageTag,y,image->rows,exception) == False)
-                break;
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
+              {
+                status=image->progress_monitor(LoadImageTag,y,image->rows,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
         }
         width=image->extract_info.height-image->rows-image->extract_info.y;
         for (i=0; i < (long) width; i++)
@@ -262,9 +275,9 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           {
             AppendImageFormat("R",image->filename);
             status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-            if (status == False)
+            if (status == MagickFalse)
               {
-                DestroyImageList(image);
+                image=DestroyImageList(image);
                 return((Image *) NULL);
               }
           }
@@ -272,7 +285,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         for (y=0; y < image->extract_info.y; y++)
           (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
         i=0;
-        span=image->rows*(image->matte != False ? 4 : 3);
+        span=image->rows*(image->matte != MagickFalse ? 4 : 3);
         for (y=0; y < (long) image->rows; y++)
         {
           if ((y > 0) || (image->previous == (Image *) NULL))
@@ -281,13 +294,18 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          (void) PushImagePixels(image,RedQuantum,scanline+x);
-          if (SyncImagePixels(image) == False)
+          (void) PushImagePixels(image,RedQuantum,scanline+offset);
+          if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(i,span))
-              if (MagickMonitor(LoadImageTag,i,span,&image->exception) == False)
-                break;
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(i,span) != MagickFalse))
+              {
+                status=image->progress_monitor(LoadImageTag,i,span,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
           i++;
         }
         width=image->extract_info.height-image->rows-image->extract_info.y;
@@ -298,9 +316,9 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
             CloseBlob(image);
             AppendImageFormat("G",image->filename);
             status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-            if (status == False)
+            if (status == MagickFalse)
               {
-                DestroyImageList(image);
+                image=DestroyImageList(image);
                 return((Image *) NULL);
               }
           }
@@ -312,13 +330,18 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           q=GetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          (void) PushImagePixels(image,GreenQuantum,scanline+x);
-          if (SyncImagePixels(image) == False)
+          (void) PushImagePixels(image,GreenQuantum,scanline+offset);
+          if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(i,span))
-              if (MagickMonitor(LoadImageTag,i,span,&image->exception) == False)
-                break;
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(i,span) != MagickFalse))
+              {
+                status=image->progress_monitor(LoadImageTag,i,span,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
           i++;
         }
         width=image->extract_info.height-image->rows-image->extract_info.y;
@@ -329,9 +352,9 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
             CloseBlob(image);
             AppendImageFormat("B",image->filename);
             status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-            if (status == False)
+            if (status == MagickFalse)
               {
-                DestroyImageList(image);
+                image=DestroyImageList(image);
                 return((Image *) NULL);
               }
           }
@@ -343,19 +366,24 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           q=GetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          (void) PushImagePixels(image,BlueQuantum,scanline+x);
-          if (SyncImagePixels(image) == False)
+          (void) PushImagePixels(image,BlueQuantum,scanline+offset);
+          if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(i,span))
-              if (MagickMonitor(LoadImageTag,i,span,&image->exception) == False)
-                break;
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(i,span) != MagickFalse))
+              {
+                status=image->progress_monitor(LoadImageTag,i,span,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
           i++;
         }
         width=image->extract_info.height-image->rows-image->extract_info.y;
         for (i=0; i < (long) width; i++)
           (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
-        if (image->matte != False)
+        if (image->matte != MagickFalse)
           {
             /*
               Read matte channel.
@@ -365,9 +393,9 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 CloseBlob(image);
                 AppendImageFormat("A",image->filename);
                 status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-                if (status == False)
+                if (status == MagickFalse)
                   {
-                    DestroyImageList(image);
+                    image=DestroyImageList(image);
                     return((Image *) NULL);
                   }
               }
@@ -381,14 +409,16 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
               q=GetImagePixels(image,0,y,image->columns,1);
               if (q == (PixelPacket *) NULL)
                 break;
-              (void) PushImagePixels(image,AlphaQuantum,scanline+x);
-              if (SyncImagePixels(image) == False)
+              (void) PushImagePixels(image,AlphaQuantum,scanline+offset);
+              if (SyncImagePixels(image) == MagickFalse)
                 break;
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(i,span))
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(i,span) != MagickFalse))
                   {
-                    status=MagickMonitor(LoadImageTag,i,span,&image->exception);
-                    if (status == False)
+                    status=image->progress_monitor(LoadImageTag,i,span,
+                      image->client_data);
+                    if (status == MagickFalse)
                       break;
                   }
               i++;
@@ -404,7 +434,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         break;
       }
     }
-    if (EOFBlob(image) != False)
+    if (EOFBlob(image) != MagickFalse)
       {
         ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
           image->filename);
@@ -427,14 +457,17 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         AllocateNextImage(image_info,image);
         if (image->next == (Image *) NULL)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
-          exception);
-        if (status == False)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
+              GetBlobSize(image),image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
   } while (count != 0);
   scanline=(unsigned char *) RelinquishMagickMemory(scanline);
@@ -473,14 +506,16 @@ ModuleExport void RegisterRGBImage(void)
   entry=SetMagickInfo("RGB");
   entry->decoder=(DecoderHandler *) ReadRGBImage;
   entry->encoder=(EncoderHandler *) WriteRGBImage;
-  entry->raw=True;
+  entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->description=AcquireString("Raw red, green, and blue samples");
   entry->module=AcquireString("RGB");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("RGBA");
   entry->decoder=(DecoderHandler *) ReadRGBImage;
   entry->encoder=(EncoderHandler *) WriteRGBImage;
-  entry->raw=True;
+  entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->description=AcquireString("Raw red, green, blue, and matte samples");
   entry->module=AcquireString("RGB");
   (void) RegisterMagickInfo(entry);
@@ -527,25 +562,24 @@ ModuleExport void UnregisterRGBImage(void)
 %
 %  The format of the WriteRGBImage method is:
 %
-%      unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteRGBImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteRGBImage return True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
-%
 %    o image_info: The image info.
 %
 %    o image:  The image.
 %
 %
 */
-static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteRGBImage(const ImageInfo *image_info,Image *image)
 {
   long
     y;
 
+  MagickBooleanType
+    status;
+
   MagickOffsetType
     scene;
 
@@ -558,8 +592,8 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
   unsigned char
     *pixels;
 
-  unsigned int
-    status;
+  unsigned long
+    depth;
 
   /*
     Allocate memory for pixels.
@@ -568,12 +602,14 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  if (image->debug != False)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
-  packet_size=(size_t) (image->depth > 8 ? 6 : 3);
+  depth=GetImageQuantumDepth(image,MagickTrue);
+  packet_size=(size_t) (3*depth/8);
   if (LocaleCompare(image_info->magick,"RGBA") == 0)
-    packet_size=(size_t) (image->depth > 8 ? 8 : 4);
-  pixels=(unsigned char *) AcquireMagickMemory(packet_size*image->columns);
+    packet_size+=depth/8;
+  pixels=(unsigned char *)
+    AcquireMagickMemory(packet_size*image->columns*sizeof(*pixels));
   if (pixels == (unsigned char *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
   if (image_info->interlace != PartitionInterlace)
@@ -582,7 +618,7 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
         Open output image file.
       */
       status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-      if (status == False)
+      if (status == MagickFalse)
         return(status);
     }
   scene=0;
@@ -593,7 +629,7 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
     */
     (void) SetImageColorspace(image,RGBColorspace);
     if (LocaleCompare(image_info->magick,"RGBA") == 0)
-      if (image->matte == False)
+      if (image->matte == MagickFalse)
         SetImageOpacity(image,OpaqueOpacity);
     switch (image_info->interlace)
     {
@@ -619,11 +655,12 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
               (void) WriteBlob(image,packet_size*image->columns,pixels);
             }
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows))
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(SaveImageTag,y,image->rows,
-                  &image->exception);
-                if (status == False)
+                status=image->progress_monitor(SaveImageTag,y,image->rows,
+                  image->client_data);
+                if (status == MagickFalse)
                   break;
               }
         }
@@ -650,11 +687,12 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
               (void) PopImagePixels(image,AlphaQuantum,pixels);
               (void) WriteBlob(image,(size_t) image->columns,pixels);
             }
-          if (QuantumTick(y,image->rows))
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(y,image->rows) != MagickFalse))
             {
-              status=MagickMonitor(SaveImageTag,y,image->rows,
-                &image->exception);
-              if (status == False)
+              status=image->progress_monitor(SaveImageTag,y,image->rows,
+                image->client_data);
+              if (status == MagickFalse)
                 break;
             }
         }
@@ -671,7 +709,7 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
             AppendImageFormat("R",image->filename);
             status=OpenBlob(image_info,image,WriteBinaryBlobMode,
               &image->exception);
-            if (status == False)
+            if (status == MagickFalse)
               return(status);
           }
         for (y=0; y < (long) image->rows; y++)
@@ -688,11 +726,16 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
             AppendImageFormat("G",image->filename);
             status=OpenBlob(image_info,image,WriteBinaryBlobMode,
               &image->exception);
-            if (status == False)
+            if (status == MagickFalse)
               return(status);
           }
-        if (MagickMonitor(SaveImageTag,100,400,&image->exception) == False)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImageTag,100,400,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
         for (y=0; y < (long) image->rows; y++)
         {
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
@@ -707,11 +750,16 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
             AppendImageFormat("B",image->filename);
             status=OpenBlob(image_info,image,WriteBinaryBlobMode,
               &image->exception);
-            if (status == False)
+            if (status == MagickFalse)
               return(status);
           }
-        if (MagickMonitor(SaveImageTag,200,400,&image->exception) == False)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImageTag,200,400,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
         for (y=0; y < (long) image->rows; y++)
         {
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
@@ -722,15 +770,20 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
         }
         if (LocaleCompare(image_info->magick,"RGBA") == 0)
           {
-            if (MagickMonitor(SaveImageTag,300,400,&image->exception) == False)
-              break;
+            if (image->progress_monitor != (MagickProgressMonitor) NULL)
+              {
+                status=image->progress_monitor(LoadImageTag,300,400,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
             if (image_info->interlace == PartitionInterlace)
               {
                 CloseBlob(image);
                 AppendImageFormat("A",image->filename);
                 status=OpenBlob(image_info,image,WriteBinaryBlobMode,
                   &image->exception);
-                if (status == False)
+                if (status == MagickFalse)
                   return(status);
               }
             for (y=0; y < (long) image->rows; y++)
@@ -746,20 +799,29 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
         if (image_info->interlace == PartitionInterlace)
           (void) CopyMagickString(image->filename,image_info->filename,
             MaxTextExtent);
-        if (MagickMonitor(SaveImageTag,400,400,&image->exception) == False)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImageTag,400,400,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
         break;
       }
     }
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == False)
-      break;
-  } while (image_info->adjoin != False);
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
+  } while (image_info->adjoin != MagickFalse);
   pixels=(unsigned char *) RelinquishMagickMemory(pixels);
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
