@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,10 +42,13 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/color_private.h"
 #include "magick/colorspace.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -56,7 +59,7 @@
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WriteTGAImage(const ImageInfo *,Image *);
 
 /*
@@ -80,10 +83,6 @@ static unsigned int
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadTGAImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a memory shortage or
-%      if the image cannot be read.
-%
 %    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
@@ -133,6 +132,9 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
   long
     y;
 
+  MagickBooleanType
+    status;
+
   PixelPacket
     pixel;
 
@@ -157,9 +159,6 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
     k,
     runlength;
 
-  unsigned int
-    status;
-
   unsigned long
     base,
     flag,
@@ -172,50 +171,81 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
-  if (image_info->debug != False)
+  if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
+  if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
     Read TGA header information.
   */
   count=ReadBlob(image,1,&tga_info.id_length);
-  tga_info.colormap_type=ReadBlobByte(image);
-  tga_info.image_type=ReadBlobByte(image);
+  tga_info.colormap_type=(unsigned char) ReadBlobByte(image);
+  tga_info.image_type=(unsigned char) ReadBlobByte(image);
   do
   {
-    if ((count == 0) || (tga_info.image_type == 0) ||
-        (tga_info.image_type > 11))
+    if ((count == 0) ||
+        ((tga_info.image_type != TGAColormap) &&
+         (tga_info.image_type != TGARGB) &&
+         (tga_info.image_type != TGAMonochrome) &&
+         (tga_info.image_type != TGARLEColormap) &&
+         (tga_info.image_type != TGARLERGB) &&
+         (tga_info.image_type != TGARLEMonochrome)) ||
+        (((tga_info.image_type == TGAColormap) ||
+         (tga_info.image_type == TGARLEColormap)) &&
+         (tga_info.colormap_type == 0)))
       ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     tga_info.colormap_index=ReadBlobLSBShort(image);
     tga_info.colormap_length=ReadBlobLSBShort(image);
-    tga_info.colormap_size=ReadBlobByte(image);
+    tga_info.colormap_size=(unsigned char) ReadBlobByte(image);
     tga_info.x_origin=ReadBlobLSBShort(image);
     tga_info.y_origin=ReadBlobLSBShort(image);
     tga_info.width=ReadBlobLSBShort(image);
     tga_info.height=ReadBlobLSBShort(image);
-    tga_info.bits_per_pixel=ReadBlobByte(image);
-    tga_info.attributes=ReadBlobByte(image);
+    tga_info.bits_per_pixel=(unsigned char) ReadBlobByte(image);
+    tga_info.attributes=(unsigned char) ReadBlobByte(image);
     /*
       Initialize image structure.
     */
-    image->matte=tga_info.bits_per_pixel == 32;
+    image->matte=tga_info.attributes & 0x0FU ? MagickTrue : MagickFalse;
     image->columns=tga_info.width;
     image->rows=tga_info.height;
-    image->depth=tga_info.bits_per_pixel <= 8 ? 8 : QuantumDepth;
-    if (tga_info.colormap_type != 0)
+    if ((tga_info.image_type != TGAColormap) &&
+        (tga_info.image_type != TGARLEColormap))
+      image->depth=(unsigned long) ((tga_info.bits_per_pixel <= 8) ? 8 :
+        (tga_info.bits_per_pixel <= 16) ? 5 :
+        (tga_info.bits_per_pixel == 24) ? 8 :
+        (tga_info.bits_per_pixel == 32) ? 8 : 8);
+    else
+      image->depth=(unsigned long) ((tga_info.colormap_size <= 8) ? 8 :
+        (tga_info.colormap_size <= 16) ? 5 :
+        (tga_info.colormap_size == 24) ? 8 :
+        (tga_info.colormap_size == 32) ? 8 : 8);
+    if ((tga_info.image_type == TGAColormap) ||
+        (tga_info.image_type == TGAMonochrome) ||
+        (tga_info.image_type == TGARLEColormap) ||
+        (tga_info.image_type == TGARLEMonochrome))
+      image->storage_class=PseudoClass;
+    image->compression=NoCompression;
+    if ((tga_info.image_type == TGARLEColormap) ||
+        (tga_info.image_type == TGARLEMonochrome))
+      image->compression=RLECompression;
+    if (image->storage_class == PseudoClass)
       {
-        if ((tga_info.image_type == TGARLEColormap) ||
-            (tga_info.image_type == TGARLERGB))
-          image->storage_class=PseudoClass;
-        image->colors=tga_info.colormap_length;
+        if (tga_info.colormap_type != 0)
+          image->colors=tga_info.colormap_length;
+        else
+          {
+            image->colors=0x01U << tga_info.bits_per_pixel;
+            if (AllocateImageColormap(image,image->colors) == MagickFalse)
+              ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+          }
       }
     if (tga_info.id_length != 0)
       {
@@ -225,22 +255,22 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           TGA image comment.
         */
-        comment=(char *) AcquireMagickMemory(tga_info.id_length+1);
+        comment=(char *) AcquireMagickMemory(tga_info.id_length+MaxTextExtent);
         if (comment == (char *) NULL)
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         (void) ReadBlob(image,tga_info.id_length,(unsigned char *) comment);
         comment[tga_info.id_length]='\0';
-        (void) SetImageAttribute(image,"comment",comment);
+        (void) SetImageAttribute(image,"Comment",comment);
         comment=(char *) RelinquishMagickMemory(comment);
       }
-    (void) ResetMagickMemory(&pixel,0,sizeof(PixelPacket));
+    (void) ResetMagickMemory(&pixel,0,sizeof(pixel));
     pixel.opacity=TransparentOpacity;
     if (tga_info.colormap_type != 0)
       {
         /*
           Read TGA raster colormap.
         */
-        if (AllocateImageColormap(image,image->colors) == False)
+        if (AllocateImageColormap(image,image->colors) == MagickFalse)
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         for (i=0; i < (long) image->colors; i++)
         {
@@ -263,8 +293,8 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               /*
                 5 bits each of red green and blue.
               */
-              j=ReadBlobByte(image);
-              k=ReadBlobByte(image);
+              j=(unsigned char) ReadBlobByte(image);
+              k=(unsigned char) ReadBlobByte(image);
               pixel.red=ScaleXToQuantum((int) (k & 0x7c) >> 2,31);
               pixel.green=ScaleXToQuantum(((int) (k & 0x03) << 3)+
                 ((int) (j & 0xe0) >> 5),31);
@@ -286,7 +316,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
           image->colormap[i]=pixel;
         }
       }
-    if ((image_info->ping != False) && (image_info->number_scenes != 0))
+    if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     /*
@@ -294,7 +324,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
     */
     base=0;
     flag=0;
-    skip=False;
+    skip=MagickFalse;
     real=0;
     index=0;
     runlength=0;
@@ -328,10 +358,10 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 flag=runlength & 0x80;
                 if (flag != 0)
                   runlength-=128;
-                skip=False;
+                skip=MagickFalse;
               }
           }
-        if (skip == False)
+        if (skip == MagickFalse)
           switch (tga_info.bits_per_pixel)
           {
             case 8:
@@ -340,7 +370,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               /*
                 Gray scale.
               */
-              index=ReadBlobByte(image);
+              index=(IndexPacket) ReadBlobByte(image);
               if (tga_info.colormap_type != 0)
                 pixel=image->colormap[ConstrainColormapIndex(image,index)];
               else
@@ -357,14 +387,14 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               /*
                 5 bits each of red green and blue.
               */
-              j=ReadBlobByte(image);
-              k=ReadBlobByte(image);
+              j=(unsigned char) ReadBlobByte(image);
+              k=(unsigned char) ReadBlobByte(image);
               pixel.red=ScaleXToQuantum((int) (k & 0x7c) >> 2,31);
               pixel.red=ScaleXToQuantum(((int) (k & 0x03) << 3)+
                 ((int) (j & 0xe0) >> 5),31);
               pixel.blue=ScaleXToQuantum((int) (j & 0x1f),31);
               if (image->storage_class == PseudoClass)
-                index=ConstrainColormapIndex(image,((unsigned short) k << 8)+j);
+                index=ConstrainColormapIndex(image,((unsigned long) k << 8)+j);
               break;
             }
             case 24:
@@ -377,11 +407,11 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               pixel.green=ScaleCharToQuantum(ReadBlobByte(image));
               pixel.red=ScaleCharToQuantum(ReadBlobByte(image));
               if (tga_info.bits_per_pixel == 32)
-                pixel.opacity=ScaleCharToQuantum(ReadBlobByte(image));
+                pixel.opacity=MaxRGB-ScaleCharToQuantum(ReadBlobByte(image));
               break;
             }
           }
-        if (status == False)
+        if (status == MagickFalse)
           ThrowReaderException(CorruptImageError,"UnableToReadImageData");
         if (image->storage_class == PseudoClass)
           indexes[x]=index;
@@ -399,14 +429,19 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
           base++;
           offset=base;
         }
-      if (SyncImagePixels(image) == False)
+      if (SyncImagePixels(image) == MagickFalse)
         break;
       if (image->previous == (Image *) NULL)
-        if (QuantumTick(y,image->rows))
-          if (MagickMonitor(LoadImageTag,y,image->rows,exception) == False)
-            break;
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
+          {
+            status=image->progress_monitor(LoadImageTag,y,image->rows,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
     }
-    if (EOFBlob(image) != False)
+    if (EOFBlob(image) != MagickFalse)
       {
         ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
           image->filename);
@@ -419,10 +454,15 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     count=ReadBlob(image,1,&tga_info.id_length);
-    tga_info.colormap_type=ReadBlobByte(image);
-    tga_info.image_type=ReadBlobByte(image);
-    status=((tga_info.image_type != 0) && (tga_info.image_type <= 11));
-    if (status == True)
+    tga_info.colormap_type=(unsigned char) ReadBlobByte(image);
+    tga_info.image_type=(unsigned char) ReadBlobByte(image);
+    status=((tga_info.image_type == TGAColormap) ||
+      (tga_info.image_type == TGARGB) ||
+      (tga_info.image_type == TGAMonochrome) ||
+      (tga_info.image_type == TGARLEColormap) ||
+      (tga_info.image_type == TGARLERGB) ||
+      (tga_info.image_type == TGARLEMonochrome)) ? MagickTrue : MagickFalse;
+    if (status == MagickTrue)
       {
         /*
           Allocate next image structure.
@@ -430,16 +470,19 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
         AllocateNextImage(image_info,image);
         if (image->next == (Image *) NULL)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
-          exception);
-        if (status == False)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
+              GetBlobSize(image),image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
-  } while (status == True);
+  } while (status == MagickTrue);
   CloseBlob(image);
   return(GetFirstImageInList(image));
 }
@@ -541,21 +584,17 @@ ModuleExport void UnregisterTGAImage(void)
 %
 %  The format of the WriteTGAImage method is:
 %
-%      unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteTGAImage return True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
-%
 %    o image_info: The image info.
 %
 %    o image:  The image.
 %
 %
 */
-static unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
 {
 #define TargaColormap 1
 #define TargaRGB 2
@@ -595,6 +634,9 @@ static unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
   long
     y;
 
+  MagickBooleanType
+    status;
+
   MagickOffsetType
     scene;
 
@@ -622,9 +664,6 @@ static unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
   unsigned char
     *targa_pixels;
 
-  unsigned int
-    status;
-
   /*
     Open output image file.
   */
@@ -632,10 +671,10 @@ static unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  if (image->debug != False)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
+  if (status == MagickFalse)
     return(status);
   scene=0;
   do
@@ -645,9 +684,9 @@ static unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
     */
     (void) SetImageColorspace(image,RGBColorspace);
     targa_info.id_length=0;
-    attribute=GetImageAttribute(image,"comment");
+    attribute=GetImageAttribute(image,"Comment");
     if (attribute != (const ImageAttribute *) NULL)
-      targa_info.id_length=Min(strlen(attribute->value),255);
+      targa_info.id_length=(unsigned char) Min(strlen(attribute->value),255);
     targa_info.colormap_type=0;
     targa_info.colormap_index=0;
     targa_info.colormap_length=0;
@@ -665,7 +704,7 @@ static unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
         */
         targa_info.image_type=TargaRGB;
         targa_info.bits_per_pixel=24;
-        if (image->matte != False)
+        if (image->matte != MagickFalse)
           {
             targa_info.bits_per_pixel=32;
             targa_info.attributes=8;  /* # of alpha bits */
@@ -725,8 +764,8 @@ static unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
     /*
       Convert MIFF to TGA raster pixels.
     */
-    count=(long) ((targa_info.bits_per_pixel*targa_info.width) >> 3);
-    targa_pixels=(unsigned char *) AcquireMagickMemory(count);
+    count=(ssize_t) ((targa_info.bits_per_pixel*targa_info.width) >> 3);
+    targa_pixels=(unsigned char *) AcquireMagickMemory((size_t) count);
     if (targa_pixels == (unsigned char *) NULL)
       ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     for (y=(long) (image->rows-1); y >= 0; y--)
@@ -739,25 +778,27 @@ static unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
       for (x=0; x < (long) image->columns; x++)
       {
         if (targa_info.image_type == TargaColormap)
-          *q++=indexes[x];
+          *q++=(unsigned char) indexes[x];
         else
           {
             *q++=ScaleQuantumToChar(p->blue);
             *q++=ScaleQuantumToChar(p->green);
             *q++=ScaleQuantumToChar(p->red);
-            if (image->matte != False)
-              *q++=ScaleQuantumToChar(p->opacity);
+            if (image->matte != MagickFalse)
+              *q++=MaxRGB-ScaleQuantumToChar(p->opacity);
             if (image->colorspace == CMYKColorspace)
               *q++=ScaleQuantumToChar(indexes[x]);
           }
         p++;
       }
-      (void) WriteBlob(image,q-targa_pixels,targa_pixels);
+      (void) WriteBlob(image,(size_t) (q-targa_pixels),targa_pixels);
       if (image->previous == (Image *) NULL)
-        if (QuantumTick(y,image->rows))
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
           {
-            status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
-            if (status == False)
+            status=image->progress_monitor(SaveImageTag,y,image->rows,
+              image->client_data);
+            if (status == MagickFalse)
               break;
           }
     }
@@ -765,11 +806,15 @@ static unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == False)
-      break;
-  } while (image_info->adjoin != False);
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
+  } while (image_info->adjoin != MagickFalse);
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
