@@ -17,7 +17,7 @@
 %                               November 2001                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,9 +41,13 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
+#include "magick/color.h"
 #include "magick/composite.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -156,17 +160,15 @@ typedef struct {
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  IsXCF() returns True if the image format type, identified by the
+%  IsXCF() returns MagickTrue if the image format type, identified by the
 %  magick string, is XCF (GIMP native format).
 %
 %  The format of the IsXCF method is:
 %
-%      unsigned int IsXCF(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsXCF(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsXCF() returns True if the image format type is XCF.
-%
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
 %
@@ -174,13 +176,13 @@ typedef struct {
 %
 %
 */
-static unsigned int IsXCF(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsXCF(const unsigned char *magick,const size_t length)
 {
   if (length < 8)
-    return(False);
+    return(MagickFalse);
   if (LocaleNCompare((char *) magick,"gimp xcf",8) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 
@@ -260,9 +262,6 @@ static CompositeOperator GIMPBlendModeToCompositeOperator( unsigned long blendMo
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method ReadBlobString returns the string on success, otherwise,
-%      a null is returned.
-%
 %    o image: The image.
 %
 %    o string: The address of a character buffer.
@@ -282,7 +281,7 @@ static char *ReadBlobStringWithLongSize(Image *image,char *string)
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  if (image->debug != False)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   length = ReadBlobMSBLong(image);
   for (i=0; i < (long) length; i++)
@@ -336,7 +335,7 @@ static int load_tile (Image* image, Image* tile_image, XCFDocInfo* inDocInfo,
   }
 
   xcfodata=(XCFPixelPacket *) RelinquishMagickMemory(xcfodata);
-  return True;
+  return MagickTrue;
 }
 
 static int load_tile_rle (Image* image, Image* tile_image, XCFDocInfo* inDocInfo,
@@ -411,8 +410,7 @@ static int load_tile_rle (Image* image, Image* tile_image, XCFDocInfo* inDocInfo
                 case 1: q->green= ScaleCharToQuantum(data);  break;
                 case 2: q->blue = ScaleCharToQuantum(data);  break;
                 case 3:
-                  q->opacity = (Quantum) ((int) data == 0 ? TransparentOpacity :
-                    ScaleCharToQuantum(255-inLayerInfo->opacity));
+                  q->opacity = (Quantum) (ScaleCharToQuantum(255-data));
                 break;
               }
               q++;
@@ -460,8 +458,7 @@ static int load_tile_rle (Image* image, Image* tile_image, XCFDocInfo* inDocInfo
                 case 1: q->green=ScaleCharToQuantum(data);  break;
                 case 2: q->blue=ScaleCharToQuantum(data);  break;
                 case 3:
-                  q->opacity=(Quantum) ((int) data == 0 ? TransparentOpacity :
-                    ScaleCharToQuantum(255-inLayerInfo->opacity));
+                  q->opacity = (Quantum) (ScaleCharToQuantum(255-data));
                   break;
               }
               q++;
@@ -471,13 +468,13 @@ static int load_tile_rle (Image* image, Image* tile_image, XCFDocInfo* inDocInfo
       SyncImagePixels(tile_image);
     }
   xcfodata=(unsigned char *) RelinquishMagickMemory(xcfodata);
-  return True;
+  return MagickTrue;
 
  bogus_rle:
   if (xcfodata)
     xcfodata=(unsigned char *) RelinquishMagickMemory(xcfodata);
-return True;
-  return False;
+return MagickTrue;
+  return MagickFalse;
 }
 
 static int load_level (Image* image, XCFDocInfo* inDocInfo, XCFLayerInfo*
@@ -505,7 +502,7 @@ static int load_level (Image* image, XCFDocInfo* inDocInfo, XCFLayerInfo*
   long
     i;
 
-  unsigned int
+  MagickBooleanType
     fail;
 
   unsigned long
@@ -535,7 +532,7 @@ static int load_level (Image* image, XCFDocInfo* inDocInfo, XCFLayerInfo*
   ntiles = ntile_rows * ntile_cols;
   for (i = 0; i < (long) ntiles; i++)
     {
-      fail = False;
+      fail = MagickFalse;
 
       if (offset == 0)
         ThrowBinaryException(CorruptImageError,"NotEnoughTiles",
@@ -571,19 +568,19 @@ static int load_level (Image* image, XCFDocInfo* inDocInfo, XCFLayerInfo*
         (int) junk_height % TILE_HEIGHT : TILE_HEIGHT);
       if (tile_image_height == 0) tile_image_height=TILE_HEIGHT;
       tile_image=CloneImage(inLayerInfo->image,tile_image_width,
-        tile_image_height,True,exception);
+        tile_image_height,MagickTrue,exception);
 
       /* read in the tile */
       switch (inDocInfo->compression)
       {
         case COMPRESS_NONE:
           if (load_tile(image,tile_image,inDocInfo,inLayerInfo,(size_t) (offset2-offset)) == 0)
-            fail = True;
+            fail = MagickTrue;
           break;
         case COMPRESS_RLE:
           if (load_tile_rle (image,tile_image,inDocInfo,inLayerInfo,
               (int) (offset2-offset)) == 0)
-            fail = True;
+            fail = MagickTrue;
           break;
         case COMPRESS_ZLIB:
           ThrowBinaryException(CoderError,"ZipCompressionNotSupported",
@@ -596,7 +593,7 @@ static int load_level (Image* image, XCFDocInfo* inDocInfo, XCFLayerInfo*
       /* composite the tile onto the layer's image, and then destroy it */
         (void) CompositeImage(inLayerInfo->image,CopyCompositeOp,tile_image,
                   destLeft * TILE_WIDTH,destTop*TILE_HEIGHT);
-        DestroyImage(tile_image);
+        tile_image=DestroyImage(tile_image);
 
       /* adjust tile position */
       destLeft++;
@@ -606,8 +603,8 @@ static int load_level (Image* image, XCFDocInfo* inDocInfo, XCFLayerInfo*
           destTop++;
         }
 
-      if (fail != False)
-          return False;
+      if (fail != MagickFalse)
+          return MagickFalse;
 
       /* restore the saved position so we'll be ready to
        *  read the next offset.
@@ -677,8 +674,8 @@ static int load_hierarchy (Image *image, XCFDocInfo* inDocInfo, XCFLayerInfo*
 }
 
 
-static int ReadOneLayer( Image* image, XCFDocInfo* inDocInfo, XCFLayerInfo*
-  outLayer )
+static MagickBooleanType ReadOneLayer(Image* image,XCFDocInfo* inDocInfo,
+  XCFLayerInfo *outLayer )
 {
   long
     i;
@@ -700,14 +697,14 @@ static int ReadOneLayer( Image* image, XCFDocInfo* inDocInfo, XCFLayerInfo*
   (void) ReadBlobStringWithLongSize(image, outLayer->name);
 
   /* allocate the image for this layer */
-  outLayer->image=CloneImage(image,outLayer->width, outLayer->height,True,
+  outLayer->image=CloneImage(image,outLayer->width, outLayer->height,MagickTrue,
      &image->exception);
   if (outLayer->image == (Image *) NULL)
-    return False;
+    return MagickFalse;
 
   /* read the layer properties! */
   foundPropEnd = 0;
-  while ( foundPropEnd  == False ) {
+  while ( foundPropEnd  == MagickFalse ) {
   PropType    prop_type = (PropType) ReadBlobMSBLong(image);
   unsigned long  prop_size = ReadBlobMSBLong(image);
 
@@ -800,7 +797,7 @@ static int ReadOneLayer( Image* image, XCFDocInfo* inDocInfo, XCFLayerInfo*
 
   /* set the compositing mode */
   outLayer->image->compose = GIMPBlendModeToCompositeOperator( outLayer->mode );
-  if ( outLayer->visible == False )
+  if ( outLayer->visible == MagickFalse )
     {
       /* BOGUS: should really be separate member var! */
       outLayer->image->compose = NoCompositeOp;
@@ -813,7 +810,7 @@ static int ReadOneLayer( Image* image, XCFDocInfo* inDocInfo, XCFLayerInfo*
   /* read in the hierarchy */
   (void) SeekBlob(image, (MagickOffsetType) hierarchy_offset, SEEK_SET);
   if (load_hierarchy (image, inDocInfo, outLayer) == 0)
-    return 0;
+    return(MagickFalse);
 
   /* read in the layer mask */
   if (layer_mask_offset != 0)
@@ -829,7 +826,7 @@ static int ReadOneLayer( Image* image, XCFDocInfo* inDocInfo, XCFLayerInfo*
       GIMP_DRAWABLE (layer_mask)->offset_x = GIMP_DRAWABLE (layer)->offset_x;
       GIMP_DRAWABLE (layer_mask)->offset_y = GIMP_DRAWABLE (layer)->offset_y;
 
-      gimp_layer_add_mask (layer, layer_mask, False);
+      gimp_layer_add_mask (layer, layer_mask, MagickFalse);
 
       layer->mask->apply_mask = apply_mask;
       layer->mask->edit_mask  = edit_mask;
@@ -848,7 +845,7 @@ static int ReadOneLayer( Image* image, XCFDocInfo* inDocInfo, XCFLayerInfo*
     }
 #endif
 
-  return True;
+  return MagickTrue;
 }
 
 /*
@@ -872,10 +869,6 @@ static int ReadOneLayer( Image* image, XCFDocInfo* inDocInfo, XCFLayerInfo*
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadXCFImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a memory shortage or
-%      if the image cannot be read.
-%
 %    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
@@ -893,12 +886,12 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   int
     foundPropEnd = 0;
 
+  MagickBooleanType
+    status;
+
   register long
     i;
 
-  unsigned int
-    status;
-
   unsigned long
     image_type;
 
@@ -913,15 +906,15 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
-  if (image_info->debug != False)
+  if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
+  if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   count=ReadBlob(image,14,(unsigned char *) magick);
@@ -950,11 +943,11 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if ( image_type == GIMP_INDEXED )
         ThrowReaderException(CoderError,"ColormapTypeNotSupported");
   SetImage(image,OpaqueOpacity);
-  image->matte=True;
+  image->matte=MagickTrue;
   /*
 	  Read properties.
   */
-  while (foundPropEnd == False)
+  while (foundPropEnd == MagickFalse)
 	{
     PropType prop_type = (PropType) ReadBlobMSBLong(image);
     unsigned long  prop_size = ReadBlobMSBLong(image);
@@ -972,7 +965,7 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 	 */
 	unsigned long num_colours = ReadBlobMSBLong(image);
 	
-        for (i=0; i < (long) 3*num_colours; i++ )
+        for (i=0; i < (long) (3L*num_colours); i++ )
           (void) ReadBlobByte(image);
     /*
       if (info->file_version == 0)
@@ -1120,9 +1113,8 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       {
         amount = (long) Min (16, prop_size);
         amount = (long) ReadBlob(image,(size_t) amount, (unsigned char *) &buf);
-	if (!amount)
-	  ThrowBinaryException(CorruptImageError,"CorruptImage",
-	                       image->filename);
+        if (!amount)
+          ThrowReaderException(CorruptImageError,"CorruptImage");
         prop_size -= Min (16, amount);
       }
     }
@@ -1130,13 +1122,13 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   }
   }
 
-  if ((image_info->ping != False) && (image_info->number_scenes != 0)) {
+  if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0)) {
   ; /* do nothing, we were just pinging! */
   } else {
     XCFLayerInfo*  layer_info;
       int  number_layers = 0,
       current_layer = 0,
-      foundAllLayers = False;
+      foundAllLayers = MagickFalse;
 
       /* BIG HACK
         because XCF doesn't include the layer count, and we
@@ -1148,10 +1140,10 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     do {
       long  offset = (long) ReadBlobMSBLong(image);
       if ( offset == 0 )
-        foundAllLayers = True;
+        foundAllLayers = MagickTrue;
       else
         number_layers++;
-    } while (foundAllLayers  == False);
+    } while (foundAllLayers  == MagickFalse);
     (void) SeekBlob(image, oldPos, SEEK_SET); /* restore the position! */
 
 
@@ -1189,10 +1181,10 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
       /* read in the layer */
       layer_ok = ReadOneLayer( image, &doc_info, &layer_info[current_layer] );
-      if (layer_ok == 0) {
+      if (layer_ok == MagickFalse) {
         int j;
         for (j=0; j < current_layer; j++)
-          DestroyImage(layer_info[j].image);
+          layer_info[j].image=DestroyImage(layer_info[j].image);
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       }
 
@@ -1208,12 +1200,14 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       /* composite the layer data onto the main image & then dispose the layer */
       (void) CompositeImage(image, OverCompositeOp, layer_info[0].image,
                layer_info[0].offset_x, layer_info[0].offset_y );
-      DestroyImage( layer_info[0].image );
+       layer_info[0].image =DestroyImage( layer_info[0].image );
 
       /* Bob says that if we do this, we'll get REAL gray images! */
       if ( image_type == GIMP_GRAY ) {
         QuantizeInfo  qi;
         GetQuantizeInfo(&qi);
+        qi.dither=IsPaletteImage(image,&image->exception) == MagickFalse ?
+          MagickTrue : MagickFalse;
         qi.colorspace = GRAYColorspace;
         (void) QuantizeImage( &qi, image );
       }
@@ -1228,7 +1222,7 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if ( layer_info[j].visible ) { /* only visible ones, please! */
             CompositeImage(image, OverCompositeOp, layer_info[j].image,
                      layer_info[j].offset_x, layer_info[j].offset_y );
-            DestroyImage( layer_info[j].image );
+             layer_info[j].image =DestroyImage( layer_info[j].image );
 
             /* Bob says that if we do this, we'll get REAL gray images! */
             if ( image_type == GIMP_GRAY ) {
@@ -1249,7 +1243,7 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         (void) CompositeImage(image, CopyCompositeOp, layer_info[number_layers-1].image,
                  layer_info[number_layers-1].offset_x,
                  layer_info[number_layers-1].offset_y );
-        DestroyImage( layer_info[number_layers-1].image );
+         layer_info[number_layers-1].image =DestroyImage( layer_info[number_layers-1].image );
 
         /* Bob says that if we do this, we'll get REAL gray images! */
         if ( image_type == GIMP_GRAY ) {
@@ -1290,7 +1284,7 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     layer_info=(XCFLayerInfo *) RelinquishMagickMemory(layer_info);
 
 #if 0  /* BOGUS: do we need the channels?? */
-    while (True)
+    while (MagickTrue)
     {
       /* read in the offset of the next channel */
       info->cp += xcf_read_int32 (info->fp, &offset, 1);
@@ -1365,7 +1359,7 @@ ModuleExport void RegisterXCFImage(void)
   entry->magick=(MagickHandler *) IsXCF;
   entry->description=AcquireString("GIMP image");
   entry->module=AcquireString("XCF");
-  entry->seekable_stream=True;
+  entry->seekable_stream=MagickTrue;
   (void) RegisterMagickInfo(entry);
 }
 
