@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,12 +41,15 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/color.h"
 #include "magick/color_private.h"
 #include "magick/colorspace.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/geometry.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -62,7 +65,7 @@
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WritePICONImage(const ImageInfo *,Image *),
   WriteXPMImage(const ImageInfo *,Image *);
 
@@ -77,17 +80,15 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  IsXPM() returns True if the image format type, identified by the
+%  IsXPM() returns MagickTrue if the image format type, identified by the
 %  magick string, is XPM.
 %
 %  The format of the IsXPM method is:
 %
-%      unsigned int IsXPM(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsXPM(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsXPM() returns True if the image format type is XPM.
-%
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
 %
@@ -95,13 +96,13 @@ static unsigned int
 %
 %
 */
-static unsigned int IsXPM(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsXPM(const unsigned char *magick,const size_t length)
 {
   if (length < 9)
-    return(False);
+    return(MagickFalse);
   if (LocaleNCompare((char *) magick+1,"* XPM *",7) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -125,10 +126,6 @@ static unsigned int IsXPM(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadXPMImage returns a pointer to the image after
-%      creating it. A null image is returned if there is a memory shortage
-%      or if the image cannot be read.
-%
 %    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
@@ -160,7 +157,7 @@ static char *ParseColor(char *data)
     {
       if (*r != *q)
         continue;
-      if (isspace((int) ((unsigned char) (*(r-1)))) == False)
+      if (isspace((int) ((unsigned char) (*(r-1)))) == 0)
         continue;
       p=r;
       for ( ; ; )
@@ -196,6 +193,9 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     none,
     y;
 
+  MagickBooleanType
+    status;
+
   register char
     *p,
     *q;
@@ -215,9 +215,6 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   size_t
     length;
 
-  unsigned int
-    status;
-
   unsigned long
     width;
 
@@ -226,15 +223,15 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
-  if (image_info->debug != False)
+  if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
+  if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
@@ -252,10 +249,11 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if ((*p == '}') && (*(p+1) == ';'))
         break;
       p+=strlen(p);
-      if (((size_t) (p-xpm_buffer)+MaxTextExtent+1) < length)
+      if ((size_t) (p-xpm_buffer+MaxTextExtent) < length)
         continue;
       length<<=1;
-      xpm_buffer=(char *) ResizeMagickMemory(xpm_buffer,(size_t) length);
+      xpm_buffer=(char *) ResizeMagickMemory(xpm_buffer,
+        (length+MaxTextExtent)*sizeof(*xpm_buffer));
       if (xpm_buffer == (char *) NULL)
         break;
       p=xpm_buffer+strlen(xpm_buffer);
@@ -290,7 +288,7 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     for (q=p+1; *q != '\0'; q++)
       if (*q == '"')
         break;
-    (void) strncpy(xpm_buffer+i,p+1,(size_t) (q-p)-1);
+    (void) CopyMagickString(xpm_buffer+i,p+1,(size_t) (q-p));
     i+=q-p-1;
     xpm_buffer[i++]='\n';
     p=q+1;
@@ -303,8 +301,8 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Initialize image structure.
   */
-  keys=(char **) AcquireMagickMemory((size_t) image->colors*sizeof(char *));
-  if ((AllocateImageColormap(image,image->colors) == False) ||
+  keys=(char **) AcquireMagickMemory((size_t) image->colors*sizeof(*keys));
+  if ((AllocateImageColormap(image,image->colors) == MagickFalse) ||
       (keys == (char **) NULL))
     {
       for (i=0; textlist[i] != (char *) NULL; i++)
@@ -331,8 +329,7 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
         keys=(char **) RelinquishMagickMemory(keys);
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       }
-    keys[j][width]='\0';
-    (void) strncpy(keys[j],p,(size_t) width);
+    (void) CopyMagickString(keys[j],p,(size_t) width+1);
     /*
       Parse color.
     */
@@ -340,7 +337,7 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     q=ParseColor(p+width);
     if (q != (char *) NULL)
       {
-        while ((isspace((int) ((unsigned char) *q)) == False) && (*q != '\0'))
+        while ((isspace((int) ((unsigned char) *q)) == 0) && (*q != '\0'))
           q++;
         (void) CopyMagickString(target,q,MaxTextExtent);
         q=ParseColor(target);
@@ -351,11 +348,11 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (LocaleCompare(target,"none") == 0)
       {
         image->storage_class=DirectClass;
-        image->matte=True;
+        image->matte=MagickTrue;
         none=j;
         (void) strcpy(target,"black");
       }
-    if (QueryColorDatabase(target,&image->colormap[j],exception) == False)
+    if (QueryColorDatabase(target,&image->colormap[j],exception) == MagickFalse)
       break;
   }
   if (j < (long) image->colors)
@@ -367,7 +364,7 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
   j=0;
   key[width]='\0';
-  if (image_info->ping == False)
+  if (image_info->ping == MagickFalse)
     {
       /*
         Read image pixels.
@@ -383,7 +380,7 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
         indexes=GetIndexes(image);
         for (x=0; x < (long) image->columns; x++)
         {
-          (void) strncpy(key,p,(size_t) width);
+          (void) CopyMagickString(key,p,(size_t) width+1);
           if (strcmp(key,keys[j]) != 0)
             for (j=0; j < (long) Max(image->colors-1,1); j++)
               if (strcmp(key,keys[j]) == 0)
@@ -396,7 +393,7 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           r++;
           p+=width;
         }
-        if (SyncImagePixels(image) == False)
+        if (SyncImagePixels(image) == MagickFalse)
           break;
       }
       if (y < (long) image->rows)
@@ -446,15 +443,15 @@ ModuleExport void RegisterXPMImage(void)
   entry=SetMagickInfo("PICON");
   entry->decoder=(DecoderHandler *) ReadXPMImage;
   entry->encoder=(EncoderHandler *) WritePICONImage;
-  entry->adjoin=False;
+  entry->adjoin=MagickFalse;
   entry->description=AcquireString("Personal Icon");
   entry->module=AcquireString("XPM");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PM");
   entry->decoder=(DecoderHandler *) ReadXPMImage;
   entry->encoder=(EncoderHandler *) WriteXPMImage;
-  entry->adjoin=False;
-  entry->stealth=True;
+  entry->adjoin=MagickFalse;
+  entry->stealth=MagickTrue;
   entry->description=AcquireString("X Windows system pixmap (color)");
   entry->module=AcquireString("XPM");
   (void) RegisterMagickInfo(entry);
@@ -462,7 +459,7 @@ ModuleExport void RegisterXPMImage(void)
   entry->decoder=(DecoderHandler *) ReadXPMImage;
   entry->encoder=(EncoderHandler *) WriteXPMImage;
   entry->magick=(MagickHandler *) IsXPM;
-  entry->adjoin=False;
+  entry->adjoin=MagickFalse;
   entry->description=AcquireString("X Windows system pixmap (color)");
   entry->module=AcquireString("XPM");
   (void) RegisterMagickInfo(entry);
@@ -510,21 +507,18 @@ ModuleExport void UnregisterXPMImage(void)
 %
 %  The format of the WritePICONImage method is:
 %
-%      unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WritePICONImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WritePICONImage return True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
-%
 %    o image_info: The image info.
 %
 %    o image:  The image.
 %
 %
 */
-static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WritePICONImage(const ImageInfo *image_info,
+  Image *image)
 {
 #define ColormapExtent  155
 #define GraymapExtent  95
@@ -583,6 +577,10 @@ static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
     k,
     y;
 
+  MagickBooleanType
+    status,
+    transparent;
+
   RectangleInfo
     geometry;
 
@@ -599,10 +597,6 @@ static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
   register PixelPacket
     *q;
 
-  unsigned int
-    status,
-    transparent;
-
   unsigned long
     characters_per_pixel,
     colors;
@@ -614,10 +608,10 @@ static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  if (image->debug != False)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
+  if (status == MagickFalse)
     return(status);
   (void) SetImageColorspace(image,RGBColorspace);
   SetGeometry(image,&geometry);
@@ -628,29 +622,29 @@ static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
   blob_info=CloneImageInfo(image_info);
   (void) AcquireUniqueFilename(blob_info->filename);
   if ((image_info->type != TrueColorType) &&
-      (IsGrayImage(image,&image->exception) != False))
+      (IsGrayImage(image,&image->exception) != MagickFalse))
     map=BlobToImage(blob_info,Graymap,GraymapExtent,&image->exception);
   else
     map=BlobToImage(blob_info,Colormap,ColormapExtent,&image->exception);
-  DestroyImageInfo(blob_info);
-  if ((picon == (Image *) NULL) || (map == (Image *) NULL))
-    return(False);
-  status=MapImage(picon,map,image_info->dither);
   (void) RelinquishUniqueFileResource(blob_info->filename);
-  DestroyImage(map);
-  transparent=False;
+  blob_info=DestroyImageInfo(blob_info);
+  if ((picon == (Image *) NULL) || (map == (Image *) NULL))
+    return(MagickFalse);
+  status=MapImage(picon,map,image_info->dither);
+  map=DestroyImage(map);
+  transparent=MagickFalse;
   if (picon->storage_class == PseudoClass)
     {
       CompressImageColormap(picon);
-      if (picon->matte != False)
-        transparent=True;
+      if (picon->matte != MagickFalse)
+        transparent=MagickTrue;
     }
   else
     {
       /*
         Convert DirectClass to PseudoClass picon.
       */
-      if (picon->matte != False)
+      if (picon->matte != MagickFalse)
         {
           /*
             Map all the transparent pixels.
@@ -663,23 +657,23 @@ static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
             for (x=0; x < (long) picon->columns; x++)
             {
               if (q->opacity == TransparentOpacity)
-                transparent=True;
+                transparent=MagickTrue;
               else
                 q->opacity=OpaqueOpacity;
               q++;
             }
-            if (SyncImagePixels(picon) == False)
+            if (SyncImagePixels(picon) == MagickFalse)
               break;
           }
         }
       (void) SetImageType(picon,PaletteType);
     }
   colors=picon->colors;
-  if (transparent != False)
+  if (transparent != MagickFalse)
     {
       colors++;
       picon->colormap=(PixelPacket *) ResizeMagickMemory((void **)
-        &picon->colormap,(size_t) colors*sizeof(PixelPacket));
+        &picon->colormap,(size_t) colors*sizeof(*picon->colormap));
       if (picon->colormap == (PixelPacket *) NULL)
         ThrowWriterException(ResourceLimitError,"MemoryAllocationError");
       for (y=0; y < (long) picon->rows; y++)
@@ -694,7 +688,7 @@ static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
             indexes[x]=(IndexPacket) picon->colors;
           q++;
         }
-        if (SyncImagePixels(picon) == False)
+        if (SyncImagePixels(picon) == MagickFalse)
           break;
       }
     }
@@ -724,7 +718,7 @@ static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
     picon->colormap[i].opacity=OpaqueOpacity;
     (void) QueryColorname(picon,picon->colormap+i,XPMCompliance,name,
       &picon->exception);
-    if (transparent != False)
+    if (transparent != MagickFalse)
       {
         if (i == (long) (colors-1))
           (void) strcpy(name,"grey75");
@@ -771,14 +765,20 @@ static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
     (void) FormatMagickString(buffer,MaxTextExtent,"\"%s\n",
       (y == (long) (picon->rows-1) ? "" : ","));
     (void) WriteBlobString(image,buffer);
-    if (QuantumTick(y,picon->rows))
-      if (MagickMonitor(SaveImageTag,y,picon->rows,&image->exception) == False)
-        break;
+    if (QuantumTick(y,picon->rows) != MagickFalse)
+      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+          (QuantumTick(y,picon->rows) != MagickFalse))
+        {
+          status=image->progress_monitor(SaveImageTag,y,picon->rows,
+            image->client_data);
+          if (status == MagickFalse)
+            break;
+        }
   }
-  DestroyImage(picon);
+  picon=DestroyImage(picon);
   (void) WriteBlobString(image,"};\n");
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
 
 /*
@@ -796,21 +796,17 @@ static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
 %
 %  The format of the WriteXPMImage method is:
 %
-%      unsigned int WriteXPMImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteXPMImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteXPMImage return True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
-%
 %    o image_info: The image info.
 %
 %    o image:  The image.
 %
 %
 */
-static unsigned int WriteXPMImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteXPMImage(const ImageInfo *image_info,Image *image)
 {
 #define MaxCixels  92
 
@@ -829,6 +825,10 @@ static unsigned int WriteXPMImage(const ImageInfo *image_info,Image *image)
     k,
     y;
 
+  MagickBooleanType
+    status,
+    transparent;
+
   register const PixelPacket
     *p;
 
@@ -842,10 +842,6 @@ static unsigned int WriteXPMImage(const ImageInfo *image_info,Image *image)
   register PixelPacket
     *q;
 
-  unsigned int
-    status,
-    transparent;
-
   unsigned long
     characters_per_pixel,
     colors;
@@ -857,25 +853,25 @@ static unsigned int WriteXPMImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  if (image->debug != False)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
+  if (status == MagickFalse)
     return(status);
   (void) SetImageColorspace(image,RGBColorspace);
-  transparent=False;
+  transparent=MagickFalse;
   if (image->storage_class == PseudoClass)
     {
       CompressImageColormap(image);
-      if (image->matte != False)
-        transparent=True;
+      if (image->matte != MagickFalse)
+        transparent=MagickTrue;
     }
   else
     {
       /*
         Convert DirectClass to PseudoClass image.
       */
-      if (image->matte != False)
+      if (image->matte != MagickFalse)
         {
           /*
             Map all the transparent pixels.
@@ -888,23 +884,23 @@ static unsigned int WriteXPMImage(const ImageInfo *image_info,Image *image)
             for (x=0; x < (long) image->columns; x++)
             {
               if (q->opacity == TransparentOpacity)
-                transparent=True;
+                transparent=MagickTrue;
               else
                 q->opacity=OpaqueOpacity;
               q++;
             }
-            if (SyncImagePixels(image) == False)
+            if (SyncImagePixels(image) == MagickFalse)
               break;
           }
         }
       (void) SetImageType(image,PaletteType);
     }
   colors=image->colors;
-  if (transparent != False)
+  if (transparent != MagickFalse)
     {
       colors++;
-      image->colormap=(PixelPacket *)
-        ResizeMagickMemory(image->colormap,(size_t) colors*sizeof(PixelPacket));
+      image->colormap=(PixelPacket *) ResizeMagickMemory(image->colormap,
+        (size_t) colors*sizeof(*image->colormap));
       if (image->colormap == (PixelPacket *) NULL)
         ThrowWriterException(ResourceLimitError,"MemoryAllocationError");
       for (y=0; y < (long) image->rows; y++)
@@ -919,7 +915,7 @@ static unsigned int WriteXPMImage(const ImageInfo *image_info,Image *image)
             indexes[x]=(IndexPacket) image->colors;
           q++;
         }
-        if (SyncImagePixels(image) == False)
+        if (SyncImagePixels(image) == MagickFalse)
           break;
       }
     }
@@ -949,7 +945,7 @@ static unsigned int WriteXPMImage(const ImageInfo *image_info,Image *image)
     image->colormap[i].opacity=OpaqueOpacity;
     (void) QueryColorname(image,image->colormap+i,XPMCompliance,name,
       &image->exception);
-    if (transparent != False)
+    if (transparent != MagickFalse)
       {
         if (i == (long) (colors-1))
           (void) strcpy(name,"None");
@@ -996,11 +992,17 @@ static unsigned int WriteXPMImage(const ImageInfo *image_info,Image *image)
     (void) FormatMagickString(buffer,MaxTextExtent,"\"%s\n",
       (y == (long) (image->rows-1) ? "" : ","));
     (void) WriteBlobString(image,buffer);
-    if (QuantumTick(y,image->rows))
-      if (MagickMonitor(SaveImageTag,y,image->rows,&image->exception) == False)
-        break;
+    if (QuantumTick(y,image->rows) != MagickFalse)
+      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+          (QuantumTick(y,image->rows) != MagickFalse))
+        {
+          status=image->progress_monitor(SaveImageTag,y,image->rows,
+            image->client_data);
+          if (status == MagickFalse)
+            break;
+        }
   }
   (void) WriteBlobString(image,"};\n");
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
