@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -51,7 +51,8 @@
 #include "magick/constitute.h"
 #include "magick/draw.h"
 #include "magick/draw_private.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/gem.h"
 #include "magick/geometry.h"
 #include "magick/log.h"
@@ -93,7 +94,7 @@
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   RenderType(Image *,const DrawInfo *,const PointInfo *,TypeMetric *),
   RenderPostscript(Image *,const DrawInfo *,const PointInfo *,TypeMetric *),
   RenderFreetype(Image *,const DrawInfo *,const char *,const PointInfo *,
@@ -140,7 +141,7 @@ static unsigned int
 %
 %  The format of the AnnotateImage method is:
 %
-%      unsigned int AnnotateImage(Image *image,DrawInfo *draw_info)
+%      MagickBooleanType AnnotateImage(Image *image,DrawInfo *draw_info)
 %
 %  A description of each parameter follows:
 %
@@ -150,7 +151,8 @@ static unsigned int
 %
 %
 */
-MagickExport unsigned int AnnotateImage(Image *image,const DrawInfo *draw_info)
+MagickExport MagickBooleanType AnnotateImage(Image *image,
+  const DrawInfo *draw_info)
 {
   char
     primitive[MaxTextExtent],
@@ -164,6 +166,9 @@ MagickExport unsigned int AnnotateImage(Image *image,const DrawInfo *draw_info)
   GeometryInfo
     geometry_info;
 
+  MagickBooleanType
+    status;
+
   PointInfo
     offset;
 
@@ -179,9 +184,6 @@ MagickExport unsigned int AnnotateImage(Image *image,const DrawInfo *draw_info)
   TypeMetric
     metrics;
 
-  unsigned int
-    status;
-
   unsigned long
     height,
     number_lines;
@@ -191,14 +193,14 @@ MagickExport unsigned int AnnotateImage(Image *image,const DrawInfo *draw_info)
   */
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  if (image->debug != False)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   assert(draw_info != (DrawInfo *) NULL);
   assert(draw_info->signature == MagickSignature);
   if (draw_info->text == (char *) NULL)
-    return(False);
+    return(MagickFalse);
   if (*draw_info->text == '\0')
-    return(False);
+    return(MagickFalse);
   text=TranslateText((ImageInfo *) NULL,image,draw_info->text);
   if (text == (char *) NULL)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
@@ -206,7 +208,7 @@ MagickExport unsigned int AnnotateImage(Image *image,const DrawInfo *draw_info)
   textlist=StringToList(text);
   text=(char *) RelinquishMagickMemory(text);
   if (textlist == (char **) NULL)
-    return(False);
+    return(MagickFalse);
   length=strlen(textlist[0]);
   for (i=1; textlist[i] != (char *) NULL; i++)
     if (strlen(textlist[i]) > length)
@@ -226,7 +228,7 @@ MagickExport unsigned int AnnotateImage(Image *image,const DrawInfo *draw_info)
       (void) ParseGeometry(annotate_info->geometry,&geometry_info);
     }
   image->storage_class=DirectClass;
-  status=True;
+  status=MagickTrue;
   for (i=0; textlist[i] != (char *) NULL; i++)
   {
     /*
@@ -413,7 +415,7 @@ MagickExport unsigned int AnnotateImage(Image *image,const DrawInfo *draw_info)
       Annotate image with text.
     */
     status=RenderType(image,annotate,&offset,&metrics);
-    if (status == False)
+    if (status == MagickFalse)
       break;
     if (annotate->decorate == LineThroughDecoration)
       {
@@ -426,8 +428,8 @@ MagickExport unsigned int AnnotateImage(Image *image,const DrawInfo *draw_info)
   /*
     Free resources.
   */
-  DestroyDrawInfo(annotate_info);
-  DestroyDrawInfo(annotate);
+  annotate_info=DestroyDrawInfo(annotate_info);
+  annotate=DestroyDrawInfo(annotate);
   text=(char *) RelinquishMagickMemory(text);
   for (i=0; textlist[i] != (char *) NULL; i++)
     textlist[i]=(char *) RelinquishMagickMemory(textlist[i]);
@@ -471,7 +473,7 @@ MagickExport unsigned int AnnotateImage(Image *image,const DrawInfo *draw_info)
 
 static int GetOneCharacter(const unsigned char *text,size_t *length)
 {
-  unsigned int
+  int
     c;
 
   if (*length < 1)
@@ -515,8 +517,7 @@ static unsigned short *EncodeSJIS(const Image *image,const char *text,
   if ((text == (char *) NULL) || (*text == '\0'))
     return((unsigned short *) NULL);
   length=strlen(text)+MaxTextExtent;
-  encoding=(unsigned short *)
-    AcquireMagickMemory(length*sizeof(unsigned short));
+  encoding=(unsigned short *) AcquireMagickMemory(length*sizeof(*encoding));
   if (encoding == (unsigned short *) NULL)
     ThrowMagickFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
       image->filename);
@@ -585,7 +586,7 @@ static unsigned short *EncodeText(const Image *image,const char *text,
   if ((text == (char *) NULL) || (*text == '\0'))
     return((unsigned short *) NULL);
   encoding=(unsigned short *)
-    AcquireMagickMemory((strlen(text)+MaxTextExtent)*sizeof(unsigned short));
+    AcquireMagickMemory((strlen(text)+MaxTextExtent)*sizeof(*encoding));
   if (encoding == (unsigned short *) NULL)
     ThrowMagickFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
       image->filename);
@@ -684,7 +685,7 @@ static long GetUnicodeCharacter(const unsigned char *text,size_t *length)
 static unsigned short *EncodeUnicode(const Image *image,const char *text,
   size_t *count)
 {
-  int
+  long
     c;
 
   register const char
@@ -703,8 +704,7 @@ static unsigned short *EncodeUnicode(const Image *image,const char *text,
   if ((text == (char *) NULL) || (*text == '\0'))
     return((unsigned short *) NULL);
   length=strlen(text)+MaxTextExtent;
-  unicode=(unsigned short *)
-    AcquireMagickMemory(length*sizeof(unsigned short));
+  unicode=(unsigned short *) AcquireMagickMemory(length*sizeof(*unicode));
   if (unicode == (unsigned short *) NULL)
     ThrowMagickFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
       image->filename);
@@ -733,6 +733,128 @@ static unsigned short *EncodeUnicode(const Image *image,const char *text,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   G e t M u l t i l i n e T y p e M e t r i c s                             %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetMultilineTypeMetrics() returns the following information for the
+%  specified font and text:
+%
+%    o character width
+%    o character height
+%    o ascent
+%    o descent
+%    o text width
+%    o text height
+%    o maximum horizontal advance
+%    o underline position
+%    o underline thickness
+%
+%  This method is like GetTypeMetrics() but it returns the maximum text width
+%  and height for multiple lines of text.
+%
+%  The format of the GetMultilineTypeMetrics method is:
+%
+%      MagickBooleanType GetMultilineTypeMetrics(Image *image,
+%        const DrawInfo *draw_info,TypeMetric *metrics)
+%
+%  A description of each parameter follows:
+%
+%    o image: The image.
+%
+%    o draw_info: The draw info.
+%
+%    o metrics: Return the font metrics in this structure.
+%
+*/
+MagickExport MagickBooleanType GetMultilineTypeMetrics(Image *image,
+  const DrawInfo *draw_info,TypeMetric *metrics)
+{
+  char
+    *text,
+    **textlist;
+
+  double
+    max_width;
+
+  DrawInfo
+    *annotate_info;
+
+  MagickBooleanType
+    status;
+
+  PointInfo
+    offset;
+
+  register long
+    i;
+
+  unsigned long
+    number_lines;
+
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+  assert(draw_info != (DrawInfo *) NULL);
+  assert(draw_info->text != (char *) NULL);
+  assert(draw_info->signature == MagickSignature);
+  if (*draw_info->text == '\0')
+    return(MagickFalse);
+  annotate_info=CloneDrawInfo((ImageInfo *) NULL,draw_info);
+  annotate_info->text=(char *) RelinquishMagickMemory(annotate_info->text);
+  /*
+    Substitute values for formating characters (%x, etc).
+  */
+  text=TranslateText((ImageInfo *) NULL,image,draw_info->text);
+  if (text == (char *) NULL)
+    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+      image->filename);
+  /*
+    Convert newlines to multiple lines of text.
+  */
+  textlist=StringToList(text);
+  text=(char *) RelinquishMagickMemory(text);
+  if (textlist == (char **) NULL)
+    return(MagickFalse);
+  annotate_info->render=MagickFalse;
+  (void) ResetMagickMemory(metrics,0,sizeof(*metrics));
+  offset.x=0.0;
+  offset.y=0.0;
+  /*
+    Find the widest of the text lines.
+  */
+  annotate_info->text = textlist[0];
+  status=RenderType(image,annotate_info,&offset,metrics);
+  max_width = metrics->width;
+  for (i=1; textlist[i] != (char *) NULL; i++)
+  {
+    annotate_info->text = textlist[i];
+    status=RenderType(image,annotate_info,&offset,metrics);
+    if (metrics->width > max_width)
+      max_width=metrics->width;
+  }
+  number_lines= i;
+  metrics->width=max_width;
+  metrics->height=number_lines*(long) (metrics->ascent-metrics->descent+0.5)+1;
+  /*
+    Free resources.
+  */
+  annotate_info->text=(char *) NULL;
+  annotate_info=DestroyDrawInfo(annotate_info);
+  for (i=0; textlist[i] != (char *) NULL; i++)
+    textlist[i]=(char *) RelinquishMagickMemory(textlist[i]);
+  textlist=(char **) RelinquishMagickMemory(textlist);
+  return(status);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   G e t T y p e M e t r i c s                                               %
 %                                                                             %
 %                                                                             %
@@ -754,7 +876,7 @@ static unsigned short *EncodeUnicode(const Image *image,const char *text,
 %
 %  The format of the GetTypeMetrics method is:
 %
-%      unsigned int GetTypeMetrics(Image *image,const DrawInfo *draw_info,
+%      MagickBooleanType GetTypeMetrics(Image *image,const DrawInfo *draw_info,
 %        TypeMetric *metrics)
 %
 %  A description of each parameter follows:
@@ -767,32 +889,32 @@ static unsigned short *EncodeUnicode(const Image *image,const char *text,
 %
 %
 */
-MagickExport unsigned int GetTypeMetrics(Image *image,const DrawInfo *draw_info,
-  TypeMetric *metrics)
+MagickExport MagickBooleanType GetTypeMetrics(Image *image,
+  const DrawInfo *draw_info,TypeMetric *metrics)
 {
   DrawInfo
     *annotate_info;
 
+  MagickBooleanType
+    status;
+
   PointInfo
     offset;
 
-  unsigned int
-    status;
-
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  if (image->debug != False)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   assert(draw_info != (DrawInfo *) NULL);
   assert(draw_info->text != (char *) NULL);
   assert(draw_info->signature == MagickSignature);
   annotate_info=CloneDrawInfo((ImageInfo *) NULL,draw_info);
-  annotate_info->render=False;
-  (void) ResetMagickMemory(metrics,0,sizeof(TypeMetric));
+  annotate_info->render=MagickFalse;
+  (void) ResetMagickMemory(metrics,0,sizeof(*metrics));
   offset.x=0.0;
   offset.y=0.0;
   status=RenderType(image,annotate_info,&offset,metrics);
-  DestroyDrawInfo(annotate_info);
+  annotate_info=DestroyDrawInfo(annotate_info);
   return(status);
 }
 
@@ -812,14 +934,11 @@ MagickExport unsigned int GetTypeMetrics(Image *image,const DrawInfo *draw_info,
 %
 %  The format of the RenderType method is:
 %
-%      unsigned int RenderType(Image *image,DrawInfo *draw_info,
+%      MagickBooleanType RenderType(Image *image,DrawInfo *draw_info,
 %        const PointInfo *offset,TypeMetric *metrics)
 %
 %  A description of each parameter follows:
 %
-%    o status: Method RenderType returns True if the text is rendered on the
-%      image, otherwise False.
-%
 %    o image: The image.
 %
 %    o draw_info: The draw info.
@@ -830,7 +949,7 @@ MagickExport unsigned int GetTypeMetrics(Image *image,const DrawInfo *draw_info,
 %
 %
 */
-static unsigned int RenderType(Image *image,const DrawInfo *draw_info,
+static MagickBooleanType RenderType(Image *image,const DrawInfo *draw_info,
   const PointInfo *offset,TypeMetric *metrics)
 {
   const TypeInfo
@@ -839,7 +958,7 @@ static unsigned int RenderType(Image *image,const DrawInfo *draw_info,
   DrawInfo
     *annotate_info;
 
-  unsigned int
+  MagickBooleanType
     status;
 
   type_info=(const TypeInfo *) NULL;
@@ -849,17 +968,19 @@ static unsigned int RenderType(Image *image,const DrawInfo *draw_info,
         return(RenderFreetype(image,draw_info,(char *) NULL,offset,metrics));
       if (*draw_info->font == '-')
         return(RenderX11(image,draw_info,offset,metrics));
+      if (IsAccessible(draw_info->font) != MagickFalse)
+        return(RenderFreetype(image,draw_info,(char *) NULL,offset,metrics));
       type_info=GetTypeInfo(draw_info->font,&image->exception);
       if (type_info == (const TypeInfo *) NULL)
-        if (IsAccessible(draw_info->font) != False)
-          return(RenderFreetype(image,draw_info,(char *) NULL,offset,metrics));
+        (void) ThrowMagickException(&image->exception,GetMagickModule(),
+          TypeWarning,"UnableToReadFont",draw_info->font);
     }
   if (type_info == (const TypeInfo *) NULL)
     type_info=GetTypeInfoByFamily(draw_info->family,draw_info->style,
       draw_info->stretch,draw_info->weight,&image->exception);
   if (type_info == (const TypeInfo *) NULL)
-    (void) ThrowMagickException(&image->exception,GetMagickModule(),
-      TypeWarning,"UnableToReadFont",draw_info->font);
+    (void) ThrowMagickException(&image->exception,GetMagickModule(),TypeWarning,
+      "UnableToReadFont",draw_info->font);
   if (type_info == (const TypeInfo *) NULL)
     type_info=GetTypeInfoByFamily("Arial",draw_info->style,draw_info->stretch,
       draw_info->weight,&image->exception);
@@ -872,7 +993,7 @@ static unsigned int RenderType(Image *image,const DrawInfo *draw_info,
   if (type_info->glyphs != (char *) NULL)
     (void) CloneString(&annotate_info->font,type_info->glyphs);
   status=RenderFreetype(image,annotate_info,type_info->encoding,offset,metrics);
-  DestroyDrawInfo(annotate_info);
+  annotate_info=DestroyDrawInfo(annotate_info);
   return(status);
 }
 
@@ -892,14 +1013,11 @@ static unsigned int RenderType(Image *image,const DrawInfo *draw_info,
 %
 %  The format of the RenderFreetype method is:
 %
-%      unsigned int RenderFreetype(Image *image,DrawInfo *draw_info,
+%      MagickBooleanType RenderFreetype(Image *image,DrawInfo *draw_info,
 %        const char *encoding,const PointInfo *offset,TypeMetric *metrics)
 %
 %  A description of each parameter follows:
 %
-%    o status: Method RenderFreetype returns True if the text is rendered on the
-%      image, otherwise False.
-%
 %    o image: The image.
 %
 %    o draw_info: The draw info.
@@ -978,7 +1096,7 @@ static int TraceQuadraticBezier(FT_Vector *control,FT_Vector *to,
   return(0);
 }
 
-static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
+static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
   const char *encoding,const PointInfo *offset,TypeMetric *metrics)
 {
 #if !defined(FT_OPEN_PATHNAME)
@@ -1034,6 +1152,9 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
   long
     y;
 
+  MagickBooleanType
+    active;
+
   MagickRealType
     fill_opacity;
 
@@ -1067,9 +1188,6 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
       0, 0
     };
 
-  unsigned int
-    active;
-
   unsigned short
     *text;
 
@@ -1092,7 +1210,7 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
       ThrowBinaryException(TypeError,"UnableToReadFont",draw_info->font);
     }
   if ((draw_info->metrics != (char *) NULL) &&
-      (IsAccessible(draw_info->metrics) != False))
+      (IsAccessible(draw_info->metrics) != MagickFalse))
     (void) FT_Attach_File(face,draw_info->metrics);
   if (face->num_charmaps != 0)
     status=FT_Set_Charmap(face,face->charmaps[0]);
@@ -1141,7 +1259,7 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
       GeometryInfo
         geometry_info;
 
-      unsigned int
+      MagickStatusType
         flags;
 
       flags=ParseGeometry(draw_info->density,&geometry_info);
@@ -1170,7 +1288,7 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
     {
       (void) FT_Done_Face(face);
       (void) FT_Done_FreeType(library);
-      return(True);
+      return(MagickTrue);
     }
   /*
     Convert text to 2-byte format as prescribed by the encoding.
@@ -1217,7 +1335,7 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
   /*
     Compute bounding box.
   */
-  if (image->debug != False)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(AnnotateEvent,GetMagickModule(),"Font %s; "
       "font-encoding %s; text-encoding %s; pointsize %g",
       draw_info->font != (char *) NULL ? draw_info->font : "none",
@@ -1237,7 +1355,7 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
   annotate_info=CloneDrawInfo((ImageInfo *) NULL,draw_info);
   (void) QueryColorDatabase("#000000ff",&annotate_info->fill,&image->exception);
   (void) CloneString(&annotate_info->primitive,"path '");
-  if (image->matte == False)
+  if (image->matte == MagickFalse)
     SetImageOpacity(image,OpaqueOpacity);
   for (i=0; i < (long) length; i++)
   {
@@ -1252,16 +1370,17 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
         origin.x+=kerning.x;
       }
     glyph.origin=origin;
-    if (draw_info->render != False)
+    if (draw_info->render != MagickFalse)
       status=FT_Load_Glyph(face,glyph.id,FT_LOAD_DEFAULT);
     else
       status=FT_Load_Glyph(face,glyph.id,FT_LOAD_NO_BITMAP);
-    if (status != False)
+    if (status != MagickFalse)
       continue;
     status=FT_Get_Glyph(face->glyph,&glyph.image);
-    if (status != False)
+    if (status != MagickFalse)
       continue;
-    FT_Outline_Get_BBox(&((FT_OutlineGlyph) glyph.image)->outline,&bounds);
+    (void) FT_Outline_Get_BBox(&((FT_OutlineGlyph) glyph.image)->outline,
+      &bounds);
     if ((i == 0) || (bounds.xMin < metrics->bounds.x1))
       metrics->bounds.x1=bounds.xMin;
     if ((i == 0) || (bounds.yMin < metrics->bounds.y1))
@@ -1270,7 +1389,7 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
       metrics->bounds.x2=bounds.xMax;
     if ((i == 0) || (bounds.yMax > metrics->bounds.y2))
       metrics->bounds.y2=bounds.yMax;
-    if (draw_info->render != False)
+    if (draw_info->render != MagickFalse)
       if ((draw_info->stroke.opacity != TransparentOpacity) ||
           (draw_info->stroke_pattern != (Image *) NULL))
         {
@@ -1284,14 +1403,14 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
         }
     FT_Vector_Transform(&glyph.origin,&affine);
     (void) FT_Glyph_Transform(glyph.image,&affine,&glyph.origin);
-    if (draw_info->render != False)
+    if (draw_info->render != MagickFalse)
       {
         /*
           Rasterize the glyph.
         */
         status=FT_Glyph_To_Bitmap(&glyph.image,ft_render_mode_normal,
-          (FT_Vector *) NULL,True);
-        if (status != False)
+          (FT_Vector *) NULL,MagickTrue);
+        if (status != MagickFalse)
           continue;
         bitmap=(FT_BitmapGlyph) glyph.image;
         image->storage_class=DirectClass;
@@ -1308,7 +1427,7 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
             }
           q=GetImagePixels(image,(long) (point.x+0.5),(long) (point.y+y+0.5),
             bitmap->bitmap.width,1);
-          active=q != (PixelPacket *) NULL;
+          active=q != (PixelPacket *) NULL ? MagickTrue : MagickFalse;
           for (x=0; x < (long) bitmap->bitmap.width; x++)
           {
             if (((long) (point.x+x+0.5) < 0) || (*p == 0) ||
@@ -1322,7 +1441,7 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
               fill_opacity=(MagickRealType) (*p)/255.0;
             else
               fill_opacity=(*p) > 127 ? 1.0 : 0.0;
-            if (active == False)
+            if (active == MagickFalse)
               q=GetImagePixels(image,(long) (point.x+x+0.5),(long)
                 (point.y+y+0.5),1,1);
             if (q == (PixelPacket *) NULL)
@@ -1334,13 +1453,13 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
             fill_color=GetFillColor(draw_info,(long) (point.x+x+0.5),(long)
               (point.y+y+0.5));
             fill_opacity=MaxRGB-fill_opacity*(MaxRGB-fill_color.opacity);
-            *q=MagickCompositeOver(&fill_color,fill_opacity,q,q->opacity);
-            if (active == False)
+            MagickCompositeOver(&fill_color,fill_opacity,q,q->opacity,q);
+            if (active == MagickFalse)
               (void) SyncImagePixels(image);
             p++;
             q++;
           }
-          if (SyncImagePixels(image) == False)
+          if (SyncImagePixels(image) == MagickFalse)
             break;
         }
       }
@@ -1356,7 +1475,7 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
   metrics->bounds.y1/=64.0;
   metrics->bounds.x2/=64.0;
   metrics->bounds.y2/=64.0;
-  if (draw_info->render != False)
+  if (draw_info->render != MagickFalse)
     if ((draw_info->stroke.opacity != TransparentOpacity) ||
         (draw_info->stroke_pattern != (Image *) NULL))
       {
@@ -1374,13 +1493,13 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
     Free resources.
   */
   text=(unsigned short *) RelinquishMagickMemory(text);
-  DestroyDrawInfo(annotate_info);
+  annotate_info=DestroyDrawInfo(annotate_info);
   (void) FT_Done_Face(face);
   (void) FT_Done_FreeType(library);
-  return(True);
+  return(MagickTrue);
 }
 #else
-static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
+static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
   const char *encoding,const PointInfo *offset,TypeMetric *metrics)
 {
   ThrowBinaryException(MissingDelegateError,"FreeTypeLibraryIsNotAvailable",
@@ -1404,14 +1523,11 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
 %
 %  The format of the RenderPostscript method is:
 %
-%      unsigned int RenderPostscript(Image *image,DrawInfo *draw_info,
+%      MagickBooleanType RenderPostscript(Image *image,DrawInfo *draw_info,
 %        const PointInfo *offset,TypeMetric *metrics)
 %
 %  A description of each parameter follows:
 %
-%    o status: Method RenderPostscript returns True if the text is rendered on
-%      the image, otherwise False.
-%
 %    o image: The image.
 %
 %    o draw_info: The draw info.
@@ -1453,8 +1569,8 @@ static char *EscapeParenthesis(const char *text)
   return(buffer);
 }
 
-static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
-  const PointInfo *offset,TypeMetric *metrics)
+static MagickBooleanType RenderPostscript(Image *image,
+  const DrawInfo *draw_info,const PointInfo *offset,TypeMetric *metrics)
 {
   char
     filename[MaxTextExtent],
@@ -1477,6 +1593,9 @@ static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
   long
     y;
 
+  MagickBooleanType
+    identity;
+
   PointInfo
     extent,
     point,
@@ -1489,13 +1608,10 @@ static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
   register PixelPacket
     *q;
 
-  unsigned int
-    identity;
-
   /*
     Render label with a Postscript font.
   */
-  if (image->debug != False)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(AnnotateEvent,GetMagickModule(),
       "Font %s; pointsize %g",draw_info->font != (char *) NULL ?
       draw_info->font : "none",draw_info->pointsize);
@@ -1507,7 +1623,7 @@ static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
     {
       ThrowFileException(&image->exception,FileOpenError,"UnableToOpenFile",
         filename);
-      return(False);
+      return(MagickFalse);
     }
   (void) fprintf(file,"%%!PS-Adobe-3.0\n");
   (void) fprintf(file,"/ReencodeType\n");
@@ -1521,8 +1637,9 @@ static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
   /*
     Sample to compute bounding box.
   */
-  identity=(unsigned int) ((draw_info->affine.sx == draw_info->affine.sy) &&
-    (draw_info->affine.rx == 0.0) && (draw_info->affine.ry == 0.0));
+  identity=(MagickBooleanType)
+    ((draw_info->affine.sx == draw_info->affine.sy) &&
+     (draw_info->affine.rx == 0.0) && (draw_info->affine.ry == 0.0));
   extent.x=0.0;
   extent.y=0.0;
   for (i=0; i <= (long) (strlen(draw_info->text)+2); i++)
@@ -1536,8 +1653,8 @@ static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
     if (point.y > extent.y)
       extent.y=point.y;
   }
-  (void) fprintf(file,"%g %g moveto\n",identity  != False ? 0.0 : extent.x/2.0,
-    extent.y/2.0);
+  (void) fprintf(file,"%g %g moveto\n",identity  != MagickFalse ? 0.0 :
+    extent.x/2.0,extent.y/2.0);
   (void) fprintf(file,"%g %g scale\n",draw_info->pointsize,
     draw_info->pointsize);
   if ((draw_info->font == (char *) NULL) || (*draw_info->font == '\0'))
@@ -1550,7 +1667,7 @@ static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
   (void) fprintf(file,"[%g %g %g %g 0 0] concat\n",draw_info->affine.sx,
     -draw_info->affine.rx,-draw_info->affine.ry,draw_info->affine.sy);
   text=EscapeParenthesis(draw_info->text);
-  if (identity == False)
+  if (identity == MagickFalse)
     (void) fprintf(file,"(%s) stringwidth pop -0.5 mul -0.5 rmoveto\n",
       text);
   (void) fprintf(file,"(%s) show\n",text);
@@ -1567,13 +1684,11 @@ static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
     (void) CloneString(&annotate_info->density,draw_info->density);
   annotate_info->antialias=draw_info->text_antialias;
   annotate_image=ReadImage(annotate_info,&image->exception);
-  DestroyImageInfo(annotate_info);
+  CatchException(&image->exception);
+  annotate_info=DestroyImageInfo(annotate_info);
   (void) RelinquishUniqueFileResource(filename);
-  if (image->exception.severity != UndefinedException)
-    MagickError(image->exception.severity,image->exception.reason,
-      image->exception.description);
   if (annotate_image == (Image *) NULL)
-    return(False);
+    return(MagickFalse);
   resolution.x=72.0;
   resolution.y=72.0;
   if (draw_info->density != (char *) NULL)
@@ -1581,7 +1696,7 @@ static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
       GeometryInfo
         geometry_info;
 
-      unsigned int
+      MagickStatusType
         flags;
 
       flags=ParseGeometry(draw_info->density,&geometry_info);
@@ -1590,8 +1705,8 @@ static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
       if ((flags & SigmaValue) == 0)
         resolution.y=resolution.x;
     }
-  if (identity == False)
-    TransformImage(&annotate_image,"0x0",(char *) NULL);
+  if (identity == MagickFalse)
+    (void) TransformImage(&annotate_image,"0x0",(char *) NULL);
   else
     {
       RectangleInfo
@@ -1603,7 +1718,7 @@ static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
       crop_info.y=(long) ((resolution.y/72.0)*extent.y/8.0+0.5);
       (void) FormatMagickString(geometry,MaxTextExtent,"%lux%lu%+ld%+ld",
         crop_info.width,crop_info.height,crop_info.x,crop_info.y);
-      TransformImage(&annotate_image,geometry,(char *) NULL);
+      (void) TransformImage(&annotate_image,geometry,(char *) NULL);
     }
   metrics->pixels_per_em.x=(resolution.y/72.0)*
     ExpandAffine(&draw_info->affine)*draw_info->pointsize;
@@ -1620,10 +1735,10 @@ static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
   metrics->bounds.y2=metrics->ascent+metrics->descent;
   metrics->underline_position=(-2.0);
   metrics->underline_thickness=1.0;
-  if (draw_info->render == False)
+  if (draw_info->render == MagickFalse)
     {
-      DestroyImage(annotate_image);
-      return(True);
+      annotate_image=DestroyImage(annotate_image);
+      return(MagickTrue);
     }
   if (draw_info->fill.opacity != TransparentOpacity)
     {
@@ -1633,9 +1748,9 @@ static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
       /*
         Render fill color.
       */
-      if (image->matte == False)
+      if (image->matte == MagickFalse)
         SetImageOpacity(image,OpaqueOpacity);
-      if (annotate_image->matte == False)
+      if (annotate_image->matte == MagickFalse)
         SetImageOpacity(annotate_image,OpaqueOpacity);
       fill_color=draw_info->fill;
       pattern=draw_info->fill_pattern;
@@ -1659,15 +1774,15 @@ static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
           q->blue=fill_color.blue;
           q++;
         }
-        if (SyncImagePixels(annotate_image) == False)
+        if (SyncImagePixels(annotate_image) == MagickFalse)
           break;
       }
       (void) CompositeImage(image,OverCompositeOp,annotate_image,
         (long) (offset->x+0.5),(long) (offset->y-(metrics->ascent+
         metrics->descent)+0.5));
     }
-  DestroyImage(annotate_image);
-  return(True);
+  annotate_image=DestroyImage(annotate_image);
+  return(MagickTrue);
 }
 
 /*
@@ -1686,14 +1801,11 @@ static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
 %
 %  The format of the RenderX11 method is:
 %
-%      unsigned int RenderX11(Image *image,DrawInfo *draw_info,
+%      MagickBooleanType RenderX11(Image *image,DrawInfo *draw_info,
 %        const PointInfo *offset,TypeMetric *metrics)
 %
 %  A description of each parameter follows:
 %
-%    o status: Method RenderX11 returns True if the text is rendered on the
-%      image, otherwise False.
-%
 %    o image: The image.
 %
 %    o draw_info: The draw info.
@@ -1705,9 +1817,12 @@ static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
 %
 */
 #if defined(HasX11)
-static unsigned int RenderX11(Image *image,const DrawInfo *draw_info,
+static MagickBooleanType RenderX11(Image *image,const DrawInfo *draw_info,
   const PointInfo *offset,TypeMetric *metrics)
 {
+  MagickBooleanType
+    status;
+
   static DrawInfo
     cache_info;
 
@@ -1735,9 +1850,6 @@ static unsigned int RenderX11(Image *image,const DrawInfo *draw_info,
   static XVisualInfo
     *visual_info;
 
-  unsigned int
-    status;
-
   unsigned long
     height,
     width;
@@ -1757,7 +1869,7 @@ static unsigned int RenderX11(Image *image,const DrawInfo *draw_info,
       (void) XSetErrorHandler(XError);
       resource_database=XGetResourceDatabase(display,GetClientName());
       XGetResourceInfo(resource_database,GetClientName(),&resource_info);
-      resource_info.close_server=False;
+      resource_info.close_server=MagickFalse;
       resource_info.colormap=PrivateColormap;
       resource_info.font=AcquireString(draw_info->font);
       resource_info.background_color=AcquireString("#ffffffffffff");
@@ -1786,7 +1898,7 @@ static unsigned int RenderX11(Image *image,const DrawInfo *draw_info,
       /*
         Initialize font info.
       */
-      font_info=XBestFont(display,&resource_info,False);
+      font_info=XBestFont(display,&resource_info,MagickFalse);
       if (font_info == (XFontStruct *) NULL)
         ThrowBinaryException(XServerError,"UnableToLoadFont",draw_info->font);
       if ((map_info == (XStandardColormap *) NULL) ||
@@ -1811,11 +1923,11 @@ static unsigned int RenderX11(Image *image,const DrawInfo *draw_info,
       */
       (void) XFreeFont(display,font_info);
       (void) CloneString(&resource_info.font,draw_info->font);
-      font_info=XBestFont(display,&resource_info,False);
+      font_info=XBestFont(display,&resource_info,MagickFalse);
       if (font_info == (XFontStruct *) NULL)
         ThrowBinaryException(XServerError,"UnableToLoadFont",draw_info->font);
     }
-  if (image->debug != False)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(AnnotateEvent,GetMagickModule(),
       "Font %s; pointsize %g",draw_info->font != (char *) NULL ?
       draw_info->font : "none",draw_info->pointsize);
@@ -1838,10 +1950,10 @@ static unsigned int RenderX11(Image *image,const DrawInfo *draw_info,
   metrics->bounds.y2=metrics->ascent+metrics->descent;
   metrics->underline_position=(-2.0);
   metrics->underline_thickness=1.0;
-  if (draw_info->render == False)
-    return(True);
+  if (draw_info->render == MagickFalse)
+    return(MagickTrue);
   if (draw_info->fill.opacity == TransparentOpacity)
-    return(True);
+    return(MagickTrue);
   /*
     Render fill color.
   */
@@ -1864,10 +1976,10 @@ static unsigned int RenderX11(Image *image,const DrawInfo *draw_info,
   if (status == 0)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
-  return(True);
+  return(MagickTrue);
 }
 #else
-static unsigned int RenderX11(Image *image,const DrawInfo *draw_info,
+static MagickBooleanType RenderX11(Image *image,const DrawInfo *draw_info,
   const PointInfo *offset,TypeMetric *metrics)
 {
   ThrowBinaryException(MissingDelegateError,"XWindowLibraryIsNotAvailable",
