@@ -1,5 +1,19 @@
 /*
-  ImageMagick Image Methods.
+  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization
+  dedicated to making software imaging solutions freely available.
+  
+  You may not use this file except in compliance with the License.
+  obtain a copy of the License at
+  
+    http://www.imagemagick.org/www/Copyright.html
+  
+  Unless required by applicable law or agreed to in writing, software
+  distributed under the License is distributed on an "AS IS" BASIS,
+  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+  See the License for the specific language governing permissions and
+  limitations under the License.
+
+  ImageMagick image methods.
 */
 #ifndef _MAGICK_IMAGE_H
 #define _MAGICK_IMAGE_H
@@ -8,8 +22,10 @@
 extern "C" {
 #endif
 
-#define ColorMatch(p,q) (((p)->red == (q)->red) && \
-  ((p)->green == (q)->green) && ((p)->blue == (q)->blue))
+#include <magick/color.h>
+
+#define ColorMatch(p,q) ((MagickBooleanType) (((p)->red == (q)->red) && \
+  ((p)->green == (q)->green) && ((p)->blue == (q)->blue)))
 #define OpaqueOpacity  ((Quantum) 0UL)
 #define PixelIntensity(pixel) ((MagickRealType) \
   (0.299*(pixel)->red+0.587*(pixel)->green+0.114*(pixel)->blue+0.5))
@@ -28,24 +44,6 @@ typedef enum
   ImageTransmitType
 } TransmitType;
 
-extern MagickExport const char
-  *BackgroundColor,
-  *BorderColor,
-  *DefaultTileFrame,
-  *DefaultTileGeometry,
-  *DefaultTileLabel,
-  *ForegroundColor,
-  *MatteColor,
-  *LoadImageTag,
-  *LoadImagesTag,
-  *PSDensityGeometry,
-  *PSPageGeometry,
-  *SaveImageTag,
-  *SaveImagesTag;
-
-extern MagickExport const unsigned long
-  UndefinedCompressionQuality;
-
 extern MagickExport const PixelPacket
   *AcquireImagePixels(const Image *,const long,const long,const unsigned long,
     const unsigned long,ExceptionInfo *);
@@ -55,16 +53,20 @@ extern MagickExport ExceptionType
 
 extern MagickExport Image
   *AllocateImage(const ImageInfo *),
-  *AppendImages(const Image *,const unsigned int,ExceptionInfo *),
+  *AppendImages(const Image *,const MagickBooleanType,ExceptionInfo *),
   *AverageImages(const Image *,ExceptionInfo *),
   *CloneImage(const Image *,const unsigned long,const unsigned long,
-    const unsigned int,ExceptionInfo *),
+    const MagickBooleanType,ExceptionInfo *),
   *CombineImages(const Image *,const ChannelType,ExceptionInfo *),
+  *DestroyImage(Image *),
   *GetImageClipMask(const Image *,ExceptionInfo *),
+  *NewMagickImage(const ImageInfo *,const unsigned long,const unsigned long,
+    const MagickPixelPacket *),
   *ReferenceImage(Image *);
 
 extern MagickExport ImageInfo
-  *CloneImageInfo(const ImageInfo *);
+  *CloneImageInfo(const ImageInfo *),
+  *DestroyImageInfo(ImageInfo *);
 
 extern MagickExport ImageType
   GetImageType(const Image *,ExceptionInfo *);
@@ -72,27 +74,13 @@ extern MagickExport ImageType
 extern MagickExport IndexPacket
   *GetIndexes(const Image *);
 
-extern MagickExport PixelPacket
-  AcquireOnePixel(const Image *,const long,const long,ExceptionInfo *),
-  *GetImagePixels(Image *,const long,const long,const unsigned long,
-    const unsigned long),
-  GetOnePixel(Image *,const long,const long),
-  *GetPixels(const Image *),
-  *SetImagePixels(Image *,const long,const long,const unsigned long,
-    const unsigned long);
-
-extern MagickExport RectangleInfo
-  GetImageBoundingBox(const Image *,ExceptionInfo *exception);
-
-extern MagickExport unsigned int
+extern MagickExport MagickBooleanType
   AllocateImageColormap(Image *,const unsigned long),
-  AnimateImages(const ImageInfo *,Image *),
   ClipImage(Image *),
-  ClipPathImage(Image *,const char *,const unsigned int),
+  ClipPathImage(Image *,const char *,const MagickBooleanType),
   GradientImage(Image *,const PixelPacket *,const PixelPacket *),
   CycleColormapImage(Image *,const long),
-  DescribeImage(Image *,FILE *,const unsigned int),
-  DisplayImages(const ImageInfo *,Image *),
+  DescribeImage(Image *,FILE *,const MagickBooleanType),
   GetImageChannelExtrema(const Image *,const ChannelType,unsigned long *,
     unsigned long *,ExceptionInfo *),
   GetImageExtrema(const Image *,unsigned long *,unsigned long *,
@@ -108,7 +96,7 @@ extern MagickExport unsigned int
   SeparateImageChannel(Image *,const ChannelType),
   SetImageChannelDepth(Image *,const ChannelType,const unsigned long),
   SetImageClipMask(Image *,const Image *),
-  SetImageInfo(ImageInfo *,const unsigned int,ExceptionInfo *),
+  SetImageInfo(ImageInfo *,const MagickBooleanType,ExceptionInfo *),
   SetImageDepth(Image *,const unsigned long),
   SetImageType(Image *,const ImageType),
   SignatureImage(Image *),
@@ -116,19 +104,28 @@ extern MagickExport unsigned int
   StripImage(Image *),
   SyncImage(Image *),
   SyncImagePixels(Image *),
-  TextureImage(Image *,const Image *),
-  TransformColorspace(Image *,const ColorspaceType);
+  TextureImage(Image *,const Image *);
+
+extern MagickExport PixelPacket
+  AcquireOnePixel(const Image *,const long,const long,ExceptionInfo *),
+  *GetImagePixels(Image *,const long,const long,const unsigned long,
+    const unsigned long),
+  GetOnePixel(Image *,const long,const long),
+  *GetPixels(const Image *),
+  *SetImagePixels(Image *,const long,const long,const unsigned long,
+    const unsigned long);
+
+extern MagickExport RectangleInfo
+  GetImageBoundingBox(const Image *,ExceptionInfo *exception);
 
 extern MagickExport unsigned long
   GetImageChannelDepth(const Image *,const ChannelType,ExceptionInfo *),
   GetImageDepth(const Image *,ExceptionInfo *),
-  GetImageQuantumDepth(const Image *);
+  GetImageQuantumDepth(const Image *,const MagickBooleanType);
 
 extern MagickExport void
   AllocateNextImage(const ImageInfo *,Image *),
   DestroyColorList(void),
-  DestroyImage(Image *),
-  DestroyImageInfo(ImageInfo *),
   DestroyImagePixels(Image *),
   GetImageException(Image *,ExceptionInfo *),
   GetImageInfo(ImageInfo *),
