@@ -16,7 +16,7 @@
 %                                 July 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,7 +42,8 @@
 #include "magick/blob.h"
 #include "magick/client.h"
 #include "magick/configure.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/hashmap.h"
 #include "magick/magic.h"
 #include "magick/memory_.h"
@@ -54,7 +55,7 @@
 /*
   Define declarations.
 */
-#define MagicFilename  "magic.mgk"
+#define MagicFilename  "magic.xml"
 
 /*
   Static declarations.
@@ -105,7 +106,6 @@ static const char
     "  <magic name=\"MNG\" offset=\"0\" target=\"\\212MNG\\r\\n\\032\\n\" />"
     "  <magic name=\"MPC\" offset=\"0\" target=\"id=MagickCache\" />"
     "  <magic name=\"MPEG\" offset=\"0\" target=\"\\000\\000\\001\\263\" />"
-    "  <magic name=\"MVG\" offset=\"0\" target=\"push graphic-context\" />"
     "  <magic name=\"PCD\" offset=\"2048\" target=\"PCD_\" />"
     "  <magic name=\"PCL\" offset=\"0\" target=\"\\033E\\033\" />"
     "  <magic name=\"PCX\" offset=\"0\" target=\"\\012\\002\" />"
@@ -159,13 +159,13 @@ static LinkedListInfo
 static SemaphoreInfo
   *magic_semaphore = (SemaphoreInfo *) NULL;
 
-static unsigned int
-  active_magic = False;
+static volatile MagickBooleanType
+  instantiate_magic = MagickFalse;
 
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   InitializeMagicList(ExceptionInfo *),
   LoadMagicLists(const char *,ExceptionInfo *);
 
@@ -209,13 +209,12 @@ static void *DestroyMagicElement(void *magic_info)
 
 MagickExport void DestroyMagicList(void)
 {
-  AcquireSemaphoreInfo(&magic_semaphore);
+  magic_semaphore=AcquireSemaphoreInfo(magic_semaphore);
   if (magic_list != (LinkedListInfo *) NULL)
-    DestroyLinkedList(magic_list,DestroyMagicElement);
-  magic_list=(LinkedListInfo *) NULL;
-  active_magic=False;
-  RelinquishSemaphoreInfo(&magic_semaphore);
-  DestroySemaphoreInfo(&magic_semaphore);
+    magic_list=DestroyLinkedList(magic_list,DestroyMagicElement);
+  instantiate_magic=MagickFalse;
+  RelinquishSemaphoreInfo(magic_semaphore);
+  magic_semaphore=DestroySemaphoreInfo(magic_semaphore);
 }
 
 /*
@@ -254,18 +253,22 @@ MagickExport const MagicInfo *GetMagicInfo(const unsigned char *magic,
   register const MagicInfo
     *p;
 
-  if ((magic_list == (LinkedListInfo *) NULL) || (active_magic == False))
-    if (InitializeMagicList(exception) == False)
+  assert(exception != (ExceptionInfo *) NULL);
+  if ((magic_list == (LinkedListInfo *) NULL) ||
+      (instantiate_magic == MagickFalse))
+    if (InitializeMagicList(exception) == MagickFalse)
       return((const MagicInfo *) NULL);
   if ((magic_list == (LinkedListInfo *) NULL) ||
-      (IsLinkedListEmpty(magic_list) == True))
+      (IsLinkedListEmpty(magic_list) == MagickTrue))
     return((const MagicInfo *) NULL);
   if ((magic == (const unsigned char *) NULL) || (length == 0))
     return((const MagicInfo *) GetElementFromLinkedList(magic_list,0));
+  if (length == 0)
+    return((const MagicInfo *) NULL);
   /*
     Search for requested magic.
   */
-  AcquireSemaphoreInfo(&magic_semaphore);
+  magic_semaphore=AcquireSemaphoreInfo(magic_semaphore);
   ResetLinkedListIterator(magic_list);
   p=(const MagicInfo *) GetNextElementInLinkedList(magic_list);
   while (p != (const MagicInfo *) NULL)
@@ -276,7 +279,7 @@ MagickExport const MagicInfo *GetMagicInfo(const unsigned char *magic,
       break;
     p=(const MagicInfo *) GetNextElementInLinkedList(magic_list);
   }
-  RelinquishSemaphoreInfo(&magic_semaphore);
+  RelinquishSemaphoreInfo(magic_semaphore);
   return((const MagicInfo *) p);
 }
 
@@ -297,7 +300,7 @@ MagickExport const MagicInfo *GetMagicInfo(const unsigned char *magic,
 %  The magic of the GetMagicInfoList function is:
 %
 %      const MagicInfo **GetMagicInfoList(const char *pattern,
-%        unsigned long *number_aliases)
+%        unsigned long *number_aliases,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -306,6 +309,7 @@ MagickExport const MagicInfo *GetMagicInfo(const unsigned char *magic,
 %    o number_aliases:  This integer returns the number of magics in the
 %      list.
 %
+%    o exception: Return any errors or warnings in this structure.
 %
 */
 
@@ -331,14 +335,11 @@ static int MagicInfoCompare(const void *x,const void *y)
 #endif
 
 MagickExport const MagicInfo **GetMagicInfoList(const char *pattern,
-  unsigned long *number_aliases)
+  unsigned long *number_aliases,ExceptionInfo *exception)
 {
   const MagicInfo
     **aliases;
 
-  ExceptionInfo
-    exception;
-
   register const MagicInfo
     *p;
 
@@ -352,29 +353,28 @@ MagickExport const MagicInfo **GetMagicInfoList(const char *pattern,
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),pattern);
   assert(number_aliases != (unsigned long *) NULL);
   *number_aliases=0;
-  GetExceptionInfo(&exception);
-  p=GetMagicInfo((unsigned char *) "*",0,&exception);
-  DestroyExceptionInfo(&exception);
+  p=GetMagicInfo((unsigned char *) "*",0,exception);
   if (p == (const MagicInfo *) NULL)
     return((const MagicInfo **) NULL);
   aliases=(const MagicInfo **) AcquireMagickMemory((size_t)
-    (GetNumberOfElementsInLinkedList(magic_list)+1)*sizeof(MagicInfo *));
+    (GetNumberOfElementsInLinkedList(magic_list)+1)*sizeof(*aliases));
   if (aliases == (const MagicInfo **) NULL)
     return((const MagicInfo **) NULL);
   /*
     Generate magic list.
   */
-  AcquireSemaphoreInfo(&magic_semaphore);
+  magic_semaphore=AcquireSemaphoreInfo(magic_semaphore);
   ResetLinkedListIterator(magic_list);
   p=(const MagicInfo *) GetNextElementInLinkedList(magic_list);
   for (i=0; p != (const MagicInfo *) NULL; )
   {
-    if ((p->stealth == False) && (GlobExpression(p->name,pattern) != False))
+    if ((p->stealth == MagickFalse) &&
+        (GlobExpression(p->name,pattern) != MagickFalse))
       aliases[i++]=p;
     p=(const MagicInfo *) GetNextElementInLinkedList(magic_list);
   }
-  RelinquishSemaphoreInfo(&magic_semaphore);
-  qsort((void *) aliases,(size_t) i,sizeof(MagicInfo *),MagicInfoCompare);
+  RelinquishSemaphoreInfo(magic_semaphore);
+  qsort((void *) aliases,(size_t) i,sizeof(*aliases),MagicInfoCompare);
   aliases[i]=(MagicInfo *) NULL;
   *number_aliases=(unsigned long) i;
   return(aliases);
@@ -396,7 +396,8 @@ MagickExport const MagicInfo **GetMagicInfoList(const char *pattern,
 %
 %  The format of the GetMagicList function is:
 %
-%      char **GetMagicList(const char *pattern,unsigned long *number_aliases)
+%      char **GetMagicList(const char *pattern,unsigned long *number_aliases,
+%        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -405,6 +406,7 @@ MagickExport const MagicInfo **GetMagicInfoList(const char *pattern,
 %    o number_aliases:  This integer returns the number of image format aliases
 %      in the list.
 %
+%    o exception: Return any errors or warnings in this structure.
 %
 */
 
@@ -428,14 +430,11 @@ static int MagicCompare(const void *x,const void *y)
 #endif
 
 MagickExport char **GetMagicList(const char *pattern,
-  unsigned long *number_aliases)
+  unsigned long *number_aliases,ExceptionInfo *exception)
 {
   char
     **aliases;
 
-  ExceptionInfo
-    exception;
-
   register const MagicInfo
     *p;
 
@@ -449,26 +448,25 @@ MagickExport char **GetMagicList(const char *pattern,
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),pattern);
   assert(number_aliases != (unsigned long *) NULL);
   *number_aliases=0;
-  GetExceptionInfo(&exception);
-  p=GetMagicInfo((unsigned char *) "*",0,&exception);
-  DestroyExceptionInfo(&exception);
+  p=GetMagicInfo((unsigned char *) "*",0,exception);
   if (p == (const MagicInfo *) NULL)
     return((char **) NULL);
   aliases=(char **) AcquireMagickMemory((size_t)
-    (GetNumberOfElementsInLinkedList(magic_list)+1)*sizeof(char *));
+    (GetNumberOfElementsInLinkedList(magic_list)+1)*sizeof(*aliases));
   if (aliases == (char **) NULL)
     return((char **) NULL);
-  AcquireSemaphoreInfo(&magic_semaphore);
+  magic_semaphore=AcquireSemaphoreInfo(magic_semaphore);
   ResetLinkedListIterator(magic_list);
   p=(const MagicInfo *) GetNextElementInLinkedList(magic_list);
   for (i=0; p != (const MagicInfo *) NULL; )
   {
-    if ((p->stealth == False) && (GlobExpression(p->name,pattern) != False))
+    if ((p->stealth == MagickFalse) &&
+        (GlobExpression(p->name,pattern) != MagickFalse))
       aliases[i++]=AcquireString(p->name);
     p=(const MagicInfo *) GetNextElementInLinkedList(magic_list);
   }
-  RelinquishSemaphoreInfo(&magic_semaphore);
-  qsort((void *) aliases,(size_t) i,sizeof(char *),MagicCompare);
+  RelinquishSemaphoreInfo(magic_semaphore);
+  qsort((void *) aliases,(size_t) i,sizeof(*aliases),MagicCompare);
   aliases[i]=(char *) NULL;
   *number_aliases=(unsigned long) i;
   return(aliases);
@@ -498,7 +496,7 @@ MagickExport char **GetMagicList(const char *pattern,
 */
 MagickExport const char *GetMagicName(const MagicInfo *magic_info)
 {
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(magic_info != (MagicInfo *) NULL);
   assert(magic_info->signature == MagickSignature);
   return(magic_info->name);
@@ -519,32 +517,28 @@ MagickExport const char *GetMagicName(const MagicInfo *magic_info)
 %
 %  The format of the InitializeMagicList method is:
 %
-%      unsigned int InitializeMagicList(ExceptionInfo *exception)
+%      MagickBooleanType InitializeMagicList(ExceptionInfo *exception)
 %
 %  A description of each parameter follows.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
 */
-static unsigned int InitializeMagicList(ExceptionInfo *exception)
+static MagickBooleanType InitializeMagicList(ExceptionInfo *exception)
 {
-  unsigned int
-    active;
-
-  active=False;
-  if ((magic_list == (LinkedListInfo *) NULL) && (active_magic == False))
+  if ((magic_list == (LinkedListInfo *) NULL) &&
+      (instantiate_magic == MagickFalse))
     {
-      AcquireSemaphoreInfo(&magic_semaphore);
-      if ((magic_list == (LinkedListInfo *) NULL) && (active_magic == False))
+      magic_semaphore=AcquireSemaphoreInfo(magic_semaphore);
+      if ((magic_list == (LinkedListInfo *) NULL) &&
+          (instantiate_magic == MagickFalse))
         {
-          active_magic=True;
-          active=True;
+          (void) LoadMagicLists(MagicFilename,exception);
+          instantiate_magic=MagickTrue;
         }
-      RelinquishSemaphoreInfo(&magic_semaphore);
-      if (active == True)
-        (void) LoadMagicLists(MagicFilename,exception);
+      RelinquishSemaphoreInfo(magic_semaphore);
     }
-  return(magic_list != (LinkedListInfo *) NULL);
+  return(magic_list != (LinkedListInfo *) NULL ? MagickTrue : MagickFalse);
 }
 
 /*
@@ -562,7 +556,7 @@ static unsigned int InitializeMagicList(ExceptionInfo *exception)
 %
 %  The format of the ListMagicInfo method is:
 %
-%      unsigned int ListMagicInfo(FILE *file,ExceptionInfo *exception)
+%      MagickBooleanType ListMagicInfo(FILE *file,ExceptionInfo *exception)
 %
 %  A description of each parameter follows.
 %
@@ -572,7 +566,8 @@ static unsigned int InitializeMagicList(ExceptionInfo *exception)
 %
 %
 */
-MagickExport unsigned int ListMagicInfo(FILE *file,ExceptionInfo *exception)
+MagickExport MagickBooleanType ListMagicInfo(FILE *file,
+  ExceptionInfo *exception)
 {
   const char
     *path;
@@ -591,14 +586,14 @@ MagickExport unsigned int ListMagicInfo(FILE *file,ExceptionInfo *exception)
 
   if (file == (const FILE *) NULL)
     file=stdout;
-  magic_info=GetMagicInfoList("*",&number_aliases);
+  magic_info=GetMagicInfoList("*",&number_aliases,exception);
   if (magic_info == (const MagicInfo **) NULL)
-    return(False);
+    return(MagickFalse);
   j=0;
   path=(const char *) NULL;
   for (i=0; i < (long) number_aliases; i++)
   {
-    if (magic_info[i]->stealth != False)
+    if (magic_info[i]->stealth != MagickFalse)
       continue;
     if ((path == (const char *) NULL) ||
         (LocaleCompare(path,magic_info[i]->path) != 0))
@@ -620,7 +615,7 @@ MagickExport unsigned int ListMagicInfo(FILE *file,ExceptionInfo *exception)
   }
   (void) fflush(file);
   magic_info=(const MagicInfo **) RelinquishMagickMemory((void *) magic_info);
-  return(True);
+  return(MagickTrue);
 }
 
 /*
@@ -639,7 +634,7 @@ MagickExport unsigned int ListMagicInfo(FILE *file,ExceptionInfo *exception)
 %
 %  The format of the LoadMagicList method is:
 %
-%      unsigned int LoadMagicList(const char *xml,const char *filename,
+%      MagickBooleanType LoadMagicList(const char *xml,const char *filename,
 %        const unsigned long depth,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
@@ -654,7 +649,7 @@ MagickExport unsigned int ListMagicInfo(FILE *file,ExceptionInfo *exception)
 %
 %
 */
-static unsigned int LoadMagicList(const char *xml,const char *filename,
+static MagickBooleanType LoadMagicList(const char *xml,const char *filename,
   const unsigned long depth,ExceptionInfo *exception)
 {
   char
@@ -665,7 +660,7 @@ static unsigned int LoadMagicList(const char *xml,const char *filename,
   MagicInfo
     *magic_info = (MagicInfo *) NULL;
 
-  unsigned int
+  MagickStatusType
     status;
 
   /*
@@ -674,7 +669,7 @@ static unsigned int LoadMagicList(const char *xml,const char *filename,
   (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
     "Loading magic file \"%s\" ...",filename);
   if (xml == (char *) NULL)
-    return(False);
+    return(MagickFalse);
   if (magic_list == (LinkedListInfo *) NULL)
     {
       magic_list=NewLinkedList(0);
@@ -682,10 +677,10 @@ static unsigned int LoadMagicList(const char *xml,const char *filename,
         {
           ThrowFileException(exception,ResourceLimitError,
             "MemoryAllocationFailed",filename);
-          return(False);
+          return(MagickFalse);
         }
     }
-  status=True;
+  status=MagickTrue;
   token=AcquireString((char *) xml);
   for (q=(char *) xml; *q != '\0'; )
   {
@@ -746,11 +741,11 @@ static unsigned int LoadMagicList(const char *xml,const char *filename,
        /*
           Allocate memory for the magic list.
         */
-        magic_info=(MagicInfo *) AcquireMagickMemory(sizeof(MagicInfo));
+        magic_info=(MagicInfo *) AcquireMagickMemory(sizeof(*magic_info));
         if (magic_info == (MagicInfo *) NULL)
           ThrowMagickFatalException(ResourceLimitFatalError,
             "MemoryAllocationFailed",filename);
-        (void) ResetMagickMemory(magic_info,0,sizeof(MagicInfo));
+        (void) ResetMagickMemory(magic_info,0,sizeof(*magic_info));
         magic_info->path=AcquireString(filename);
         magic_info->signature=MagickSignature;
         continue;
@@ -760,7 +755,7 @@ static unsigned int LoadMagicList(const char *xml,const char *filename,
     if (LocaleCompare(keyword,"/>") == 0)
       {
         status=AppendElementToLinkedList(magic_list,magic_info);
-        if (status == False)
+        if (status == MagickFalse)
           (void) ThrowMagickException(exception,GetMagickModule(),
             ResourceLimitError,"MemoryAllocationFailed",magic_info->name);
         magic_info=(MagicInfo *) NULL;
@@ -787,7 +782,7 @@ static unsigned int LoadMagicList(const char *xml,const char *filename,
       {
         if (LocaleCompare((char *) keyword,"offset") == 0)
           {
-            magic_info->offset=(ssize_t) atol(token);
+            magic_info->offset=(MagickOffsetType) atol(token);
             break;
           }
         break;
@@ -797,7 +792,7 @@ static unsigned int LoadMagicList(const char *xml,const char *filename,
       {
         if (LocaleCompare((char *) keyword,"stealth") == 0)
           {
-            magic_info->stealth=(unsigned int)
+            magic_info->stealth=(MagickBooleanType)
               (LocaleCompare(token,"True") == 0);
             break;
           }
@@ -822,7 +817,7 @@ static unsigned int LoadMagicList(const char *xml,const char *filename,
               if (*p == '\\')
                 {
                   p++;
-                  if (isdigit((int) ((unsigned char) *p)) != False)
+                  if (isdigit((int) ((unsigned char) *p)) != 0)
                     {
                       char
                         *end;
@@ -862,8 +857,8 @@ static unsigned int LoadMagicList(const char *xml,const char *filename,
   }
   token=(char *) RelinquishMagickMemory(token);
   if (magic_list == (LinkedListInfo *) NULL)
-    return(False);
-  return(status);
+    return(MagickFalse);
+  return(status != 0 ? MagickTrue : MagickFalse);
 }
 
 /*
@@ -882,7 +877,7 @@ static unsigned int LoadMagicList(const char *xml,const char *filename,
 %
 %  The format of the LoadMagicLists method is:
 %
-%      unsigned int LoadMagicLists(const char *filename,
+%      MagickBooleanType LoadMagicLists(const char *filename,
 %        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
@@ -893,29 +888,39 @@ static unsigned int LoadMagicList(const char *xml,const char *filename,
 %
 %
 */
-static unsigned int LoadMagicLists(const char *filename,
+static MagickBooleanType LoadMagicLists(const char *filename,
   ExceptionInfo *exception)
 {
+  char
+    path[MaxTextExtent];
+
   const StringInfo
     *option;
 
   LinkedListInfo
     *options;
 
-  unsigned int
+  MagickStatusType
     status;
 
-  status=False;
+  status=MagickFalse;
+  *path='\0';
   options=GetConfigureOptions(filename,exception);
   option=(const StringInfo *) GetNextElementInLinkedList(options);
   while (option != (const StringInfo *) NULL)
   {
+    (void) CopyMagickString(path,option->path,MaxTextExtent);
     status|=LoadMagicList((const char *) option->datum,option->path,0,
       exception);
     option=(const StringInfo *) GetNextElementInLinkedList(options);
   }
-  DestroyConfigureOptions(options);
-  if (status == False)
-    status=LoadMagicList(MagicMap,"built-in",0,exception);
-  return(status);
+  options=DestroyConfigureOptions(options);
+  if ((magic_list == (LinkedListInfo *) NULL) || 
+      (IsLinkedListEmpty(magic_list) != MagickFalse))
+    {
+      (void) ThrowMagickException(exception,GetMagickModule(),ConfigureError,
+			  "UnableToOpenConfigureFile",path);
+      status|=LoadMagicList(MagicMap,"built-in",0,exception);
+    }
+  return(status != 0 ? MagickTrue : MagickFalse);
 }
