@@ -17,7 +17,7 @@
 %                                 March 2000                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -46,7 +46,8 @@
 #include "magick/compare.h"
 #include "magick/draw.h"
 #include "magick/effect.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/fx.h"
 #include "magick/hashmap.h"
 #include "magick/memory_.h"
@@ -114,6 +115,187 @@ static const OptionInfo
     { "UserSpaceOnUse", (long) UserSpaceOnUse },
     { (char *) NULL, (long) UndefinedPathUnits }
   },
+  CommandOptions[] =
+  {
+    { "-adjoin", 0L },
+    { "-affine", 1L },
+    { "-annotate", 2L },
+    { "-antialias", 0L },
+    { "-append", 0L },
+    { "-authenticate", 1L },
+    { "-average", 0L },
+    { "-background", 1L },
+    { "-bias", 1L },
+    { "-black-threshold", 1L },
+    { "-blend", 1L },
+    { "-blue-primary", 1L },
+    { "-blur", 1L },
+    { "-border", 1L },
+    { "-bordercolor", 1L },
+    { "-box", 1L },
+    { "-cache", 1L },
+    { "-channel", 1L },
+    { "-charcoal", 1L },
+    { "-chop", 1L },
+    { "-clip-path", 1L },
+    { "-clone", 1L },
+    { "-coalesce", 0L },
+    { "-colorize", 1L },
+    { "-colors", 1L },
+    { "-colorspace", 1L },
+    { "-combine", 0L },
+    { "-comment", 1L },
+    { "-compose", 1L },
+    { "-composite", 0L },
+    { "-compress", 1L },
+    { "-contrast", 0L },
+    { "-convolve", 1L },
+    { "-crop", 1L },
+    { "-cycle", 1L },
+    { "-deconstruct", 0L },
+    { "-debug", 1L },
+    { "-define", 1L },
+    { "+define", 1L },
+    { "-delay", 1L },
+    { "-delete", 1L },
+    { "-density", 1L },
+    { "-depth", 1L },
+    { "-despeckle", 0L },
+    { "-display", 1L },
+    { "-dispose", 1L },
+    { "-dither", 0L },
+    { "-draw", 1L },
+    { "-edge", 1L },
+    { "-emboss", 1L },
+    { "-encoding", 1L },
+    { "-endian", 1L },
+    { "-enhance", 0L },
+    { "-equalize", 0L },
+    { "-evaluate", 2L },
+    { "-extract", 1L },
+    { "-family", 1L },
+    { "-fill", 1L },
+    { "-filter", 1L },
+    { "-flatten", 0L },
+    { "-flip", 0L },
+    { "-flop", 0L },
+    { "-floodfill", 2L },
+    { "-family", 1L },
+    { "-font", 1L },
+    { "-format", 1L },
+    { "-frame", 1L },
+    { "-fuzz", 1L },
+    { "-fx", 1L },
+    { "-gamma", 1L },
+    { "-gaussian", 1L },
+    { "-geometry", 1L },
+    { "-green-primary", 1L },
+    { "-gravity", 1L },
+    { "-help", 0L },
+    { "-implode", 1L },
+    { "-insert", 1L },
+    { "-intent", 1L },
+    { "-interlace", 1L },
+    { "-label", 1L },
+    { "-lat", 1L },
+    { "-level", 1L },
+    { "-limit", 2L },
+    { "-linewidth", 1L },
+    { "-list", 1L },
+    { "-log", 1L },
+    { "-loop", 1L },
+    { "-map", 1L },
+    { "-mask", 1L },
+    { "-matte", 0L },
+    { "-mattecolor", 1L },
+    { "-median", 1L },
+    { "-mode", 1L },
+    { "-modulate", 1L },
+    { "-monochrome", 0L },
+    { "-morph", 1L },
+    { "-mosaic", 0L },
+    { "-negate", 0L },
+    { "-normalize", 0L },
+    { "-noise", 1L },
+    { "+noise", 1L },
+    { "-opaque", 1L },
+    { "-ordered-dither", 1L },
+    { "-orient", 1L },
+    { "-page", 1L },
+    { "-paint", 1L },
+    { "-pen", 1L },
+    { "-pointsize", 1L },
+    { "-posterize", 1L },
+    { "-preview", 1L },
+    { "-profile", 1L },
+    { "+profile", 1L },
+    { "-process", 2L },
+    { "-quality", 1L },
+    { "-radial-blur", 1L },
+    { "-raise", 1L },
+    { "-random-threshold", 1L },
+    { "-region", 1L },
+    { "-raise", 1L },
+    { "-red-primary", 1L },
+    { "-render", 0L },
+    { "-repage", 1L },
+    { "-resample", 1L },
+    { "-resize", 1L },
+    { "-roll", 1L },
+    { "-rotate", 1L },
+    { "-sample", 1L },
+    { "-sampling-factor", 1L },
+    { "-scale", 1L },
+    { "-scene", 1L },
+    { "-seed", 1L },
+    { "-segment", 1L },
+    { "-separate", 0L },
+    { "-shade", 1L },
+    { "-shadow", 1L },
+    { "-sharpen", 1L },
+    { "-shave", 1L },
+    { "-shear", 1L },
+    { "-size", 1L },
+    { "-snaps", 1L },
+    { "-solarize", 1L },
+    { "-splice", 1L },
+    { "-spread", 1L },
+    { "-stegano", 1L },
+    { "-stroke", 1L },
+    { "-strokewidth", 1L },
+    { "-stretch", 1L },
+    { "-strip", 0L },
+    { "-style", 1L },
+    { "-support", 1L },
+    { "-swap", 1L },
+    { "-swirl", 1L },
+    { "-texture", 1L },
+    { "-threshold", 1L },
+    { "-thumbnail", 1L },
+    { "-tile", 1L },
+    { "-tint", 1L },
+    { "-transparent", 1L },
+    { "-treedepth", 1L },
+    { "-trim", 0L },
+    { "-type", 1L },
+    { "-undercolor", 1L },
+    { "-units", 1L },
+    { "-unsharp", 1L },
+    { "-update", 1L },
+    { "-verbose", 0L },
+    { "-version", 0L },
+    { "-view", 1L },
+    { "-virtual-pixel", 1L },
+    { "-watermark", 1L },
+    { "-wave", 1L },
+    { "-weight", 1L },
+    { "-window", 1L },
+    { "-window_group", 1L },
+    { "-white-point", 1L },
+    { "-white-threshold", 1L },
+    { "-write", 1L },
+    { (char *) NULL, (long) 0L }
+  },
   CompositeOptions[] =
   {
     { "Undefined", (long) UndefinedCompositeOp },
@@ -158,7 +340,7 @@ static const OptionInfo
     { "Overlay", (long) OverlayCompositeOp },
     { "Over", (long) OverCompositeOp },
     { "Plus", (long) PlusCompositeOp },
-    { "ReplaceCompositeOp", (long) ReplaceCompositeOp },
+    { "Replace", (long) ReplaceCompositeOp },
     { "Saturate", (long) SaturateCompositeOp },
     { "Screen", (long) ScreenCompositeOp },
     { "SoftLight", (long) SoftLightCompositeOp },
@@ -181,6 +363,7 @@ static const OptionInfo
     { "Group4", (long) Group4Compression },
     { "JPEG", (long) JPEGCompression },
     { "Lossless", (long) LosslessJPEGCompression },
+    { "LosslessJPEG", (long) LosslessJPEGCompression },
     { "LZW", (long) LZWCompression },
     { "None", (long) NoCompression },
     { "RLE", (long) RLECompression },
@@ -204,7 +387,7 @@ static const OptionInfo
     { "XYZ", (long) XYZColorspace },
     { "YCbCr", (long) YCbCrColorspace },
     { "YCC", (long) YCCColorspace },
-    { "YIZ", (long) YIQColorspace },
+    { "YIQ", (long) YIQColorspace },
     { "YPbPr", (long) YPbPrColorspace },
     { "YUV", (long) YUVColorspace },
     { (char *) NULL, (long) UndefinedColorspace }
@@ -347,6 +530,21 @@ static const OptionInfo
     { "Round", (long) RoundJoin },
     { (char *) NULL, (long) UndefinedJoin }
   },
+  ListOptions[] =
+  {
+    { "Coder", 1L },
+    { "Color", 1L },
+    { "Configure", 1L },
+    { "Delegate", 1L },
+    { "Format", 1L },
+    { "Locale", 1L },
+    { "Log", 1L },
+    { "Magic", 1L },
+    { "Module", 1L },
+    { "Resource", 1L },
+    { "Type", 1L },
+    { (char *) NULL, 0L }
+  },
   LogEventOptions[] =
   {
     { "Undefined", (long) UndefinedEvents },
@@ -366,6 +564,7 @@ static const OptionInfo
     { "Trace", (long) TraceEvent },
     { "Transform", (long) TransformEvent },
     { "User", (long) UserEvent },
+    { "Wand", (long) WandEvent },
     { "X11", (long) X11Event },
     { (char *) NULL, (long) UndefinedEvents }
   },
@@ -399,23 +598,24 @@ static const OptionInfo
   },
   MogrifyOptions[] =
   {
-    { "Append", True },
-    { "Average", True },
-    { "Coalesce", True },
-    { "Combine", True },
-    { "Compose", True },
-    { "Deconstruct", True },
-    { "Delete", True },
-    { "Flatten", True },
-    { "Fx", True },
-    { "Insert", True },
-    { "Map", True },
-    { "Morph", True },
-    { "Mosaic", True },
-    { "Process", True },
-    { "Swap", True },
-    { "Write", True },
-    { (char *) NULL, False }
+    { "append", MagickTrue },
+    { "average", MagickTrue },
+    { "crop", MagickTrue },
+    { "coalesce", MagickTrue },
+    { "combine", MagickTrue },
+    { "composite", MagickTrue },
+    { "deconstruct", MagickTrue },
+    { "delete", MagickTrue },
+    { "flatten", MagickTrue },
+    { "fx", MagickTrue },
+    { "insert", MagickTrue },
+    { "map", MagickTrue },
+    { "morph", MagickTrue },
+    { "mosaic", MagickTrue },
+    { "process", MagickTrue },
+    { "swap", MagickTrue },
+    { "write", MagickTrue },
+    { (char *) NULL, MagickFalse }
   },
   NoiseOptions[] =
   {
@@ -560,7 +760,7 @@ static const OptionInfo
 %
 %  The format of the CloneImageOptions method is:
 %
-%      unsigned int CloneImageOptions(ImageInfo *image_info,
+%      MagickBooleanType CloneImageOptions(ImageInfo *image_info,
 %        const ImageInfo *clone_info)
 %
 %  A description of each parameter follows:
@@ -571,7 +771,7 @@ static const OptionInfo
 %
 %
 */
-MagickExport unsigned int CloneImageOptions(ImageInfo *image_info,
+MagickExport MagickBooleanType CloneImageOptions(ImageInfo *image_info,
   const ImageInfo *clone_info)
 {
   const char
@@ -580,12 +780,12 @@ MagickExport unsigned int CloneImageOptions(ImageInfo *image_info,
 
   assert(image_info != (ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
-  if (image_info->debug != False)
+  if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(clone_info != (const ImageInfo *) NULL);
   assert(clone_info->signature == MagickSignature);
   if (clone_info->options == (HashmapInfo *) NULL)
-    return(True);
+    return(MagickTrue);
   ResetImageOptionIterator(clone_info);
   for (key=GetNextImageOption(clone_info); key != (char *) NULL; )
   {
@@ -594,7 +794,7 @@ MagickExport unsigned int CloneImageOptions(ImageInfo *image_info,
       (void) SetImageOption(image_info,key,value);
     key=GetNextImageOption(clone_info);
   }
-  return(True);
+  return(MagickTrue);
 }
 
 /*
@@ -612,7 +812,8 @@ MagickExport unsigned int CloneImageOptions(ImageInfo *image_info,
 %
 %  The format of the DefineImageOption method is:
 %
-%      unsigned int DefineImageOption(ImageInfo *image_info,const char *option)
+%      MagickBooleanType DefineImageOption(ImageInfo *image_info,
+%        const char *option)
 %
 %  A description of each parameter follows:
 %
@@ -622,19 +823,19 @@ MagickExport unsigned int CloneImageOptions(ImageInfo *image_info,
 %
 %
 */
-MagickExport unsigned int DefineImageOption(ImageInfo *image_info,
+MagickExport MagickBooleanType DefineImageOption(ImageInfo *image_info,
   const char *option)
 {
   char
     key[MaxTextExtent],
     value[MaxTextExtent];
 
+  MagickBooleanType
+    status;
+
   register char
     *p;
 
-  unsigned int
-    status;
-
   assert(image_info != (ImageInfo *) NULL);
   assert(option != (const char *) NULL);
   (void) CopyMagickString(key,option,MaxTextExtent);
@@ -673,7 +874,7 @@ MagickExport unsigned int DefineImageOption(ImageInfo *image_info,
 %    o option: The image option.
 %
 */
-MagickExport unsigned int DeleteImageOption(ImageInfo *image_info,
+MagickExport MagickBooleanType DeleteImageOption(ImageInfo *image_info,
   const char *option)
 {
   char
@@ -682,17 +883,17 @@ MagickExport unsigned int DeleteImageOption(ImageInfo *image_info,
 
   assert(image_info != (ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
-  if (image_info->debug != False)
+  if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   if (image_info->options == (HashmapInfo *) NULL)
-    return(False);
+    return(MagickFalse);
   (void) CopyMagickString(key,option,MaxTextExtent);
   LocaleLower(key);
   value=(char *) RemoveEntryFromHashmap(image_info->options,key);
   if (value == (char *) NULL)
-    return(False);
+    return(MagickFalse);
   value=(char *) RelinquishMagickMemory(value);
-  return(True);
+  return(MagickTrue);
 }
 
 /*
@@ -721,8 +922,7 @@ MagickExport unsigned int DeleteImageOption(ImageInfo *image_info,
 MagickExport void DestroyImageOptions(ImageInfo *image_info)
 {
   if (image_info->options != (HashmapInfo *) NULL)
-    DestroyHashmap(image_info->options);
-  image_info->options=(HashmapInfo *) NULL;
+    image_info->options=DestroyHashmap(image_info->options);
 }
 
 /*
@@ -758,7 +958,7 @@ MagickExport const char *GetImageOption(const ImageInfo *image_info,
 
   assert(image_info != (ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
-  if (image_info->debug != False)
+  if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   if (image_info->options == (HashmapInfo *) NULL)
     return((const char *) NULL);
@@ -801,6 +1001,7 @@ static const OptionInfo *GetOptionInfo(const MagickOption value)
     case MagickClassOptions: return(ClassOptions);
     case MagickClipPathOptions: return(ClipPathOptions);
     case MagickColorspaceOptions: return(ColorspaceOptions);
+    case MagickCommandOptions: return(CommandOptions);
     case MagickCompositeOptions: return(CompositeOptions);
     case MagickCompressionOptions: return(CompressionOptions);
     case MagickDecorationOptions: return(DecorationOptions);
@@ -815,6 +1016,7 @@ static const OptionInfo *GetOptionInfo(const MagickOption value)
     case MagickInterlaceOptions: return(InterlaceOptions);
     case MagickLineCapOptions: return(LineCapOptions);
     case MagickLineJoinOptions: return(LineJoinOptions);
+    case MagickListOptions: return(ListOptions);
     case MagickLogEventOptions: return(LogEventOptions);
     case MagickMetricOptions: return(MetricOptions);
     case MagickMethodOptions: return(MethodOptions);
@@ -840,21 +1042,21 @@ MagickExport char **GetMagickOptions(const MagickOption value)
     **values;
 
   const OptionInfo
-    *value_info;
+    *option_info;
 
   register long
     i;
 
-  value_info=GetOptionInfo(value);
-  if (value_info == (const OptionInfo *) NULL)
+  option_info=GetOptionInfo(value);
+  if (option_info == (const OptionInfo *) NULL)
     return((char **) NULL);
-  for (i=0; value_info[i].mnemonic != (const char *) NULL; i++);
-  values=(char **) AcquireMagickMemory((size_t) (i+1)*sizeof(char *));
+  for (i=0; option_info[i].mnemonic != (const char *) NULL; i++);
+  values=(char **) AcquireMagickMemory((size_t) (i+1)*sizeof(*values));
   if (values == (char **) NULL)
     ThrowMagickFatalException(ResourceLimitFatalError,"UnableToAcquireString",
-      value_info[0].mnemonic);
-  for (i=0; value_info[i].mnemonic != (const char *) NULL; i++)
-    values[i]=AcquireString(value_info[i].mnemonic);
+      option_info[0].mnemonic);
+  for (i=0; option_info[i].mnemonic != (const char *) NULL; i++)
+    values[i]=AcquireString(option_info[i].mnemonic);
   values[i]=(char *) NULL;
   return(values);
 }
@@ -885,7 +1087,7 @@ MagickExport char *GetNextImageOption(const ImageInfo *image_info)
 {
   assert(image_info != (ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
-  if (image_info->debug != False)
+  if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   if (image_info->options == (HashmapInfo *) NULL)
     return((char *) NULL);
@@ -922,18 +1124,18 @@ MagickExport const char *MagickOptionToMnemonic(const MagickOption value,
   const long type)
 {
   const OptionInfo
-    *value_info;
+    *option_info;
 
   register long
     i;
 
-  value_info=GetOptionInfo(value);
-  if (value_info == (const OptionInfo *) NULL)
+  option_info=GetOptionInfo(value);
+  if (option_info == (const OptionInfo *) NULL)
     return((const char *) NULL);
-  for (i=0; value_info[i].mnemonic != (const char *) NULL; i++)
-    if (type == value_info[i].type)
+  for (i=0; option_info[i].mnemonic != (const char *) NULL; i++)
+    if (type == option_info[i].type)
       break;
-  return(value_info[i].mnemonic);
+  return(option_info[i].mnemonic);
 }
 
 /*
@@ -956,7 +1158,7 @@ MagickExport const char *MagickOptionToMnemonic(const MagickOption value,
 %
 %  A description of each parameter follows:
 %
-%    o value_list: One or more values separated by commas.
+%    o options: One or more values separated by commas.
 %
 %
 */
@@ -968,7 +1170,7 @@ MagickExport long ParseChannelOption(const char *channels)
   register long
     i;
 
-  channel=ParseMagickOption(MagickChannelOptions,True,channels);
+  channel=ParseMagickOption(MagickChannelOptions,MagickTrue,channels);
   if (channel >= 0)
     return(channel);
   channel=0;
@@ -1054,36 +1256,39 @@ MagickExport long ParseChannelOption(const char *channels)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  ParseMagickOption() parses a string and returns an enumerated value type(s).
+%  ParseMagickOption() parses a string and returns an enumerated option type(s).
 %
 %  The format of the ParseMagickOption method is:
 %
-%      long ParseMagickOption(const MagickOption value,
-%        const unsigned int list,const char *value_list)
+%      long ParseMagickOption(const MagickOption option,
+%        const MagickBooleanType list,const char *options)
 %
 %  A description of each parameter follows:
 %
-%    o value: The value.
+%    o option: The option.
 %
-%    o list: A value other than zero permits more than one value separated by
+%    o list: A option other than zero permits more than one option separated by
 %      commas.
 %
-%    o value_list: One or more values separated by commas.
+%    o options: One or more options separated by commas.
 %
 %
 */
-MagickExport long ParseMagickOption(const MagickOption value,
-  const unsigned int list,const char *value_list)
+MagickExport long ParseMagickOption(const MagickOption option,
+  const MagickBooleanType list,const char *options)
 {
   char
     *q,
     token[MaxTextExtent];
 
   const OptionInfo
-    *value_info;
+    *option_info;
 
   long
-    value_types;
+    option_types;
+
+  MagickBooleanType
+    negate;
 
   register char
     *p;
@@ -1091,18 +1296,17 @@ MagickExport long ParseMagickOption(const MagickOption value,
   register long
     i;
 
-  unsigned int
-    negate;
-
-  value_info=GetOptionInfo(value);
-  if (value_info == (const OptionInfo *) NULL)
+  option_info=GetOptionInfo(option);
+  if (option_info == (const OptionInfo *) NULL)
     return(-1);
-  value_types=0;
-  for (q=(char *) value_list; *q != '\0'; )
+  option_types=0;
+  for (q=(char *) options; *q != '\0'; )
   {
     GetToken(q,&q,token);
-    negate=(*token == '!');
-    if (negate != False)
+    if ((*token == '-') || (*token == '+'))
+      GetToken(q,&q,token+1);
+    negate=(*token == '!') ? MagickTrue : MagickFalse;
+    if (negate != MagickFalse)
       GetToken(q,&q,token);
     p=strchr(token+1,'-');
     if (p != (char *) NULL)
@@ -1110,24 +1314,24 @@ MagickExport long ParseMagickOption(const MagickOption value,
     p=strchr(token+1,'_');
     if (p != (char *) NULL)
       (void) CopyMagickString(p,p+1,MaxTextExtent);
-    for (i=0; value_info[i].mnemonic != (char *) NULL; i++)
-      if (LocaleCompare(token,value_info[i].mnemonic) == 0)
+    for (i=0; option_info[i].mnemonic != (char *) NULL; i++)
+      if (LocaleCompare(token,option_info[i].mnemonic) == 0)
         {
           if (*token == '!')
-            value_types=value_types &~ value_info[i].type;
+            option_types=option_types &~ option_info[i].type;
           else
-            value_types=value_types | value_info[i].type;
+            option_types=option_types | option_info[i].type;
           break;
         }
-    if (value_info[i].mnemonic == (char *) NULL)
+    if (option_info[i].mnemonic == (char *) NULL)
       return(-1);
-    if (list == False)
+    if (list == MagickFalse)
       break;
     GetToken(q,&q,token);
     if (*token != ',')
       break;
   }
-  return(value_types);
+  return(option_types);
 }
 
 /*
@@ -1161,7 +1365,7 @@ MagickExport char *RemoveImageOption(ImageInfo *image_info,const char *option)
 
   assert(image_info != (ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
-  if (image_info->debug != False)
+  if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   if (image_info->options == (HashmapInfo *) NULL)
     return((char *) NULL);
@@ -1198,7 +1402,7 @@ MagickExport void ResetImageOptionIterator(const ImageInfo *image_info)
 {
   assert(image_info != (ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
-  if (image_info->debug != False)
+  if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   if (image_info->options == (HashmapInfo *) NULL)
     return;
@@ -1220,8 +1424,8 @@ MagickExport void ResetImageOptionIterator(const ImageInfo *image_info)
 %
 %  The format of the SetImageOption method is:
 %
-%      unsigned int SetImageOption(ImageInfo *image_info,const char *option,
-%        const char *value)
+%      MagickBooleanType SetImageOption(ImageInfo *image_info,
+%        const char *option,const char *value)
 %
 %  A description of each parameter follows:
 %
@@ -1232,18 +1436,18 @@ MagickExport void ResetImageOptionIterator(const ImageInfo *image_info)
 %    o values: The image option values.
 %
 */
-MagickExport unsigned int SetImageOption(ImageInfo *image_info,
+MagickExport MagickBooleanType SetImageOption(ImageInfo *image_info,
   const char *option,const char *value)
 {
   char
     key[MaxTextExtent];
 
-  unsigned int
+  MagickBooleanType
     status;
 
   assert(image_info != (ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
-  if (image_info->debug != False)
+  if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   if (image_info->options == (HashmapInfo *) NULL)
     image_info->options=NewHashmap(SmallHashmapSize,HashStringType,
